package org.ow2.choreos.iots.common;

import java.util.Hashtable;
import java.util.Map;

public class Location {
	static Map<String, Double> locationAndAreaMap = new Hashtable<String, Double>();
	static Map<String, BorderPoints> locationAndAreaCoords = new Hashtable<String, BorderPoints>();
	static {
		locationAndAreaMap.put("Paris", 1771500.0);
		locationAndAreaMap.put("Beijing", 17000.0);
		locationAndAreaMap.put("Airport", 10000.0);
		locationAndAreaMap.put("Louvre", 60.6);
		
	}

	static {
		locationAndAreaCoords.put("Beijing", new BorderPoints(new Point(
				5700, 4400), new Point(5800, 4400), new Point(
				5700, 4500), new Point(5800, 4500)));
		locationAndAreaCoords.put("Paris", new BorderPoints(new Point(
				2411.7, 250.2), new Point(2415.6, 250.2), new Point(
				2411.7, 267.9), new Point(2415.6, 267.9)));
		locationAndAreaCoords.put("Louvre", new BorderPoints(new Point(
				2413.19, 259.06), new Point(2413.68, 259.06), new Point(
				2413.19, 260.20), new Point(2413.68, 260.20)));
		
	}
	double myArea;
	String myName;
	double xCoordinate;
	double yCoordinate;
	long myTime;
	BorderPoints myBorderPoints;
	String myLocationType;
Point bottomLeft, upperRight; 
	/**
	 * The location can be defined based on a regions name only, like Paris
	 * 
	 * @param name
	 */
	public Location(String name) {
		myName = name;
		myLocationType = "area";

	}

	
	/**
	 * The location can be a point of interest
	 * 
	 * @param name
	 * @param xCoord
	 *            : the x coordinate of the point of interest
	 * @param yCoord
	 *            : the y coordinate of the point in interest
	 */
	public Location(String name, double xCoord, double yCoord) {
		myName = name;
		xCoordinate = xCoord;
		yCoordinate = yCoord;
		myLocationType = "point";
	}
	
	/**
	 * The location can be a point of interest
	 * 
	 * @param name
	 * @param xCoord
	 *            : the x coordinate of the point of interest
	 * @param yCoord
	 *            : the y coordinate of the point in interest
	 */
	public Location(String name, Point bottomLeft, Point upperRight) {
		this.myName = name;
		this.bottomLeft = bottomLeft;
		this.upperRight = upperRight;
		this.myLocationType = "area";
	}
	
	public Point getBottomLeft(){
		return this.bottomLeft;
	}

	public Point getUpperRight(){
		return this.upperRight;
	}
	/**
	 * The location can be defined based on a BorderPoints object which allows
	 * us to specify the x, y coordinates of the bottom left, right and upper
	 * left, right points of the area
	 * 
	 * @param name
	 * @param areaPoints
	 */
	public Location(String name, BorderPoints areaPoints) {
		myName = name;
		myBorderPoints = areaPoints;
		myLocationType = "area";

	}

	public Location(String name, double area) {
		myName = name;
		myArea = area;
	}

	public String getLocationType() {
		return myLocationType;
	}

	/**
	 * 
	 * @param x
	 * @param y
	 * @param totalDispTime
	 *            this is a time long, not a period
	 */
	public Location(double x, double y, long totalDispTime) {
		xCoordinate = x;
		yCoordinate = y;
		myTime = totalDispTime;
	}

	public long getTime() {
		return myTime;
	}

	public double getArea() {
		// TODO get AREA from ontology
		myArea = locationAndAreaMap.get(myName);
		return myArea;
	}

	public void setArea(double value) {
		myArea = value;
	}

	public String getLocationName() {
		return myName;
	}

	public Double getXCoordinate() {
		return xCoordinate;
	}

	public Double getYCoordinate() {
		return yCoordinate;
	}

	public void setXYCoordinates(double x, double y) {
		xCoordinate = x;
		yCoordinate = y;
	}

	public BorderPoints getBorderPoints() {
		return myBorderPoints;
	}
	
	public BorderPoints getBorderPoints(String location) {
		return locationAndAreaCoords.get(location);
	}

}
