/**
 * @author Kostas Lekkas -- lekkas@vtrip.net
 */
package net.vtrip.things;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.net.URLConnection;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.ws.rs.ApplicationPath;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.HeaderParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.ResponseBuilder;
import javax.ws.rs.core.Response.Status;
import javax.ws.rs.core.UriInfo;

import org.codehaus.jettison.json.JSONException;
import org.codehaus.jettison.json.JSONObject;
import org.java_websocket.server.WebSocketServer;

import fr.inria.arles.websockets.WebSocket_Server;

/**
 * REST Web Service
 * 
 * @author Lekkas
 */

@Path("/")
public class RESTAPI {

	private WebSocket_Server _WS_Server = null;

	public RESTAPI() {
		_WS_Server = WebSocket_Server.getInstance();
	}

	@GET
	@Path("/statusws")
	@Produces("text/plain")
	public String Status_WebSocket() {
		if (_WS_Server != null)
			return _WS_Server.Get_Server_Status() + "";
		else
			return "0";
	}

	@GET
	@Path("/stopws")
	@Produces("text/plain")
	public String Stop_WebSocket() {
		if (_WS_Server != null) {
			_WS_Server.WebSocket_Server_Stop();
			return _WS_Server.Get_Server_Status() + "";
		} else
			return "0";
	}

	@GET
	@Path("/startws")
	@Produces("text/plain")
	public String Start_WebSocket() {
		if (_WS_Server != null) {
			_WS_Server.WebSocket_Server_Start();
			return _WS_Server.Get_Server_Status() + "";
		} else
			return "0";
	}

	@GET
	@Produces("application/json")
	public String getErr1() {
		throwErrorMsg("Wrong request format");
		return null;
	}

	@GET
	@Path("/pendingrequests/{device_id}")
	@Produces("application/json")
	public String getPendingRequests(@PathParam("device_id") String deviceID) {
		String pending = Resources.getPendingRequests(deviceID);
		System.out.println("Asking PENDING requests for " + deviceID + " : "
				+ pending);
		return pending;

	}

	@POST
	@Path("/sendresponse/{device_id}/{service_name}")
	@Consumes("application/json")
	public Response SendResponse(@PathParam("device_id") String deviceID,
			@PathParam("service_name") String serviceName, String msg) {
		System.out
				.println("Got RESPONSE for " + deviceID + " : " + serviceName);
		Resources.newSensorDataReply(deviceID, serviceName, msg);
		return Response.ok().build();
	}

	@GET
	@Path("/{device_id}/{service_name}")
	@Produces("application/json")
	public Response getData(@PathParam("device_id") String deviceID,
			@PathParam("service_name") String serviceName) {
		System.out.println("Got sensor REQUEST for " + deviceID + " : "
				+ serviceName);

		String reply = Resources.getSensorData(deviceID, serviceName);

//		if (reply.length() == 0) // reply size == 0 ; No reply returned in time.
//									// Return GATEWAY_TIME_OUT
//		{
//			return Response.status(504).header("Access-Control-Allow-Origin", "*").header("Access-Control-Max-Age", "1800").build();
//		} else
			return Response.ok().header("Access-Control-Allow-Origin", "*").header("Access-Control-Max-Age", "1800")
					.entity(reply).build();
	}

	@POST
	@Path("/{device_id}/{service_name}")
	public Response pushData(@PathParam("device_id") String deviceID,
			@PathParam("service_name") String serviceName, String msg) {
		
		if (!serviceName.equals("pushmessage"))
			return Response.ok().header("Access-Control-Allow-Origin", "*").build();
		
		System.out.println("Got PUSH REQUEST for " + deviceID + " : "
				+ serviceName);

		Resources.pushDataToDevice(deviceID, serviceName, msg);
		return Response.ok().header("Access-Control-Allow-Origin", "*").build();
	}

	private void throwErrorMsg(String Msg) {
		ResponseBuilder builder = Response.status(Status.BAD_REQUEST);
		builder.type("text/json");
		String error = "";
		JSONObject obj = new JSONObject();
		try {
			obj.put("error", Msg);
			error = obj.toString();
		} catch (JSONException ex) {
			System.out.println("Error: " + ex.toString());
		}

		builder.entity(error);
		throw new WebApplicationException(builder.build());
	}

}