package test.java;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.Executor;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.http.client.HttpClient;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.PoolingClientConnectionManager;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import test.java.interfaces.MainControl;
import test.java.services.LocationService;
import test.java.services.NoiseLevelService;

/**
 * Implementation of the MainControl interface.
 * TODO: Make this object singleton.
 * 
 * @author lekkas@vtrip.net
 *
 */
public class MainControlImpl implements MainControl {
	
	private String proxyAddr;
	
	public static ArrayList<VirtualDevice> devs;
	private static int NTHREADS = 1;
	private static int POLLDELAY = 5000;
	
	private ScheduledThreadPoolExecutor executor;
	
	public MainControlImpl(String proxyAddr) {
		this.proxyAddr = proxyAddr;
	}
	
	public String getProxyAddr() {
		return proxyAddr;
	}
	
	@Override
	public void startService(int numOfDevices) {
		
		executor = new ScheduledThreadPoolExecutor(NTHREADS);
		
		/*
		 * Step 1: Create list of virtual devices;
		 */
		createVirtualDevices(numOfDevices);
		
		/*
		 * Step 2: Start scheduled tasks that checks for pending 
		 * requests and serve them (if any).
		 */
		
		PoolingClientConnectionManager cm = new PoolingClientConnectionManager();
		cm.setMaxTotal(50);
		HttpClient httpclient = new DefaultHttpClient(cm);

      	
		Random rand = new Random(System.currentTimeMillis());
		
		for(int i = 0; i < devs.size(); i++) {
			String deviceID = devs.get(i).getID();
			long initial_delay = (long)(rand.nextFloat()*2000);
			System.out.println("scheduling "+deviceID+" with initial delay = "+initial_delay);
			String getURL = proxyAddr+"/pendingrequests/"+deviceID;
			System.out.println("GET :"+getURL);
			HttpGet httpget = new HttpGet(getURL);
			executor.scheduleWithFixedDelay(new PollAndServeTask(httpclient, httpget, deviceID, 
					i, this.proxyAddr), 
					initial_delay, 
					POLLDELAY, TimeUnit.MILLISECONDS);
		}
	}

	@Override
	public void stopService() {
		executor.shutdown();
	}

	private void createVirtualDevices(int numOfDevices) {
		devs = new ArrayList<VirtualDevice>();
		
		for(int i = 0; i < numOfDevices; i++) {
			VirtualDevice d = new VirtualDevice();
			d.addService(new NoiseLevelService(d.getID()));
			d.addService(new LocationService(d.getID()));
			devs.add(d);
			System.out.println(d.getID());
		}
	}
	
	
	
}
