package test.java;

import java.io.IOException;
import java.util.ArrayList;

import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.protocol.BasicHttpContext;
import org.apache.http.protocol.HttpContext;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

public class PollAndServeTask implements Runnable {

	private final HttpClient httpClient;
    private final HttpContext context;
    private final HttpGet httpget;
    private String deviceID;
    private int devIndex;
    private String proxyAddr;

    public PollAndServeTask(HttpClient httpClient, HttpGet httpget, String deviceID, 
    		int devIndex, String proxyAddr) {
		this.httpClient = httpClient;
		this.httpget = httpget;
		this.context = new BasicHttpContext();
        this.deviceID = deviceID;
        this.devIndex = devIndex;
        this.proxyAddr = proxyAddr;
	}
	@Override
	public void run() {
		System.out.println(deviceID+" polling...");
		HttpPost postRequest = null;

		ResponseHandler<String> responseHandler = new BasicResponseHandler();
        String JSONServiceRequests;
		try {
			JSONServiceRequests = httpClient.execute(httpget, responseHandler);
			System.out.println(deviceID+" " + JSONServiceRequests);
			ArrayList<String> req_list = parseJSONServiceRequests(JSONServiceRequests, deviceID);
			for(int j = 0 ; j < req_list.size(); j++) {
            	String JSONResp = MainControlImpl.devs.get(devIndex).getJSONServiceResponse(req_list.get(j));
            	if(JSONResp != null) {
            		System.out.println(deviceID+": Serving request "+req_list.get(j));
            		String postURL = proxyAddr+"/sendresponse/"+deviceID+"/"+req_list.get(j);
       	            postRequest = new HttpPost(postURL);
       	            System.out.println("post : "+postURL);
       	            StringEntity input = new StringEntity(JSONResp);
       	    		input.setContentType("application/json");
       	    		postRequest.setEntity(input);
       	    		httpClient.execute(postRequest);
            	}
            	
			}
			
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			httpget.releaseConnection();
			if(postRequest != null)
				postRequest.releaseConnection();
		}
        
		
	}
	
	private ArrayList<String> parseJSONServiceRequests(String JSONServiceRequests, String deviceID) {
		ArrayList<String> reqList = new ArrayList<String>();
		
		try {
        	JSONParser parser = new JSONParser();
        	JSONObject pendingReqsObj = (JSONObject)parser.parse(JSONServiceRequests);
        	JSONArray pendingreqsArray = (JSONArray)pendingReqsObj.get("pendingRequests");
        	
        	for(int i = 0 ; i < pendingreqsArray.size(); i++) {
        		JSONObject req = (JSONObject)pendingreqsArray.get(i);
        		reqList.add((String)req.get("serviceName"));
        	}
						
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return reqList;
	}

}
