package test.java;
import java.util.HashSet;
import java.util.Random;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Formatter;

import test.java.services.Service;

/**
 * A VirtualDevice object is created for each unique deviceID.
 * This could change in the future; instead, we could have a
 * list of the valid device IDs and simply generate random values
 * for the noiselevel and location services.
 * 
 * @author lekkas@vtrip.net
 *
 */
public class VirtualDevice {
		
	private String ID;
	private HashSet<Service> services; 
	private static Random rand = new Random(System.currentTimeMillis());
	
	public VirtualDevice() {
		/*
		 * Create list of supported services
		 */
		services = new HashSet<Service>();

		/*  
		 * Create unique ID
		 */
		long rand_long = rand.nextLong();
		String str = ""+rand_long;
		try {
			MessageDigest md;
			md = MessageDigest.getInstance("SHA-1");
			ID = byteArray2Hex(md.digest(str.getBytes()));
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} 
	}
	
	public String getID() {
		return ID;
	}
	
	public boolean hasService(String serviceName) {
		for(Service srv : services) {
			if(srv.getName().equals(serviceName))
				return true;
		}
		return false;
	}
	
	public boolean addService(Service service) {
		if(!hasService(service.getName())) {
			services.add(service);
			return true;
		}
		return false;
	}
	
	public String getJSONServiceResponse(String serviceName) {
		for(Service srv : services) {
			if(srv.getName().equals(serviceName)) {
				return srv.getJSONData();
			}
		}
		return null;
	}
	private String byteArray2Hex(final byte[] hash) {
		Formatter formatter = new Formatter();
		for (byte b : hash) {
			formatter.format("%02x", b);
		}
		return formatter.toString();
	}
	
}
