package org.ow2.choreos.registry;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map.Entry;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * Servlet implementation class Registry
 */
public class DecideAndRegisterServiceServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    /**
     * Default constructor.
     */
    public DecideAndRegisterServiceServlet() {
    }

    /**
     * @param request
     * @param response
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse
     * response)
     */
    @Override
    protected void doPost(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException {
        PrintWriter pr = response.getWriter();
        String relativeWebPath = "/sensorsAndActuators.rdf";
        String absoluteDiskPath = getServletContext().getRealPath(
                relativeWebPath);
        // System.out.println(absoluteDiskPath);

        String serviceAddress = request.getParameter("address");
        String deviceIdString = request.getParameter("device_id");
        String serviceName = request.getParameter("name");
        String serviceType = request.getParameter("type");
        String accuracyString = request.getParameter("accuracy");
        String sensorType = request.getParameter("sensor");
        String rangeString = request.getParameter("range");
        String unit = request.getParameter("unit");
        String globalRegion = request.getParameter("global_region");
        String concept = request.getParameter("concept");
        String locationX = request.getParameter("coordinateX");
        String locationY = request.getParameter("coordinateY");
        String dataType = request.getParameter("dataType");
        String period = request.getParameter("expiry_date");
        String estimatedPath = request.getParameter("estimated_path");
        String timeOfRequestString = request.getParameter("time_of_request");
        String thresholdString = request.getParameter("threshold");

        if ((serviceAddress == null) || (deviceIdString == null)
                || (serviceName == null) || (serviceType == null)
                || (accuracyString == null) || (sensorType == null)
                || (rangeString == null) || (globalRegion == null)
                || (concept == null) || (locationX == null)
                || (locationY == null) || (dataType == null)
                || (period == null) || (unit == null)
                || (estimatedPath == null) || (timeOfRequestString == null)
                || (thresholdString == null)) {
            response.sendError(500);
            return;
        }
        try {
            // the time of the request is sent, then i need to know that time at
            // which it starts
            long timeOfRequestReceipt = new Date().getTime();
            // int deviceId = Integer.parseInt(deviceIdString);
            long registrationExpDate = Long.parseLong(period);
            long timeOfRequest = Long.parseLong(timeOfRequestString);
            double range = Double.parseDouble(rangeString);
            double accuracy = Double.parseDouble(accuracyString);
            double locationXDouble = Double.parseDouble(locationX);
            double locationYDouble = Double.parseDouble(locationY);
            double threshold = Double.parseDouble(thresholdString);

            HashMap<Boolean, ArrayList<Double>> decisionMap = RegistryMap
                    .decide(absoluteDiskPath, sensorType, estimatedPath,
                            concept, threshold, range);
            // the time after it decided to know how long the decision takes
            long timeOfDecision = new Date().getTime();

            boolean result = true;
            for (Entry<Boolean, ArrayList<Double>> entry : decisionMap
                    .entrySet()) {
                Boolean decision = entry.getKey();
                if (decision == true) {
                    result = RegistryMap.add(serviceAddress, deviceIdString,
                            serviceName, serviceType, accuracy, sensorType,
                            range, globalRegion, concept, dataType,
                            locationXDouble, locationYDouble,
                            registrationExpDate, unit, estimatedPath);
                }
                // time after i added it
                long timeOfAdding = new Date().getTime();

                String timeString = decision + "," + serviceAddress + ","
                        + timeOfRequest + "," + timeOfRequestReceipt + ","
                        + timeOfDecision + "," + timeOfAdding + ","
                        + (timeOfDecision - timeOfRequestReceipt) + ","
                        + (timeOfAdding - timeOfDecision) + ","
                        + entry.getValue().get(0) + ","
                        + entry.getValue().get(1) + "\n";

                pr.print(timeString);
            }
            if (result == false) {

                response.sendError(400);

            }

        } catch (Exception e) {
            response.sendError(500);
        }
    }
}
