/**
 * ****************************************************************
 * Author: Georgios Mathioudakis (georgios.mathioudakis@inria.fr) Project:
 * RegistryManager package: org.ow2.choreos.registry Created: 3 juil. 2013
 * Updated: Status: Description:
 * ****************************************************************
 */
package org.ow2.choreos.registry;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.ow2.choreos.iots.common.ServiceDescription;

public class DevicesAroundPoint extends HttpServlet {

    private static final long serialVersionUID = 1L;

    public DevicesAroundPoint() {
        super();
    }

    /**
     * @param request
     * @param response
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse
     * response)
     */
    @Override
    protected void doGet(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json");

        PrintWriter pr = response.getWriter();

        String center_X = request.getParameter("center_X");
        String center_Y = request.getParameter("center_Y");
        String range_string = request.getParameter("range"); // In meters
        String[] concepts = request.getParameterValues("concept");

        if ((center_X == null) || (center_Y == null) || (range_string == null) || (concepts == null)) {
            response.sendError(500);
            return;
        }

        double lat = Double.valueOf(center_X);
        double lon = Double.valueOf(center_Y);
        double range = Double.valueOf(range_string); // In meters

        try {
            Set<ServiceDescription> result = RegistryMap.findDevice("Paris",
                    concepts);

            if (result != null) {
                JSONArray ansArray = new JSONArray();
                Map<String, JSONArray> resultMap = new HashMap<String, JSONArray>();

                JSONObject obj = new JSONObject();
                for (ServiceDescription service : result) {
                    double distance = distFrom(lat, lon, service.getLocationX(),
                            service.getLocationY());
                    if (distance < range) {
                        JSONObject serviceObject = service.serializeAsJSON();
                        serviceObject.put("distance_meters", distance);
                        ansArray.add(serviceObject);
                        resultMap.put("value", ansArray);
                    }

                }

                obj.putAll(resultMap);
                obj.writeJSONString(pr);

            } else {
                response.sendError(404);
            }

        } catch (Exception e) {
            response.sendError(500);
        }
    }

    private double distFrom(double lat1, double lng1, double lat2, double lng2) {
        double earthRadius = 3958.75;
        double dLat = Math.toRadians(lat2 - lat1);
        double dLng = Math.toRadians(lng2 - lng1);
        double a = Math.sin(dLat / 2) * Math.sin(dLat / 2)
                + Math.cos(Math.toRadians(lat1))
                * Math.cos(Math.toRadians(lat2)) * Math.sin(dLng / 2)
                * Math.sin(dLng / 2);
        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        double miles = earthRadius * c;
        double meters = (miles / 0.62137) * 1000;
        return meters;
    }

}
