package org.ow2.choreos.registry;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.ow2.choreos.iots.common.ServiceDescription;

/**
 * Servlet implementation class FindDevices
 */
public class FindDevicesServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    /**
     * @see HttpServlet#HttpServlet()
     */
    public FindDevicesServlet() {
        super();
    }

    /**
     * @param request
     * @param response
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse
     * response)
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

        response.setContentType("application/json");
        response.addHeader("Access-Control-Allow-Origin", "*");
        response.addHeader("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE");
        response.addHeader("Access-Control-Allow-Headers", "Content-Type");
        response.addHeader("Access-Control-Max-Age", "1800");//30 min

        PrintWriter pr = response.getWriter();

        String globalRegion = request.getParameter("global_region");
        String[] concepts = request.getParameterValues("concept");

        if ((globalRegion == null) || (concepts == null)) {
            response.sendError(500);
            return;
        }
        try {
            Set<ServiceDescription> result = RegistryMap.findDevice(
                    globalRegion, concepts);

            if (result != null) {
                JSONArray ansArray = new JSONArray();
                Map<String, JSONArray> resultMap = new HashMap<String, JSONArray>();

                JSONObject obj = new JSONObject();
                Iterator<ServiceDescription> iter = result.iterator();
                int i = 0;
                while (iter.hasNext()) {
                    JSONObject serviceObject = iter.next().serializeAsJSON();
                    ansArray.add(serviceObject);

                    resultMap.put("value", ansArray);
                    i++;
                }

                obj.putAll(resultMap);
                obj.writeJSONString(pr);

            } else {
                response.sendError(400);
            }

        } catch (IOException e) {
            System.out.println(e.getMessage());
            response.sendError(500);
        }
    }

}
