package org.ow2.choreos.registry;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Set;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.ow2.choreos.iots.common.Point;
import org.ow2.choreos.iots.common.ServiceDescription;
import org.ow2.choreos.statistics.StatUtil;

/**
 * Servlet implementation class FindDevices
 */
public class FindSubSetOfDevicesAroundPointSim extends HttpServlet {

    private static final long serialVersionUID = 1L;

    /**
     * @see HttpServlet#HttpServlet()
     */
    public FindSubSetOfDevicesAroundPointSim() {
        super();
    }

    /**
     * @param request
     * @param response
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse
     * response)
     */
    @Override
    protected void doGet(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json");

        PrintWriter pr = response.getWriter();

        String globalRegion = request.getParameter("global_region");
        String pointOfInterestX = request.getParameter("point_of_interest_x");
        String pointOfInterestY = request.getParameter("point_of_interest_y");
        String distribution = request.getParameter("distribution");
        String[] dmax = request.getParameterValues("distance");
        //String coverage = request.getParameter("coverage");
        String size = request.getParameter("size");
        String time = request.getParameter("time");
        String[] concepts = request.getParameterValues("concept");

        if ((globalRegion == null) || (concepts == null)
                || (distribution == null) || (dmax == null)
                || (time == null)
                || (size == null) || (pointOfInterestX == null)
                || (pointOfInterestY == null)) {
            response.sendError(500);
            return;
        }
        try {
            String resultString = "";
            double dmaxVal = Double.parseDouble(dmax[0]);
            double pointOfInterestXVal = Double.parseDouble(pointOfInterestX);
            double pointOfInterstYVal = Double.parseDouble(pointOfInterestY);
            int n = Integer.parseInt(size);
            long t = Long.parseLong(time);
            Point locOfInterest = new Point(pointOfInterestXVal,
                    pointOfInterstYVal);
            Set<ServiceDescription> result = RegistryMap.findSubSetOfDevice(globalRegion, concepts, distribution, dmaxVal, locOfInterest, t, n);

            if (result != null) {
                resultString = resultString
                        + StatUtil.finalCoverage
                        + ","
                        + StatUtil.initialCoverage
                        + ","
                        + StatUtil.timeBefore
                        + ","
                        + StatUtil.timeAfter
                        + ","
                        + (StatUtil.timeAfterInterpolate - StatUtil.timeBeforeInterpolate)
                        + "," + result.size() + "," + RegistryMap.regSet.size()
                        + "," + StatUtil.noOfAvServices + "," + t;
                resultString += "\n";
                pr.print(resultString);
            } else {
                response.sendError(400);
            }

        } catch (IOException e) {
            System.out.println(e.getMessage());
            response.sendError(500);
        } catch (NumberFormatException e) {
            System.out.println(e.getMessage());
            response.sendError(500);
        }
    }
}
