package org.ow2.choreos.registry;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.ow2.choreos.iots.common.BorderPoints;
import org.ow2.choreos.iots.common.Point;
import org.ow2.choreos.iots.common.ServiceDescription;

/**
 * Servlet implementation class FindDevices
 */
//TODO: File is changed in Sara's version. 
public class FindSubSetOfDevicesInArea extends HttpServlet {

    private static final long serialVersionUID = 1L;

    /**
     * @see HttpServlet#HttpServlet()
     */
    public FindSubSetOfDevicesInArea() {
        super();
    }

    /**
     * @param request
     * @param response
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse
     * response)
     */
    @Override
    protected void doGet(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("application/json");

        PrintWriter pr = response.getWriter();

        String globalRegion = request.getParameter("global_region");
        String XBL = request.getParameter("bottom_left_point_of_interest_x");
        String YBL = request.getParameter("bottom_left_point_of_interest_y");
        String XBR = request.getParameter("bottom_right_point_of_interest_x");
        String YBR = request.getParameter("bottom_right_point_of_interest_y");
        String XUL = request.getParameter("upper_left_point_of_interest_x");
        String YUL = request.getParameter("upper_left_point_of_interest_y");
        String XUR = request.getParameter("upper_right_point_of_interest_x");
        String YUR = request.getParameter("upper_right_point_of_interest_y");

        String distribution = request.getParameter("distribution");
        String subsetSize = request.getParameter("subset_size");
        String[] concepts = request.getParameterValues("concept");

        if ((globalRegion == null) || (concepts == null)
                || (distribution == null) || (subsetSize == null)
                || (XBL == null) || (YBL == null) || (XBR == null)
                || (YBR == null) || (XUL == null) || (YUL == null)
                || (XUR == null) || (YUR == null)) {
            response.sendError(500);
            return;
        }
        try {

            BorderPoints AOI = new BorderPoints(
                    new Point(Double.parseDouble(XBL), Double.parseDouble(YBL)),
                    new Point(Double.parseDouble(XBR), Double.parseDouble(YBR)),
                    new Point(Double.parseDouble(XUL), Double.parseDouble(YUL)),
                    new Point(Double.parseDouble(XUR), Double.parseDouble(XUR)));
            int subsetSizeVal = Integer.parseInt(subsetSize);
            Set<ServiceDescription> result = RegistryMap
                    .findUniformSubSetOfDevice(globalRegion, concepts,
                            distribution, subsetSizeVal, AOI, new Date().getTime());

            if (result != null) {
                JSONArray ansArray = new JSONArray();
                Map<String, JSONArray> resultMap = new HashMap<String, JSONArray>();

                JSONObject obj = new JSONObject();
                Iterator<ServiceDescription> iter = result.iterator();
                int i = 0;
                while (iter.hasNext()) {
                    JSONObject serviceObject = iter.next().serializeAsJSON();
                    ansArray.add(serviceObject);

                    resultMap.put("value", ansArray);
                    i++;
                }

                obj.putAll(resultMap);
                obj.writeJSONString(pr);

            } else {
                response.sendError(400);
            }

        } catch (IOException e) {
            System.out.println(e.getMessage());
            response.sendError(500);
        } catch (NumberFormatException e) {
            System.out.println(e.getMessage());
            response.sendError(500);
        }
    }
}
