package org.ow2.choreos.registry;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.json.simple.JSONObject;
import org.ow2.choreos.iots.common.BorderPoints;
import org.ow2.choreos.iots.common.Point;

/**
 * Servlet implementation class GetNodes
 */
public class GetNodesInAreaServlet extends HttpServlet {

    private static final long serialVersionUID = 1L;

    /**
     * @see HttpServlet#HttpServlet()
     */
    public GetNodesInAreaServlet() {
        super();
    }

    /**
     * @param request
     * @param response
     * @throws javax.servlet.ServletException
     * @throws java.io.IOException
     * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse
     * response)
     */
    @Override
    protected void doGet(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException {
        PrintWriter pr = response.getWriter();
        response.setContentType("application/json");
        String deviceIdString = request.getParameter("device_id");
        String globalRegion = request.getParameter("global_region");
        String[] concepts = request.getParameterValues("concept");
        String XBL = request.getParameter("bottom_left_point_of_interest_x");
        String YBL = request.getParameter("bottom_left_point_of_interest_y");
        String XUR = request.getParameter("upper_right_point_of_interest_x");
        String YUR = request.getParameter("upper_right_point_of_interest_y");

        if ((deviceIdString == null) || (globalRegion == null)
                || (concepts == null) || (XBL == null) || (YBL == null)
                || (XUR == null) || (YUR == null)) {
            response.sendError(500);
            return;
        }
        try {
            BorderPoints AOI = new BorderPoints(
                    new Point(Double.parseDouble(XBL), Double.parseDouble(YBL)),
                    new Point(Double.parseDouble(XUR), Double.parseDouble(YBL)),
                    new Point(Double.parseDouble(XBL), Double.parseDouble(YUR)),
                    new Point(Double.parseDouble(XUR), Double.parseDouble(YUR)));
            // int deviceId = Integer.parseInt(deviceIdString);
            HashMap<String, Double> result = RegistryMap
                    .getNumberOfNodesInArea(deviceIdString, globalRegion,
                            concepts, AOI);
            if (result != null) {

                JSONObject obj = new JSONObject();
                obj.put("value", result);

                // System.out.println(obj);
                obj.writeJSONString(pr);
            } else {
                response.sendError(400);
            }

        } catch (IOException e) {
            System.out.println(e.getMessage());
            response.sendError(500);
        } catch (NumberFormatException e) {
            System.out.println(e.getMessage());
            response.sendError(500);
        }
    }

}
