package org.ow2.choreos.registry;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import org.ow2.choreos.iots.common.BorderPoints;
import org.ow2.choreos.iots.common.Location;
import org.ow2.choreos.iots.common.Point;
import org.ow2.choreos.iots.common.ServiceDescription;
import org.ow2.choreos.iots.common.SubstituteSensorSet;
import org.ow2.choreos.statistics.StatUtil;

public class RegistryMap {

    public static Set<ServiceDescription> regSet = new HashSet<ServiceDescription>();
    public static HashMap<String, Long> regMap = new HashMap<String, Long>();
    public static long timeOfReg;
    public static long timeOfLookReq = 0;
    public static long timeOfLookDecision = 0;

    // just send the info to the storage in every method
    // add the service desciption to the reg map
    /**
     * The method adds a service to a registry Set by creating a
     * ServiceDescription object. For now storage is in memory and not
     * consistent
     *
     * @param serviceAddress
     * @param deviceId
     * @param serviceName
     * @param serviceType
     * @param accuracy
     * @param sensorType
     * @param range
     * @param globalRegion
     * @param concept
     * @param dataType
     * @param locationX
     * @param locationY
     * @param registrationExpiry : the period after the registration time to
     * keep the service registered
     * @param unit
     * @param estimatedPath
     * @return
     * @throws SQLException
     */
    public static boolean add(String serviceAddress, String deviceId,
            String serviceName, String serviceType, double accuracy,
            String sensorType, double range, String globalRegion,
            String concept, String dataType, double locationX,
            double locationY, long registrationExpiry, String unit,
            String estimatedPath) throws SQLException {
        timeOfReg = new Date().getTime();

        boolean result = false;
        ServiceDescription myServiceDescription = new ServiceDescription(
                deviceId, serviceAddress, serviceName, serviceType, accuracy,
                globalRegion, range, sensorType, concept, dataType, locationX,
                locationY, unit, estimatedPath);
        try {
            for (ServiceDescription s : regSet) //				if (s.getAddress().toString().equals(serviceAddress)
            //						&& s.getConcept().toString().equals(concept)) {
            {
                if (s.getAddress().equals(serviceAddress)) {
                    result = false;
                    return result;
                }
            }
            regMap.put(serviceAddress, registrationExpiry);
            result = regSet.add(myServiceDescription);
            return result;

        } catch (Exception e) {
            return false;
        }
        /*
         * regMap.put(serviceAddress, registrationExpiry); return
         * SQLStorage.save( serviceAddress, deviceId, serviceName, serviceType,
         * accuracy, sensorType, range, globalRegion, concept, dataType,
         * locationX, locationY);
         */

    }

    /**
     * This method is used to remove a service from the registry based on its
     * address.
     *
     * @param serviceAddress
     * @return
     */
    public static boolean remove(String serviceAddress) {
        int index = -1;

        for (ServiceDescription s : regSet) {
            if (s.getAddress().toString().equals(serviceAddress)) {

                try {
                    regSet.remove(s);
                    return true;
                } catch (Exception e) {
                    return false;
                }

            }
        }
        return false;
        // return SQLStorage.removeServiceReq( serviceAddress);
    }

    /**
     * The method returns the number of registered services per service Type,
     * which is determined based on the concept they measure and within a
     * specific geographical region. We provide the id of the device requesting
     * this information so that it doesn t get counted if it had already
     * registered
     *
     * @param deviceId
     * @param globalRegion
     * @param concepts
     * @return
     * @throws SQLException
     */
    public static HashMap<String, Double> getNumberOfNodes(String deviceId,
            String globalRegion, String[] concepts) throws SQLException {
        HashMap<String, Double> NodesPerType = new HashMap<String, Double>();
        int index = -1;
        boolean checked = false;
        for (int i = 0; i < concepts.length; i++) {
            String concept = concepts[i];
            for (int j = 0; j <= i; j++) {
                if (concept.equals(concepts[j]) && i != j) {
                    checked = true;
                    break;
                } else {
                    checked = false;
                }
            }
            if (checked == false) {
                Iterator<ServiceDescription> iter = regSet.iterator();

                while (iter.hasNext()) {
                    ServiceDescription s = (ServiceDescription) iter.next();
                    if (s.getConcept().toString().equals(concept)
                            && (s.getDeviceID().equals(deviceId) == false)
                            && s.getGlobalRegion().toString()
                            .equals(globalRegion)) {
                        if (NodesPerType.get(concept) == null) {
                            NodesPerType.put(concept, 1.0);
                        } else {
                            NodesPerType.put(concept,
                                    NodesPerType.get(concept) + 1);
                        }

                    }

                }
                if (NodesPerType.get(concept) == null) {
                    NodesPerType.put(concept, 0.0);
                }
            }
        }

        return NodesPerType;
        /*
         * return SQLStorage.findNumberOfServicesReq( deviceId, globalRegion,
         * concepts);
         */
    }

    /**
     * The method finds the registered services based on their geographical
     * location and the concept they measure which is specified by the requestor
     *
     * @param globalRegion
     * @param concepts
     * @return
     */
    public static Set<ServiceDescription> findDevice(String globalRegion,
            String[] concepts) {
        // cleanUp();
        Set<ServiceDescription> thisSet = new HashSet();

        for (int i = 0; i < concepts.length; i++) {
            String concept = concepts[i];
            Iterator<ServiceDescription> iter = regSet.iterator();
            while (iter.hasNext()) {

                ServiceDescription s = (ServiceDescription) iter.next();
                if (s.getConcept().toString().equals(concept)
                        && s.getGlobalRegion().toString().equals(globalRegion)) {

                    thisSet.add(s);

                }
            }

        }
        return thisSet;

        /*
         * return SQLStorage.executeLookupQueryReq( globalRegion, concepts);
         */
    }

    /**
     * When services register, they register for a certain period, after this
     * period they should update the registration expiry date or else they will
     * be removed upon receiving a lookup request
     *
     * @param serviceAddress
     * @param regExpDate : the expiry date is a time duration
     * @return
     */
    public static boolean updateRegisteredService(String serviceAddress,
            long regExpDate) {
        try {
            regMap.remove(serviceAddress);
            regMap.put(serviceAddress, regExpDate);
            System.out.println(" updated service " + regMap);

        } catch (Exception e) {
            return false;
        }
        return true;
    }

    /**
     * This is to update the location of a device with registered services
     *
     * @param deviceID The device's unique ID
     * @param location_X The new latitude value
     * @param location_Y The new longitude value
     * @return
     */
    public static boolean updateRegisteredDevice(String deviceID,
            String location_X, String location_Y) {

        HashSet<ServiceDescription> servicesToAdd = new HashSet<ServiceDescription>();
        HashSet<ServiceDescription> servicesToRemove = new HashSet<ServiceDescription>();

        try {
            for (ServiceDescription s : regSet) {
                if (s.getDeviceID().equals(deviceID)) {
                    double new_location_X = 0;
                    double new_location_Y = 0;
                    try {
                        new_location_X = Double.valueOf(location_X);
                        new_location_Y = Double.valueOf(location_Y);
                    } catch (Exception e) {
                        System.out.println("Error converting location to Double");
                    }
                    ServiceDescription updatedService = new ServiceDescription(
                            s.getDeviceID(), s.getAddress(), s.getName(),
                            s.getType(), s.getAccuracy(), s.getGlobalRegion(),
                            s.getRange(), s.getSensor(), s.getConcept(),
                            s.getDataType(), new_location_X, new_location_Y,
                            s.getUnit(), s.getEstimatedPath());

                    servicesToAdd.add(updatedService);
                    servicesToRemove.add(s);
                }
            }

            removeServicesFromRegSet(servicesToRemove);
            addServicesToRegSet(servicesToAdd);

        } catch (Exception e) {
            return false;
        }
        return true;
    }

    public static void addServicesToRegSet(HashSet<ServiceDescription> services) {
        for (ServiceDescription s : services) {
            regSet.add(s);
        }
    }

    public static void removeServicesFromRegSet(HashSet<ServiceDescription> services) {
        for (ServiceDescription s : services) {
            regSet.remove(s);
        }
    }

    /**
     * The method removes all registered services whose registration date
     * expired. it is called upon receiving a lookup request
     *
     */
    public static void cleanUp() {
        HashMap<String, Long> tempMap = new HashMap<String, Long>();
        tempMap.putAll(regMap);
        for (Entry<String, Long> entry : regMap.entrySet()) {
            long expDate = entry.getValue();
            if (expDate <= new Date().getTime()) {
                tempMap.remove(entry.getKey());
                // System.out.println("removed service" + regMap);
            }
        }
        regMap = tempMap;

    }

    /**
     * The method removes all registered services
     *
     * @return
     */
    public static boolean empty() {
        regSet = new HashSet<ServiceDescription>();
        regMap = new HashMap<String, Long>();
        regSet.clear();
        regMap.clear();
        return true;
    }

    /**
     * This method returns the subset of devices based on a required size and a
     * known distribution. It will call the StatUtils method which will
     * determine randomly which devices to return as discovered. for now we
     * assume probabilistic lookup works for one concept (it is used for events
     * that spread and events that need 6 sensors)
     *
     *
     * @param globalRegion
     * @param concepts
     * @param distribution : the distribution that nodes to select to follow
     * @param dmaxVal : the maximum distance beyond which nodes are not
     * considered
     * @param t
     * @param subsetSizeVal : the total number to nodes to select
     * @param locOfInterest : the location at which the event of interest is
     * taking place
     * @return
     */
    public static Set<ServiceDescription> findSubSetOfDevice(
            String globalRegion, String[] concepts, String distribution,
            double dmaxVal, Point locOfInterest, long t, int subsetSizeVal) {
        double ratio = 10;
        Set<ServiceDescription> fullSet = new HashSet<ServiceDescription>();
        Set<ServiceDescription> subSet = new HashSet<ServiceDescription>();
        Map<String, List<Location>> fullMap = new HashMap<String, List<Location>>();
        List<Location> estimatedPathPoints = new ArrayList<Location>();

        fullSet = findDevice(globalRegion, concepts);
        double radius = 0;
        for (ServiceDescription s : fullSet) {
            estimatedPathPoints = new ArrayList<Location>();
            String estimatedPathString = s.getEstimatedPath();
            String[] estimatedPathArr = estimatedPathString.split(";");
            radius = s.getRange();
            for (String estimatedPath : estimatedPathArr) {
                String[] locCoords = estimatedPath.split(",");
                double xCoord = Double.parseDouble(locCoords[0]);
                double yCoord = Double.parseDouble(locCoords[1]);
                Long time = Long.parseLong(locCoords[2]);
                estimatedPathPoints.add(new Location(xCoord, yCoord, time));
            }
            fullMap.put(s.getAddress(), estimatedPathPoints);
        }
        List<String> selectedSubSet = new ArrayList<String>();
        if (distribution.equals("normal")) {
            selectedSubSet = StatUtil.getFinalSubSet(fullMap, dmaxVal,
                    locOfInterest, subsetSizeVal, ratio, t);
        }
        // get the subset from a uniform distrubtion if event doesnt spread.
        // todo: 6 should not be a constant
        if (dmaxVal == 0) {
            selectedSubSet = StatUtil.getUniformFinalSubSet(fullMap,
                    new BorderPoints(new Point(locOfInterest.getX() - radius,
                                    locOfInterest.getY() - radius), new Point(
                                    locOfInterest.getX() + radius, locOfInterest.getY()
                                    - radius), new Point(locOfInterest.getX()
                                    - radius, locOfInterest.getY() + radius),
                            new Point(locOfInterest.getX() + radius,
                                    locOfInterest.getY() + radius)), 6, 100, t);
        }
        for (String service : selectedSubSet) {
            for (ServiceDescription sd : fullSet) {
                if (sd.getAddress().equals(service)) {
                    subSet.add(sd);
                }
            }
        }

        return subSet;
    }

    public static Set<ServiceDescription> findUniformSubSetOfDevice(
            String globalRegion, String[] concepts, String distribution,
            int subsetSizeVal, BorderPoints AOI, long t) {
        double ratio = 10;
        Set<ServiceDescription> fullSet = new HashSet<ServiceDescription>();
        Set<ServiceDescription> subSet = new HashSet<ServiceDescription>();
        Map<String, List<Location>> fullMap = new HashMap<String, List<Location>>();
        List<Location> estimatedPathPoints = new ArrayList<Location>();

        fullSet = findDevice(globalRegion, concepts);
        double radius = 0;
        for (ServiceDescription s : fullSet) {
            estimatedPathPoints = new ArrayList<Location>();
            String estimatedPathString = s.getEstimatedPath();
            String[] estimatedPathArr = estimatedPathString.split(";");
            radius = s.getRange();
            for (String estimatedPath : estimatedPathArr) {
                String[] locCoords = estimatedPath.split(",");
                double xCoord = Double.parseDouble(locCoords[0]);
                double yCoord = Double.parseDouble(locCoords[1]);
                Long time = Long.parseLong(locCoords[2]);
                estimatedPathPoints.add(new Location(xCoord, yCoord, time));
            }
            fullMap.put(s.getAddress(), estimatedPathPoints);
        }
        List<String> selectedSubSet = new ArrayList<String>();

        if (distribution.equals("uniform")) {
            selectedSubSet = StatUtil.getUniformFinalSubSet(fullMap, AOI,
                    subsetSizeVal, ratio, t);
        }
        for (String service : selectedSubSet) {
            for (ServiceDescription sd : fullSet) {
                if (sd.getAddress().equals(service)) {
                    subSet.add(sd);
                }
            }
        }

        return subSet;
    }

    /**
     * The method is used to determine whether or not a node should register by
     * computing whether or not its path is covered by registered similar nodes,
     * without including composition.
     *
     * @param estimatedPath
     * @param concept
     * @param threshold
     * @param range
     * @return
     */
    public static HashMap decide(String absoluteDiskPath, String sensorType,
            String estimatedPath, String concept, double threshold, double range) {
        Map<Integer, SubstituteSensorSet> setsOfSubSensors = new HashMap<Integer, SubstituteSensorSet>();
        ArrayList<String> conceptsOfInterest = new ArrayList<String>();
        boolean intersectionStatus = false;
        //StatUtil.initialize(absoluteDiskPath);
        boolean intersectionWithComposition;
        List<Location> mainPathPoints = new ArrayList<Location>();
        String[] mainPathArr = estimatedPath.split(";");
        for (int i = 0; i < mainPathArr.length; i++) {
            String[] mainLocCoords = mainPathArr[i].split(",");
            double mainXCoord = Double.parseDouble(mainLocCoords[0]);
            double mainYCoord = Double.parseDouble(mainLocCoords[1]);
            Long mainTime = Long.parseLong(mainLocCoords[2]);
            mainPathPoints.add(new Location(mainXCoord, mainYCoord, mainTime));
        }
        ArrayList<Location> coveredLocations = new ArrayList<Location>();
        try {
            // Get substitution sets
//			setsOfSubSensors = StatUtil
//					.getSubstitutionSets(sensorType, concept);
            // for (Entry<Integer, SubstituteSensorSet> entry : setsOfSubSensors
            // .entrySet()) {
            // SubstituteSensorSet setOfSubs = entry.getValue();
            // for (int i = 0; i < setOfSubs.getAllConcepts().size(); i++)
            // if (conceptsOfInterest.contains(setOfSubs.getAllConcepts()
            // .get(i)) == false)
            // conceptsOfInterest.add(setOfSubs.getAllConcepts()
            // .get(i));
            //
            // }
        } catch (NullPointerException e) {
            System.out
                    .println("RegistrationEstimator.registerMe: getSubstitutionSets returned a null set");
            return null;
        }
        double noOfCheckedServices = 0;
        // Start checking coverage for each location
        for (Location MPL : mainPathPoints) {
            // get the path of each registered services of same concept
            serviceCheckLoop:
            for (ServiceDescription s : regSet) {
                if (s.getConcept().equals(concept)) {
                    noOfCheckedServices++;
                    ArrayList<Location> estimatedPathPoints = new ArrayList<Location>();
                    String path = s.getEstimatedPath();
                    String[] estimatedPathArr = path.split(";");

                    for (int i = 0; i < estimatedPathArr.length; i++) {
                        String[] locCoords = estimatedPathArr[i].split(",");
                        double xCoord = Double.parseDouble(locCoords[0]);
                        double yCoord = Double.parseDouble(locCoords[1]);
                        Long time = Long.parseLong(locCoords[2]);
                        estimatedPathPoints.add(new Location(xCoord, yCoord,
                                time));

                    }
                    // check if it intersects with new services at the current
                    // location
                    intersectionStatus = StatUtil.intersectsOneLoc(MPL,
                            estimatedPathPoints, s.getRange());
                    /*
                     * for (int i = 0; i < locs.size(); i++) for (Location l :
                     * locs) if (locs.get(i).getXCoordinate() !=
                     * l.getXCoordinate() && locs.get(i).getYCoordinate() != l
                     * .getYCoordinate()) coveredLocations.add(l);
                     */
                    boolean alreadyAdded = false;
                    // if it intersects with it, add the location to covered
                    // locations and stop checking services
                    if (intersectionStatus == true) {

                        coveredLocations.add(MPL);

                        System.out.println("covered locations are "
                                + coveredLocations);
                        break serviceCheckLoop;

                    }
                }
            }
            // if after checking all services of the same type, we find that
            // none intersect with the path , start checking
            // compositions
            if (intersectionStatus == false) {
                boolean foundOneCoverage = false;
                // get substitution sets
                setCoverageCheck:
                for (Entry<Integer, SubstituteSensorSet> entry : setsOfSubSensors
                        .entrySet()) {
                    SubstituteSensorSet setOfSubs = entry.getValue();
                    // for each set, get concepts
                    for (int index = 0; index < setOfSubs.getAllConcepts()
                            .size(); index++) {
                        // check coverage for services of each concept

                        String thisConcept = setOfSubs.getAllConcepts().get(
                                index);
                        foundOneCoverage = false;
                        SubServiceCheckLoop:
                        for (ServiceDescription subS : regSet) {
                            if (subS.getConcept().equals(thisConcept)
                                    && thisConcept.equals(concept) == false) {
                                noOfCheckedServices++;
                                ArrayList<Location> estimatedPathPoints = new ArrayList<Location>();
                                String path = subS.getEstimatedPath();
                                String[] estimatedPathArr = path.split(";");

                                for (int j = 0; j < estimatedPathArr.length; j++) {
                                    String[] locCoords = estimatedPathArr[j]
                                            .split(",");
                                    double xCoord = Double
                                            .parseDouble(locCoords[0]);
                                    double yCoord = Double
                                            .parseDouble(locCoords[1]);
                                    Long time = Long.parseLong(locCoords[2]);
                                    estimatedPathPoints.add(new Location(
                                            xCoord, yCoord, time));

                                }
                                intersectionWithComposition = StatUtil
                                        .intersectsOneLoc(MPL,
                                                estimatedPathPoints,
                                                subS.getRange());
                                // if a service measuring the current concept
                                // interests with the path, we stop checking
                                // services and move to the next concept

                                if (intersectionWithComposition == true) {
                                    foundOneCoverage = true;
                                    break SubServiceCheckLoop;
                                } else {
                                    foundOneCoverage = false;
                                }
                            }
                        }
                    }

                    // if foundOneCoverage is true, it means that each concept
                    // in the current set is covered so
                    // the location can be considered as covered. We can stop
                    // checking coverage.
                    if (foundOneCoverage == true) {

                        coveredLocations.add(MPL);
                        break setCoverageCheck;

                    }
                }

            }
        }

        // Now we start computing the percentage of coverage
        boolean registrationDecision = false;
        double distance = 0;
        double totalDistance = 0;
        double oldX = 0;
        double oldY = 0;
        int locationCounter = 0;
        double oldXTotal = 0;
        double oldYTotal = 0;
        for (Location ml : mainPathPoints) {

            // because there might be overlaps between sensing coverage of
            // locations, we can t go beyond
            // the distance between two locations if they are less than
            // twice the sensing range (i add full range backwads and
            // intersection of ranges covered forward)
            if (locationCounter > 0) {
                totalDistance = totalDistance
                        + Math.sqrt(Math.pow((oldXTotal - ml.getXCoordinate()),
                                        2)
                                + Math.pow((oldYTotal - ml.getYCoordinate()), 2));
            }

            if (coveredLocations.contains(ml) == true) {
                if (locationCounter == 0) {

                    distance = distance + range;

                } else if (locationCounter < mainPathPoints.size() - 1) {
                    // totalDistance = totalDistance
                    // + Math.sqrt(Math.pow((oldXTotal - ml.getXCoordinate()),
                    // 2)
                    // + Math.pow((oldYTotal - ml.getYCoordinate()), 2));
                    if (Math.sqrt(Math.pow((ml.getXCoordinate() - oldX), 2)
                            + Math.pow((ml.getYCoordinate() - oldY), 2)) >= 2 * range) // totalDistance = totalDistance
                    // + Math.sqrt(Math.pow((oldXTotal -
                    // ml.getXCoordinate()),
                    // 2)
                    // + Math.pow((oldYTotal - ml.getYCoordinate()), 2));
                    {
                        distance = distance + 2 * range;
                    } else {
                        distance = distance
                                + Math.sqrt(Math.pow(
                                                (ml.getXCoordinate() - oldX), 2)
                                        + Math.pow(
                                                (ml.getYCoordinate() - oldY), 2));
                    }
                } else if (locationCounter == mainPathPoints.size() - 1) {
                    if (Math.sqrt(Math.pow((ml.getXCoordinate() - oldX), 2)
                            + Math.pow((ml.getYCoordinate() - oldY), 2)) >= 2 * range) {
                        distance = distance + range;
                    } else {
                        distance = distance
                                - range
                                + Math.sqrt(Math.pow(
                                                (ml.getXCoordinate() - oldX), 2)
                                        + Math.pow(
                                                (ml.getYCoordinate() - oldY), 2));
                    }
                }

                oldX = ml.getXCoordinate();
                oldY = ml.getYCoordinate();
            }
            oldXTotal = ml.getXCoordinate();
            oldYTotal = ml.getYCoordinate();

            locationCounter++;
        }
        System.out.println("distance " + distance);
        System.out.println("totalDistance " + totalDistance);
        System.out.println(distance / totalDistance);
        if (distance / totalDistance >= (threshold)) {
            registrationDecision = false;
        } else {
            registrationDecision = true;

        }
        HashMap registrationMap = new HashMap<Boolean, ArrayList<Double>>();
        ArrayList<String> vals = new ArrayList<String>();
        vals.add(String.valueOf(noOfCheckedServices));
        vals.add(estimatedPath);

        registrationMap.put(registrationDecision, vals);
        return registrationMap;

    }

    public static Set<ServiceDescription> findDeviceInSubRegion(
            String globalRegion, BorderPoints AOI, String[] concepts) {
        long t = new Date().getTime();
        List<ServiceDescription> servicesToCheck = new ArrayList<ServiceDescription>();
        List<Location> estimatedPathPoints;
        Map<String, List<Location>> fullMap = new HashMap<String, List<Location>>();
        Map<String, Location> oneLocPerDeviceMap = new HashMap<String, Location>();
        Set<ServiceDescription> selectedSet = new HashSet<ServiceDescription>();
        for (ServiceDescription sd : regSet) {
            if (sd.getGlobalRegion().equals(globalRegion)) {
                servicesToCheck.add(sd);
            }
        }

        for (ServiceDescription s : servicesToCheck) {
            estimatedPathPoints = new ArrayList<Location>();
            String estimatedPathString = s.getEstimatedPath();
            String[] estimatedPathArr = estimatedPathString.split(";");

            for (String estimatedPath : estimatedPathArr) {
                String[] locCoords = estimatedPath.split(",");
                double xCoord = Double.parseDouble(locCoords[0]);
                double yCoord = Double.parseDouble(locCoords[1]);
                Long time = Long.parseLong(locCoords[2]);
                // System.out.println(xCoord);
                estimatedPathPoints.add(new Location(xCoord, yCoord, time));
            }
            fullMap.put(s.getAddress(), estimatedPathPoints);
        }

        // check where services are at time t
        oneLocPerDeviceMap = StatUtil.interpolateAllServices(fullMap, t);
        for (Entry<String, Location> entry : oneLocPerDeviceMap.entrySet()) {
            Location curLoc = entry.getValue();
            if (curLoc.getXCoordinate() < AOI.getBottomRightPoint().getX()
                    && curLoc.getXCoordinate() > AOI.getBottomLeftPoint()
                    .getX()
                    && curLoc.getYCoordinate() < AOI.getUpperLeftPoint().getY()
                    && curLoc.getYCoordinate() > AOI.getBottomLeftPoint()
                    .getY()) {

                for (ServiceDescription sd : servicesToCheck) {
                    if (sd.getAddress().equals(entry.getKey())) {
                        selectedSet.add(sd);
                    }
                }
            }
        }

        return selectedSet;
    }

    /**
     * This method returns the subset of devices based on a required size and a
     * known distribution. It will call the StatUtils method which will
     * determine randomly which devices to return as discovered. for now we
     * assume probabilistic lookup works for one concept (it is used for events
     * that spread and events that need 6 sensors)
     *
     *
     * @param globalRegion
     * @param concepts
     * @param distribution : the distribution that nodes to select to follow
     * @param dmaxVal : the maximum distance beyond which nodes are not
     * considered
     * @param locOfInterest : the location at which the event of interest is
     * taking place
     * @return
     */
    public static Set<ServiceDescription> findSubSetOfDeviceBasedOnCoverage(
            String globalRegion, String[] concepts, String distribution,
            double dmaxVal, Point locOfInterest, long t, double c) {
        double ratio = 10;
        Set<ServiceDescription> fullSet = new HashSet<ServiceDescription>();
        Set<ServiceDescription> subSet = new HashSet<ServiceDescription>();
        Map<String, List<Location>> fullMap = new HashMap<String, List<Location>>();
        List<Location> estimatedPathPoints = new ArrayList<Location>();

        fullSet = findDevice(globalRegion, concepts);
        double radius = 0;
        // parse the device's estimated path
        for (ServiceDescription s : fullSet) {
            estimatedPathPoints = new ArrayList<Location>();
            String estimatedPathString = s.getEstimatedPath();
            String[] estimatedPathArr = estimatedPathString.split(";");
            radius = s.getRange();
            for (int i = 0; i < estimatedPathArr.length; i++) {
                String[] locCoords = estimatedPathArr[i].split(",");
                double xCoord = Double.parseDouble(locCoords[0]);
                double yCoord = Double.parseDouble(locCoords[1]);
                Long time = Long.parseLong(locCoords[2]);
                // System.out.println(xCoord);
                estimatedPathPoints.add(new Location(xCoord, yCoord, time));

            }
            fullMap.put(s.getAddress(), estimatedPathPoints);
        }
        List<String> selectedSubSet = new ArrayList<String>();
        // Map<String, Location> selectedSubSet = new HashMap<String,
        // Location>();
        // get the subset from a normal distribution if event spreads
        if (distribution.equals("normal")) {
            selectedSubSet = StatUtil.getFinalSubSetBasedOnCoverage(fullMap,
                    dmaxVal, locOfInterest, ratio, t, radius, c);
        }
        // get the subset from a uniform distrubtion if event doesnt spread.
        // todo: 6 should not be a constant
        if (dmaxVal == 0) {
            selectedSubSet = StatUtil.getUniformFinalSubSetBasedOnCoverage(
                    fullMap, new BorderPoints(new Point(locOfInterest.getX()
                                    - radius, locOfInterest.getY() - radius),
                            new Point(locOfInterest.getX() + radius,
                                    locOfInterest.getY() - radius), new Point(
                                    locOfInterest.getX() - radius,
                                    locOfInterest.getY() + radius), new Point(
                                    locOfInterest.getX() + radius,
                                    locOfInterest.getY() + radius)), c, ratio,
                    t, radius);
        }
        for (String service : selectedSubSet) {
            for (ServiceDescription sd : fullSet) {
                if (sd.getAddress().equals(service)) {
                    subSet.add(sd);
                }
            }
        }
        return subSet;
    }

    public static Set<ServiceDescription> findUniformSubSetOfDeviceBasedOnCoverage(
            String globalRegion, String[] concepts, String distribution,
            BorderPoints AOI, long t, double c) {
        double ratio = 20;
        Set<ServiceDescription> fullSet = new HashSet<ServiceDescription>();
        Set<ServiceDescription> subSet = new HashSet<ServiceDescription>();
        Map<String, List<Location>> fullMap = new HashMap<String, List<Location>>();
        List<Location> estimatedPathPoints = new ArrayList<Location>();

        fullSet = findDevice(globalRegion, concepts);
        double radius = 0;
        for (ServiceDescription s : fullSet) {
            estimatedPathPoints = new ArrayList<Location>();
            String estimatedPathString = s.getEstimatedPath();
            String[] estimatedPathArr = estimatedPathString.split(";");
            radius = s.getRange();
            for (int i = 0; i < estimatedPathArr.length; i++) {
                String[] locCoords = estimatedPathArr[i].split(",");
                double xCoord = Double.parseDouble(locCoords[0]);
                double yCoord = Double.parseDouble(locCoords[1]);
                Long time = Long.parseLong(locCoords[2]);
                // System.out.println(xCoord);
                estimatedPathPoints.add(new Location(xCoord, yCoord, time));

            }
            fullMap.put(s.getAddress(), estimatedPathPoints);
        }
        List<String> selectedSubSet = new ArrayList<String>();
        // Map<String, Location> selectedSubSet = new HashMap<String,
        // Location>();
        if (distribution.equals("uniform")) {
            timeOfLookReq = new Date().getTime();
        }
        selectedSubSet = StatUtil.getUniformFinalSubSetBasedOnCoverage(fullMap,
                AOI, c, ratio, t, radius);
        timeOfLookDecision = new Date().getTime();
        for (String service : selectedSubSet) {
            for (ServiceDescription sd : fullSet) {
                if (sd.getAddress().equals(service)) {
                    subSet.add(sd);
                }
            }
        }
        // for (Entry<String, Location> entry : selectedSubSet.entrySet()) {
        // for (ServiceDescription sd : fullSet) {
        // if (sd.getAddress().equals(entry.getKey())) {
        // sd.setLocationX(entry.getValue().getXCoordinate());
        // sd.setLocationY(entry.getValue().getYCoordinate());
        // subSet.add(sd);
        // break;
        // }
        // }
        // }
        if (selectedSubSet.size() != subSet.size()) {
            System.err
                    .println("something went wrong in returning the subset!!!!!");
        }
        return subSet;
    }

    /**
     * The method returns the number of nodes in a specific area. Ideally this
     * should be the area around the device as it moves. The global region can
     * be the city or the name of the region in which the device is moving, this
     * helps filter out some services before the interpolation.
     *
     * @param deviceId
     * @param globalRegion
     * @param concepts
     * @param AOI
     * @return
     */
    public static HashMap<String, Double> getNumberOfNodesInArea(
            String deviceId, String globalRegion, String[] concepts,
            BorderPoints AOI) {
        // TODO Auto-generated method stub
        long t = new Date().getTime();
        List<ServiceDescription> servicesToCheck = new ArrayList<ServiceDescription>();
        List<Location> estimatedPathPoints;
        Map<String, List<Location>> fullMap = new HashMap<String, List<Location>>();
        Map<String, Location> oneLocPerDeviceMap = new HashMap<String, Location>();
        Set<ServiceDescription> selectedSet = new HashSet<ServiceDescription>();
        for (ServiceDescription sd : regSet) {
            if (sd.getGlobalRegion().equals(globalRegion)) {
                servicesToCheck.add(sd);
            }
        }

        for (ServiceDescription s : servicesToCheck) {
            estimatedPathPoints = new ArrayList<Location>();
            String estimatedPathString = s.getEstimatedPath();
            String[] estimatedPathArr = estimatedPathString.split(";");

            for (int i = 0; i < estimatedPathArr.length; i++) {
                String[] locCoords = estimatedPathArr[i].split(",");
                double xCoord = Double.parseDouble(locCoords[0]);
                double yCoord = Double.parseDouble(locCoords[1]);
                Long time = Long.parseLong(locCoords[2]);
                // System.out.println(xCoord);
                estimatedPathPoints.add(new Location(xCoord, yCoord, time));

            }
            fullMap.put(s.getAddress(), estimatedPathPoints);
        }

        // check where services are at time t
        oneLocPerDeviceMap = StatUtil.interpolateAllServices(fullMap, t);
        for (Entry<String, Location> entry : oneLocPerDeviceMap.entrySet()) {
            Location curLoc = entry.getValue();
            if (curLoc.getXCoordinate() < AOI.getBottomRightPoint().getX()
                    && curLoc.getXCoordinate() > AOI.getBottomLeftPoint()
                    .getX()
                    && curLoc.getYCoordinate() < AOI.getUpperLeftPoint().getY()
                    && curLoc.getYCoordinate() > AOI.getBottomLeftPoint()
                    .getY()) {

                for (ServiceDescription sd : servicesToCheck) {
                    if (sd.getAddress().equals(entry.getKey())) {
                        selectedSet.add(sd);
                    }
                }
            }
        }
        HashMap<String, Double> NodesPerType = new HashMap<String, Double>();
        int index = -1;
        boolean checked = false;
        for (int i = 0; i < concepts.length; i++) {
            String concept = concepts[i];
            for (int j = 0; j <= i; j++) {
                if (concept.equals(concepts[j]) && i != j) {
                    checked = true;
                    break;
                } else {
                    checked = false;
                }
            }
            if (checked == false) {
                Iterator<ServiceDescription> iter = regSet.iterator();

                while (iter.hasNext()) {
                    ServiceDescription s = (ServiceDescription) iter.next();
                    if (s.getConcept().toString().equals(concept)
                            && (s.getDeviceID().equals(deviceId) == false)
                            && s.getGlobalRegion().toString()
                            .equals(globalRegion)) {
                        if (NodesPerType.get(concept) == null) {
                            NodesPerType.put(concept, 1.0);
                        } else {
                            NodesPerType.put(concept,
                                    NodesPerType.get(concept) + 1);
                        }

                    }

                }
                if (NodesPerType.get(concept) == null) {
                    NodesPerType.put(concept, 0.0);
                }
            }
        }

        return NodesPerType;
    }
}
