package org.ow2.choreos.statistics;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.math3.distribution.NormalDistribution;
import org.ow2.choreos.iots.common.BorderPoints;
import org.ow2.choreos.iots.common.Location;
import org.ow2.choreos.iots.common.Point;
import org.ow2.choreos.iots.common.SubstituteSensorSet;
import org.ow2.choreos.iots.Objects.Rectangle;
import org.ow2.choreos.iots.linesweep.LineSweep;

// in this class, we consider that x and y have the same variance and therefore when we get the range
//in the getLookupArary, we use it to get both x and y values
//
public class StatUtil {
	// protected static Model model;
    // public static String Sensors_NAMESPACE =
    // "http://www.semanticweb.org/ontologies/2011/7/S&A.owl#";
    // public static String RDF_NAMESPACE =
    // "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
    // public static String RDFS_NAMESPACE =
    // "http://www.w3.org/2000/01/rdf-schema#";
    public static double finalCoverage = 0;
    public static double interpolatedSubSetSize = 0;
    public static double initialCoverage = 0;
    public static int noOfAvServices = 0;
    public static List<String> newArray;
    public static long timeBeforeInterpolate = 0;
    public static long timeAfterInterpolate = 0;
    public static long timeBeforeGenRec = 0;
    public static long timeAfterGenRec = 0;
    public static long timeBeforeSweep = 0;
    public static long timeAfterSweep = 0;
    public static long timeBefore = 0;
    public static long timeAfter = 0;
    public static int outerLoop = 0;

	// private static String kbFile =
	// "/WebContent/WEB-INF/lib/sensorsAndActuators.rdf";
    static Map<Integer, SubstituteSensorSet> setsOfSubSensors = new HashMap<Integer, SubstituteSensorSet>();

    /**
     * The method returns an arraylist containing the ranges under the PDF that
     * satisfy the required area portions
     *
     *
     * @param areaRatio : the ratio of the p-quantile
     * @param distance : the total distance to be able to specify what the
     * variance value should be
     * @return a hashmap containing the distances from the mean for each subarea
     */
    public static List<Double> getLookupArray(double areaRatio, double distance) {
        double newRange, oldRange = 0;
        List<Double> lookupArray = new ArrayList<Double>();
        NormalDistribution nDist = new NormalDistribution(0, distance / 3);
        // to start from the 5th P-quantile
        for (double i = (5 + areaRatio / 10); i < 10; i = i + (areaRatio / 10)) {
            newRange = nDist.inverseCumulativeProbability(i / 10);
            lookupArray.add(newRange);

            oldRange = lookupArray.get(lookupArray.size() - 1);
            // System.out.println(i);
        }
        // System.out.println(" size " + lookupArray.size());

        return lookupArray;

    }

    /**
     * the method is used to construct a grid made of concentric unequal
     * rectangles from which sensor nodes should be selected
     *
     * @param eventOfInterestPoint : the location of the event of interest
     * around which the concentric rectangles will be constructed
     * @param lookupArray : the array list containing the computed ranges that
     * satisfy the required area ratios under the PDF curve
     * @return a HashMap containing the id of each square and the border points
     * of each rectangle
     */
	// public static Map<Integer, BorderPoints> getGridPoints(
    // List<Double> lookupArray, Point eventOfInterestPoint) {
    // Map<Integer, BorderPoints> gridPoints = new HashMap<Integer,
    // BorderPoints>();
    // int index = 0;
    // for (double range : lookupArray) {
    // // System.out.println(lookupArray);
    // index++;
    // // System.out.println("range for i" + index + "is " + range);
    // double xMin = eventOfInterestPoint.getX() - range;
    //
    // double xMax = eventOfInterestPoint.getX() + range;
    // double yMin = eventOfInterestPoint.getY() - range;
    // double yMax = eventOfInterestPoint.getY() + range;
    // gridPoints.put(index, new BorderPoints(new Point(xMin, yMin),
    // new Point(xMax, yMin), new Point(xMin, yMax), new Point(
    // xMax, yMax)));
    //
    // }
    //
    // return gridPoints;
    //
    // }
    public static Map<Integer, BorderPoints> getGridPoints(
            List<Double> lookupArray, Point eventOfInterestPoint) {
        Map<Integer, BorderPoints> gridPoints = new HashMap<Integer, BorderPoints>();
        int index = 0;
        double xMin, xMax, yMin, yMax;
        BorderPoints gridPoint = null;
        for (double range : lookupArray) {
			// System.out.println(lookupArray);

            // System.out.println("range for i" + index + "is " + range);
            if (index == 0) {
                xMin = eventOfInterestPoint.getX() - range;

                xMax = eventOfInterestPoint.getX() + range;
                yMin = eventOfInterestPoint.getY() - range;
                yMax = eventOfInterestPoint.getY() + range;
            } else {
                xMin = gridPoint.getBottomLeftPoint().getX() - range;

                xMax = gridPoint.getBottomRightPoint().getX() + range;
                yMin = gridPoint.getBottomLeftPoint().getY() - range;
                yMax = gridPoint.getUpperLeftPoint().getY() + range;
            }
            gridPoint = new BorderPoints(new Point(xMin, yMin), new Point(xMax,
                    yMin), new Point(xMin, yMax), new Point(xMax, yMax));
            gridPoints.put(index, gridPoint);
            index++;
        }

        return gridPoints;

    }

    /**
     * The method create the grid from which nodes should be uniformly selected
     * by dividing the area of interest into a set of equal squares starting
     * from the bottom left coordinates of the area of interest
     *
     * @param areaOfInterest : the area within which measurements should take
     * place
     * @param ratio : the ratio by which the side of the area should be divided
     * to create the grid squares
     * @return: the grid points starting from the bottom left of the area of
     * interest with index 0
     */
    public static Map<Integer, BorderPoints> getUniformGridPoints(
            BorderPoints areaOfInterest, double ratio) {
        Map<Integer, BorderPoints> gridPoints = new HashMap<Integer, BorderPoints>();
        double width = areaOfInterest.getWidth();
        double height = areaOfInterest.getHeight();
        int counter = 0;
        for (int i = 0; i <= Math.ceil(100 / ratio) - 1; i++) {
            for (int j = 0; j <= Math.ceil(100 / ratio) - 1; j++) {

                gridPoints.put(counter, new BorderPoints(new Point(
                        areaOfInterest.getBottomLeftPoint().getX() + i * width
                        * ratio / 100, areaOfInterest
                        .getBottomLeftPoint().getY()
                        + j
                        * height
                        * ratio / 100), new Point(areaOfInterest
                                .getBottomLeftPoint().getX()
                                + (i + 1)
                                * width
                                * ratio
                                / 100, areaOfInterest.getBottomLeftPoint().getY() + j
                                * height * ratio / 100), new Point(areaOfInterest
                                .getBottomLeftPoint().getX() + i * width * ratio / 100,
                                areaOfInterest.getBottomLeftPoint().getY() + (j + 1)
                                * height * ratio / 100), new Point(
                                areaOfInterest.getBottomLeftPoint().getX() + (i + 1)
                                * width * ratio / 100, areaOfInterest
                                .getBottomLeftPoint().getY()
                                + (j + 1)
                                * height
                                * ratio / 100)));
                counter++;

            }
        }
        return gridPoints;
    }

    /**
     * The method takes a hashmap containing the device ids and paths of phones
     * that measure the requested concept in the region of interest and we
     * estimate their current location
     *
     * @param fullMap
     */
    public static Map<String, Location> interpolateAllServices(
            Map<String, List<Location>> fullMap, long t) {

        Map<String, Location> interpolatedMap = new HashMap<String, Location>();
        int index = 0;
        Location selectedLocation = null;

        for (Entry<String, List<Location>> entry : fullMap.entrySet()) {
            List<Location> locations = entry.getValue();
            try {
                index = lowerBinarySearch(locations, 0, locations.size() - 1, t);
                selectedLocation = locations.get(index);
            } catch (ArrayIndexOutOfBoundsException e) {
                System.out.println("error in interpolateAllServices at "
                        + entry.getKey());
            }
            Location locationNow = null;
            if (index < 0) {
                continue;
            } else if (selectedLocation.getTime() == t) {
                locationNow = selectedLocation;
            } else if (index == locations.size() - 1) {
                continue;
            } else {
                locationNow = interpolate(selectedLocation,
                        locations.get(index + 1), t);
            }
            interpolatedMap.put(entry.getKey(), locationNow);

        }

        return interpolatedMap;
    }

    private static Location interpolate(Location minTimeLocation,
            Location maxTimeLocation, long time) {

        double alpha = ((double) (time - minTimeLocation.getTime()))
                / ((maxTimeLocation.getTime() - minTimeLocation.getTime()));

        return new Location(
                minTimeLocation.getXCoordinate()
                + alpha
                * (maxTimeLocation.getXCoordinate() - minTimeLocation
                .getXCoordinate()),
                minTimeLocation.getYCoordinate()
                + alpha
                * (maxTimeLocation.getYCoordinate() - minTimeLocation
                .getYCoordinate()), time);
    }

    /**
     * The method takes the path of registered services and estimates the
     * location of the registered services at a specific time t
     *
     * @param registeredSL : the path of the registered services
     * @param t : the time at which we want to estimate the location
     * @return Location: the estimated location
     */
    public static Location interpolateOneService(
            ArrayList<Location> registeredSL, long t) {

        int index = lowerBinarySearch(registeredSL, 0, registeredSL.size() - 1,
                t);
        Location locationNow = null;
        if (index != -1) {
            Location selectedLocation = registeredSL.get(index);

            if (index < 0) {

            } else if (selectedLocation.getTime() == t) {
                locationNow = selectedLocation;
            } else if (index == registeredSL.size() - 1) {

            } else {
                locationNow = interpolate(selectedLocation,
                        registeredSL.get(index + 1), t);
            }
        }
        return locationNow;
    }

    /**
     * The method gets the index of the biggest time that is smaller than time
     * now
     *
     * @param arr the array of locations of nodes, each location has a time
     * @param minIndex
     * @param maxIndex
     * @param t : the time to which we are comparing times in the array, i.e.
     * time now
     * @return
     */
    public static int lowerBinarySearch(List<Location> arr, int minIndex,
            int maxIndex, long t) {

        if (arr == null || arr.size() < maxIndex + 1 || minIndex < 0
                || maxIndex < minIndex) {
            // basic error checks
            return -1;
        } else {
            int tempLength = maxIndex - minIndex + 1;

            if (tempLength == 1) {

                if (arr.get(minIndex).getTime() <= t) {
                    return minIndex;
                } else // the value is larger
                {
                    return minIndex - 1; // this will also return -1 if the
                }											// minIndex give was 1, so all is
                // well
            } else if (tempLength == 2) {
                // maxIndex is minIndex + 1, fun times
                if (arr.get(minIndex).getTime() > t) {
                    // they are all larger
                    return minIndex - 1; // this will also return -1 if the
                    // minIndex give was 1, so all is
                    // well
                } else if (arr.get(maxIndex).getTime() <= t) {
                    // min is smaller, and so is max, so we return max
                    return maxIndex;
                } else {
                    // min is smaller, but max is larger. we return min
                    return minIndex;
                }
            } else {

                // tempLength is larger than 2, time to split
                int splitIndex = minIndex + tempLength / 2; // integer division

                if (arr.get(splitIndex).getTime() == t) {
                    return splitIndex;
                } else if (arr.get(splitIndex).getTime() > t) {
                    // check in lower array
                    return lowerBinarySearch(arr, minIndex, splitIndex - 1, t);
                } else {
					// split point still smaller
                    // check in upper array
                    return lowerBinarySearch(arr, splitIndex + 1, maxIndex, t);
                }
            }

        }
    }

    /**
     * The method takes the map containing the address of all services that
     * measure a concept and all their locations it uses dmax and the required
     * size of the subset and the location of interest to generate a subset of
     * services that are normally distributed around the locations of interest,
     * by dividing the area into concentric square and selecting an equal number
     * of sensors in each rectangle. important to note that the difference
     * between the square and the ones inside it as the grid square
     *
     * @param fullMap
     * @param dMax
     * @param locOfInterest
     * @param subsetSize
     * @return
     */
    public static List<String> getFinalSubSet(
            Map<String, List<Location>> fullMap, double dMax,
            Point locOfInterest, int subsetSize, double ratio, long t) {
        subsetSize = subsetSize + subsetSize / 2;
        Map<String, Location> oneLocPerDeviceMap = new HashMap<String, Location>();
        List<Double> lookupArray = new ArrayList<Double>();
        Map<Integer, BorderPoints> gridMap = new HashMap<Integer, BorderPoints>();
        Map<Integer, List<String>> serviceSubSetGrid = new HashMap<Integer, List<String>>();
        // check where services are at time t
        timeBeforeInterpolate = new Date().getTime();
        oneLocPerDeviceMap = StatUtil.interpolateAllServices(fullMap, t);
        timeAfterInterpolate = new Date().getTime();
        // compose the are into a grid

        lookupArray = getLookupArray(ratio, dMax);
        gridMap = getGridPoints(lookupArray, locOfInterest);
        // location services in grids
        serviceSubSetGrid = StatUtil.identifySquares(oneLocPerDeviceMap,
                gridMap);

        List<Rectangle> coverageRectangles = StatUtil.generateRectangles(
                oneLocPerDeviceMap, serviceSubSetGrid, 10);

        double coverage = LineSweep.sweep(coverageRectangles);
        double totalArea = Math.pow(2 * dMax, 2);
        double coveragePercentage = (coverage / totalArea) * 100.0;
        initialCoverage = coveragePercentage;

        noOfAvServices = 0;
        for (Entry<Integer, List<String>> entry : serviceSubSetGrid.entrySet()) {
            noOfAvServices = noOfAvServices + entry.getValue().size();
        }
        timeBefore = new Date().getTime();

        List<String> finalDeviceSubList = new ArrayList<String>();
        List<String> tempArray = new ArrayList<String>();
        List<String> newArray = new ArrayList<String>();
        if (noOfAvServices < subsetSize) {
            for (Entry<Integer, List<String>> entry : serviceSubSetGrid
                    .entrySet()) {
                newArray.addAll(entry.getValue());
            }
            timeAfter = new Date().getTime();
            coverageRectangles = StatUtil.generateRectangles(oneLocPerDeviceMap,
                    finalDeviceSubList, 10);
            coverage = LineSweep.sweep(coverageRectangles);
            coveragePercentage = (coverage / totalArea) * 100.0;
            finalCoverage = coveragePercentage;
            return newArray;
        } else {
            for (Entry<Integer, List<String>> entry : serviceSubSetGrid
                    .entrySet()) {
                List<String> tempDeviceSubList = new ArrayList<String>();
                newArray = entry.getValue();
                // I remove all services that are in the inner square
                boolean result = newArray.removeAll(tempArray);

                tempArray = newArray;
                serviceSubSetGrid.put(entry.getKey(), newArray);
                int counter = 0;
				// uniformly select services based on a pre-defined size.
                // TODO: this should change be updated to have the size selected
                // based on the binary search and add the sweep to check
                // coverage
                // after the selection
                counterLoop:
                while (counter < subsetSize / serviceSubSetGrid.size()
                        && newArray.size() > 0) {

                    int index = (int) (Math.ceil(Math.random()
                            * newArray.size())) - 1;
                    if (tempDeviceSubList.contains(newArray
                            .get(index)) == false
                            && finalDeviceSubList.contains(newArray
                                    .get(index)) == false) {
                        tempDeviceSubList.add(newArray.get(index));
                        newArray.remove(index);
                        // System.out.println(newArray.size());
                        counter++;
                    }
					// check if all services possible where selected, and if so
                    // stop
                    // or the loop will keep running
                    if (counter == noOfAvServices) {
                        break;
                    }
                    if (newArray.size() == 0) {
                        break counterLoop;
                    }

                }
                finalDeviceSubList.addAll(tempDeviceSubList);
            }
        }

        timeAfter = new Date().getTime();

        coverageRectangles = StatUtil.generateRectangles(oneLocPerDeviceMap,
                finalDeviceSubList, 10);
        coverage = LineSweep.sweep(coverageRectangles);
        coveragePercentage = (coverage / totalArea) * 100.0;
        finalCoverage = coveragePercentage;
        return finalDeviceSubList;
    }

    /**
     * The method takes the map containing the address of all services that
     * measure a concept and all their locations it uses dmax and the required
     * size of the subset and the location of interest to generate a subset of
     * services that are normally distributed around the locations of interest,
     * by dividing the area into concentric square and selecting an equal number
     * of sensors in each rectangle. important to note that the difference
     * between the square and the ones inside it as the grid square
     *
     * @param fullMap
     * @param dMax
     * @param locOfInterest
     * @param subsetSize
     * @return
     */
    public static List<String> getFinalSubSetBasedOnCoverage(
            Map<String, List<Location>> fullMap, double dMax,
            Point locOfInterest, double ratio, long t, double r, double c) {
        // TODO fix like uniform
        Map<String, Location> oneLocPerDeviceMap = new HashMap<String, Location>();
        List<Double> lookupArray = new ArrayList<Double>();
        Map<Integer, BorderPoints> gridMap = new HashMap<Integer, BorderPoints>();
        Map<Integer, List<String>> serviceSubSetGrid = new HashMap<Integer, List<String>>();
        // check where services are at time t
        oneLocPerDeviceMap = StatUtil.interpolateAllServices(fullMap, t);
        // compose the are into a grid
        lookupArray = getLookupArray(ratio, dMax);
        gridMap = getGridPoints(lookupArray, locOfInterest);
        // location services in grids
        serviceSubSetGrid = StatUtil.identifySquares(oneLocPerDeviceMap,
                gridMap);
        noOfAvServices = 0;
        for (Entry<Integer, List<String>> entry : serviceSubSetGrid.entrySet()) {
            noOfAvServices = noOfAvServices + entry.getValue().size();
        }
        // generate the rectangles that represent the coverage of each service
        List<Rectangle> coverageRectangles = StatUtil.generateRectangles(
                oneLocPerDeviceMap, serviceSubSetGrid, r);
        // get full coverage
        double coverage = LineSweep.sweep(coverageRectangles);
        System.out
                .println("total coverage in getFinalSubSetBasedOnCoverage is "
                        + coverage);
        // the area is always 4*dmax
        double totalArea = Math.pow(2 * dMax, 2);
        System.out.println("total area in getFinalSubSetBasedOnCoverage is "
                + totalArea);
        double coveragePercentage = (coverage / totalArea) * 100.0;
        System.out
                .println("total coveragePercentage in getFinalSubSetBasedOnCoverage is "
                        + coveragePercentage);
        initialCoverage = coveragePercentage;
        int subsetSize = noOfAvServices;

		// Map<String, Location> finalDeviceSubList = new HashMap<String,
        // Location>();
        List<String> finalDeviceSubList = new ArrayList<String>();
        List<String> tempArray = new ArrayList<String>();
        List<String> newArray = new ArrayList<String>();
		// if full coverage less than required coverage all devices should be
        // selected
        if (coveragePercentage < c) {

            for (Entry<Integer, List<String>> entry : serviceSubSetGrid
                    .entrySet()) {

                newArray = entry.getValue();
                finalDeviceSubList.addAll(newArray);
            }
        } else {
            subsetSize = noOfAvServices / 4;
            coveragePercentage = 0;
            double loops = 0;
            whileLoop:
            while (coveragePercentage < c) {
                finalDeviceSubList = new ArrayList<String>();
                System.out
                        .println("subset size to check in getFinalSubSetBasedOnCoverage is "
                                + subsetSize);

                for (Entry<Integer, List<String>> entry : serviceSubSetGrid
                        .entrySet()) {
                    if (finalDeviceSubList.size() < subsetSize) {
						// Map<String, Location> tempDeviceSubList = new
                        // HashMap<String, Location>();
                        List<String> tempDeviceSubList = new ArrayList<String>();
                        newArray = entry.getValue();
                        // I remove all services that are in the inner square
                        boolean result = newArray.removeAll(tempArray);

                        tempArray = newArray;

                        serviceSubSetGrid.put(entry.getKey(), newArray);

                        if (newArray.size() != 0) {
                            int counter = 0;
                            double limit = 0;
                            limit = Math.ceil((subsetSize)
                                    / serviceSubSetGrid.size());
							// uniformly select services based on a pre-defined
                            // size.
                            while ((double) counter < limit + loops) {
                                int index = (int) Math.abs((Math.ceil(Math
                                        .random() * newArray.size())) - 1);
								// System.out.println("finalDeviceSubList is "
                                // + finalDeviceSubList
                                // + " current item is "
                                // + newArray.get(index));

                                if (tempDeviceSubList.contains(newArray
                                        .get(index)) == false
                                        && finalDeviceSubList.contains(newArray
                                                .get(index)) == false) {

                                    tempDeviceSubList.add(newArray.get(index));
                                    counter++;
                                }

								// check if all services possible where
                                // selected, and if
                                // so stop
                                // or the loop will keep running
                                if (counter == noOfAvServices) {
                                    System.out
                                            .println("selected all devices in the interpolated map already");
                                    // break whileLoop;
                                }
                                // if (newArray.size() == 0) {
                                if (counter == newArray.size()) {
                                    System.out
                                            .println("selected all devices in the current grid already");
                                    break;
                                }
                            }
                            finalDeviceSubList.addAll(tempDeviceSubList);
                            // totalCounter = totalCounter + counter;

                        }

                    } else {
                        break;
                    }
                }

                // oldSubSize = subsetSize;
                coverageRectangles = StatUtil.generateRectangles(
                        oneLocPerDeviceMap, finalDeviceSubList, r);
                coverage = LineSweep.sweep(coverageRectangles);
                coveragePercentage = (coverage / totalArea) * 100.0;

                System.out
                        .println("final coverage of chosen subset in getFinalSubSetBasedOnCoverage is "
                                + coverage);
                double oldSubsetSize = subsetSize;
                subsetSize = subsetSize + (noOfAvServices - subsetSize) / 2;
                if (oldSubsetSize == subsetSize && subsetSize > 1) {
                    loops = loops + 1;
                    subsetSize = (int) Math.ceil(subsetSize + loops
                            * (subsetSize / serviceSubSetGrid.size()));

                }

                if (loops > 10) {
                    for (Entry<Integer, List<String>> entry : serviceSubSetGrid
                            .entrySet()) {
                        finalDeviceSubList = new ArrayList<String>();
                        newArray = entry.getValue();
						// System.out.println(" number of services in current grid is "
                        // + newArray.size());
                        for (int i = 0; i < newArray.size(); i++) {
                            finalDeviceSubList.add(newArray.get(i));
                        }
                    }
                    break whileLoop;
                }

				// newSubSize = subsetSize;
            }

        }
        System.out
                .println("final coverage percentage of chosen subset in getFinalSubSetBasedOnCoverage is "
                        + coveragePercentage);
        finalCoverage = coveragePercentage;
        return finalDeviceSubList;
    }

    /**
     * The method takes the map containing the address of all services that
     * measure a concept and all their locations it uses the required size of
     * the subset and the area of interest to generate a subset of services that
     * are uniformly distributed within the area of interest, by dividing the
     * area into equal square and selecting an equal number of sensors in each
     * square. The method is also called if the event of interest is a point has
     * does not spread and sensors to select should be within an small area of
     * +/- sensing range and the number of sensors should be 6.
     *
     * @param fullMap
     * @param areaOfInterest
     * @param subsetSize
     * @return
     */
    public static List<String> getUniformFinalSubSet(
            Map<String, List<Location>> fullMap, BorderPoints areaOfInterest,
            int subsetSize, double ratio, long t) {
        Map<String, Location> oneLocPerDeviceMap = new HashMap<String, Location>();
        Map<Integer, BorderPoints> gridMap = new HashMap<Integer, BorderPoints>();
        Map<Integer, List<String>> serviceSubSetGrid = new HashMap<Integer, List<String>>();
        oneLocPerDeviceMap = StatUtil.interpolateAllServices(fullMap, t);
        List<String> finalDeviceSubList = new ArrayList<String>();
        List<String> newArray = new ArrayList<String>();
        timeBefore = new Date().getTime();
        gridMap = getUniformGridPoints(areaOfInterest, ratio);

        serviceSubSetGrid = StatUtil.identifySquares(oneLocPerDeviceMap,
                gridMap);
        List<Rectangle> coverageRectangles = StatUtil.generateRectangles(
                oneLocPerDeviceMap, serviceSubSetGrid, 10);

        double coverage = LineSweep.sweep(coverageRectangles);
        double totalArea = (areaOfInterest.getWidth() + 2 * 10)
                * (areaOfInterest.getHeight() + 2 * 10);
        double coveragePercentage = (coverage / totalArea) * 100.0;
        initialCoverage = coveragePercentage;

        noOfAvServices = 0;
        for (Entry<Integer, List<String>> entry : serviceSubSetGrid.entrySet()) {
            noOfAvServices = noOfAvServices + entry.getValue().size();

            newArray.addAll(entry.getValue());
        }
        if (noOfAvServices < subsetSize) {
            timeAfter = new Date().getTime();

            coverageRectangles = StatUtil.generateRectangles(oneLocPerDeviceMap,
                    finalDeviceSubList, 10);
            coverage = LineSweep.sweep(coverageRectangles);
            coveragePercentage = (coverage / totalArea) * 100.0;
            finalCoverage = coveragePercentage;
            return newArray;
        } else {
            subsetSize = subsetSize + subsetSize / 2;
            for (Entry<Integer, List<String>> entry : serviceSubSetGrid.entrySet()) {

                newArray = entry.getValue();
                serviceSubSetGrid.put(entry.getKey(), newArray);
                double counter = 0;
			// TODO: the size should be selected based on the binary search and
                // the coverage should be computed after the selected here
                //TODO comment this or put it at the beginning not here.

                counterLoop:
                while (counter < subsetSize / serviceSubSetGrid.size()
                        && newArray.size() > 0) {
                    int index = (int) (Math.ceil(Math.random() * newArray.size())) - 1;
                    if (finalDeviceSubList.contains(newArray.get(index)) == false) {
                        finalDeviceSubList.add(newArray.get(index));
                        newArray.remove(index);
					// System.out.println(newArray.size());

                        counter++;
                    }
                    if (counter == noOfAvServices) {
                        break;
                    }
                    if (newArray.size() == 0) {
                        break counterLoop;
                    }

                }
            }
            timeAfter = new Date().getTime();

            coverageRectangles = StatUtil.generateRectangles(oneLocPerDeviceMap,
                    finalDeviceSubList, 10);
            coverage = LineSweep.sweep(coverageRectangles);
            coveragePercentage = (coverage / totalArea) * 100.0;
            finalCoverage = coveragePercentage;
        }
        return finalDeviceSubList;
    }

    /**
     * The method takes the map containing the address of all services that
     * measure a concept and all their locations it uses a generated size of the
     * subset and the area of interest to generate a subset of services that are
     * uniformly distributed within the area of interest and should satisfy a
     * coverage threshold c, by dividing the area into equal square and
     * selecting an equal number of sensors in each square. The method is also
     * called if the event of interest is a point has does not spread and
     * sensors to select should be within an small area of +/- sensing range and
     * the number of sensors should be 6. TODO not all services in the
     * interpolated map are in the grid, so the size don t have to be the same
     * and i might selected less than the subsetsize but it s ok because i
     * compute their coverage afterwards so if it s enough then it doesnt
     * matter, i actually don t need to make sure i get the same size here but i
     * do for the method that takes it as input.
     *
     * @param fullMap
     * @param areaOfInterest
     * @param subsetSize
     * @return
     */
    public static List<String> getUniformFinalSubSetBasedOnCoverage(
            Map<String, List<Location>> fullMap, BorderPoints areaOfInterest,
            double c, double ratio, long t, double r) {
        Map<String, Location> oneLocPerDeviceMap = new HashMap<String, Location>();
        Map<Integer, BorderPoints> gridMap = new HashMap<Integer, BorderPoints>();
        Map<Integer, List<String>> serviceSubSetGrid = new HashMap<Integer, List<String>>();
        timeBeforeInterpolate = new Date().getTime();
        oneLocPerDeviceMap = StatUtil.interpolateAllServices(fullMap, t);
        timeAfterInterpolate = new Date().getTime();
        interpolatedSubSetSize = oneLocPerDeviceMap.size();
        List<String> finalDeviceSubList = new ArrayList<String>();
        newArray = new ArrayList<String>();
        // create the uniform grid
        gridMap = getUniformGridPoints(areaOfInterest, ratio);
        // determine services belong to which square of the grid

        serviceSubSetGrid = StatUtil.identifySquares(oneLocPerDeviceMap,
                gridMap);
        noOfAvServices = 0;
        for (Entry<Integer, List<String>> entry : serviceSubSetGrid.entrySet()) {
            noOfAvServices = noOfAvServices + entry.getValue().size();
        }

        // generate the rectangles that represent the coverage of each service
        timeBeforeGenRec = new Date().getTime();
        List<Rectangle> coverageRectangles = StatUtil.generateRectangles(
                oneLocPerDeviceMap, serviceSubSetGrid, r);
        timeAfterGenRec = new Date().getTime();
        // get full coverage
        timeBeforeSweep = new Date().getTime();
        double coverage = LineSweep.sweep(coverageRectangles);
        timeAfterSweep = new Date().getTime();
        double totalArea = (areaOfInterest.getWidth() + 2 * r)
                * (areaOfInterest.getHeight() + 2 * r);

        double coveragePercentage = (coverage / totalArea) * 100.0;
        initialCoverage = coveragePercentage;
        int subsetSize = noOfAvServices;
        System.out
                .println("total area in getUniformFinalSubSetBasedOnCoverage is "
                        + totalArea);

        System.out
                .println("full set of available services size in getUniformFinalSubSetBasedOnCoverage is "
                        + noOfAvServices);
		// check the full coverage first, because if all available sensors are
        // not enough they should all be selected
        System.out.println("initial coverage percentage is "
                + coveragePercentage);
        if (coveragePercentage < (c + (c / 100)) || noOfAvServices == 1) {

            for (Entry<Integer, List<String>> entry : serviceSubSetGrid
                    .entrySet()) {

                newArray = entry.getValue();
				// System.out.println(" number of services in current grid is "
                // + newArray.size());
                // for (int i = 0; i < newArray.size(); i++)
                // finalDeviceSubList.put(newArray.get(i),
                // oneLocPerDeviceMap.get(newArray.get(i)));
                finalDeviceSubList.addAll(newArray);
            }

        } else {
            subsetSize = noOfAvServices / 4;
            coveragePercentage = 0;
            double loops = 0;

            whileLoop:
            while (coveragePercentage < c) {
                outerLoop++;
                finalDeviceSubList = new ArrayList<String>();

                System.out
                        .println("subset size to check in getUniformFinalSubSetBasedOnCoverage is "
                                + subsetSize);

                System.out
                        .println("trying to select a size equal to computed subset size");
                for (Entry<Integer, List<String>> entry : serviceSubSetGrid
                        .entrySet()) {
                    if (finalDeviceSubList.size() < subsetSize
                            && subsetSize <= noOfAvServices) {
						// Map<String, Location> tempDeviceSubList = new
                        // HashMap<String, Location>();
                        List<String> tempDeviceSubList = new ArrayList<String>();
                        newArray = entry.getValue();
                        serviceSubSetGrid.put(entry.getKey(), newArray);

                        if (newArray.size() != 0) {

                            int counter = 0;

							// while ((double) counter < (double) subsetSize
                            // / gridMap.size()) {
                            double limit = 0;
                            limit = Math.ceil((subsetSize)
                                    / serviceSubSetGrid.size());
							// counterLoop: while (counter <= limit + loops
                            // && newArray.size() > 0) {
                            // commented above to test if it makes difference.
                            // uncomment
                            counterLoop:
                            while (counter <= (subsetSize)
                                    / serviceSubSetGrid.size()
                                    && newArray.size() > 0) {
								// System.out
                                // .println("trying to selected a subset within a grid");
                                int index = (int) Math.abs((Math.ceil(Math
                                        .random() * newArray.size())) - 1);
								// System.out.println("randomly selected index "
                                // + index);
                                // if
                                // (tempDeviceSubList.get(newArray.get(index))
                                // == null
                                // && finalDeviceSubList.get(newArray
                                // .get(index)) == null) {
                                if (tempDeviceSubList.contains(newArray
                                        .get(index)) == false
                                        && finalDeviceSubList.contains(newArray
                                                .get(index)) == false) {
									// checking

									// tempDeviceSubList.put(newArray.get(index),
                                    // oneLocPerDeviceMap.get(newArray
                                    // .get(index)));
                                    tempDeviceSubList.add(newArray.get(index));
                                    newArray.remove(index);
                                    // System.out.println(newArray.size());
                                    counter++;
                                }

								// System.out.println(counter);
                                // check if all services possible where
                                // selected, and if
                                // so stop
                                // or the loop will keep running
                                if (counter == noOfAvServices) {
									// System.out
                                    // .println("selected all devices in the interpolated map already");
                                    break whileLoop;
                                }
                                if (newArray.size() == 0) {
                                    // if (counter == newArray.size()) {
                                    System.out
                                            .println("selected all devices in the current grid already");
                                    break counterLoop;
                                }

                            } // end of counter
                            finalDeviceSubList.addAll(tempDeviceSubList);

                        }
						// else if (repeating == true
                        // && newArray.size() > 0) {
                        // System.out
                        // .println("repeating because the selected subset is not in the requested size");
                        // double additionalCounter = 0;
                        //
                        // while (additionalCounter < (newArray.size() -
                        // subsetSize
                        // / gridMap.size())) {
                        // int index = (int) (Math.ceil(Math
                        // .random() * newArray.size())) - 1;
                        // System.out.println("selected index"
                        // + index);
                        // if (finalDeviceSubList.get(newArray
                        // .get(index)) == null) {
                        // System.out.println("added service "
                        // + newArray.get(index));
                        // finalDeviceSubList
                        // .put(newArray.get(index),
                        // oneLocPerDeviceMap
                        // .get(newArray
                        // .get(index)));
                        // additionalCounter++;
                        //
                        // }
                        //
                        // System.out
                        // .println("the number of services in current grid is"
                        // + newArray.size()
                        // + ", number of desired services in grid is + "
                        // + (subsetSize / gridMap
                        // .size()));
                        // }
                        // }
                    } else {
						// I think i should remove everything except the break
                        // because i m computing it afterwards
                        coverageRectangles = StatUtil.generateRectangles(
                                oneLocPerDeviceMap, finalDeviceSubList, r);
                        coverage = LineSweep.sweep(coverageRectangles);
                        coveragePercentage = (coverage / totalArea) * 100.0;
                        break whileLoop;
                    }
					// }
                    // repeating = true;

                }
                System.out.println("size we actually selected"
                        + finalDeviceSubList.size());
                coverageRectangles = StatUtil.generateRectangles(
                        oneLocPerDeviceMap, finalDeviceSubList, r);
                coverage = LineSweep.sweep(coverageRectangles);
                coveragePercentage = (coverage / totalArea) * 100.0;
				// System.out
                // .println("current coverage of chosen subset in getUniformFinalSubSetBasedOnCoverage is "
                // + coverage);
                // System.out
                // .println("current coverage percentage of chosen subset in getUniformFinalSubSetBasedOnCoverage is "
                // + coveragePercentage);
                double oldSubsetSize = subsetSize;
                subsetSize = subsetSize + (noOfAvServices - subsetSize) / 4;
                if (oldSubsetSize == subsetSize && subsetSize > 1) {
                    loops = loops + 1;
                    subsetSize = (int) Math.ceil(subsetSize + loops
                            * (subsetSize / serviceSubSetGrid.size()));

                }
                if (outerLoop > 3) {
                    for (Entry<Integer, List<String>> entry : serviceSubSetGrid
                            .entrySet()) {

                        newArray = entry.getValue();
						// System.out.println(" number of services in current grid is "
                        // + newArray.size());
                        // for (int i = 0; i < newArray.size(); i++)
                        // finalDeviceSubList.put(newArray.get(i),
                        // oneLocPerDeviceMap.get(newArray.get(i)));
                        finalDeviceSubList.addAll(newArray);
                    }
                    break whileLoop;
                }
            }
        }
		// System.out
        // .println("coverage percentage in getUniformFinalSubSetBasedOnCoverage is"
        // + coveragePercentage);
        finalCoverage = coveragePercentage;

        return finalDeviceSubList;
    }

    /**
     * The method creates coverage rectangles of individual services to use them
     * to compute complete coverage
     *
     * @param oneLocPerDeviceMap : the map contains the service address and its
     * location at the time of the request
     * @return array list of rectangles
     */
	// private static List<Rectangle> generateRectangles(
    // Map<String, Location> oneLocPerDeviceMap, double r) {
    // List<Rectangle> covRecs = new ArrayList<Rectangle>();
    // for (Entry<String, Location> entry : oneLocPerDeviceMap.entrySet()) {
    // String id = entry.getKey();
    // Location loc = entry.getValue();
    // Rectangle curRec = new Rectangle(loc.getXCoordinate() - r,
    // loc.getYCoordinate() - r, loc.getXCoordinate() + r,
    // loc.getYCoordinate() + r, id);
    // covRecs.add(curRec);
    //
    // }
    // return covRecs;
    //
    // }
    private static List<Rectangle> generateRectangles(
            Map<String, Location> oneLocPerDeviceMap,
            List<String> selectedServices, double r) {
        List<Rectangle> covRecs = new ArrayList<Rectangle>();
        for (String s : selectedServices) {

            Location loc = oneLocPerDeviceMap.get(s);
            Rectangle curRec = new Rectangle(loc.getXCoordinate() - r,
                    loc.getYCoordinate() - r, loc.getXCoordinate() + r,
                    loc.getYCoordinate() + r, s);
            covRecs.add(curRec);

        }
        return covRecs;

    }

    private static List<Rectangle> generateRectangles(
            Map<String, Location> oneLocPerDeviceMap,
            Map<Integer, List<String>> serviceSubSetGrid, double r) {
        List<Rectangle> covRecs = new ArrayList<Rectangle>();
        for (Entry<Integer, List<String>> serviceEntry : serviceSubSetGrid
                .entrySet()) {
            List<String> servicesInSubgrid = serviceEntry.getValue();
            if (servicesInSubgrid.isEmpty() == false) {

                for (String s : servicesInSubgrid) {
                    Location loc = oneLocPerDeviceMap.get(s);
                    if (loc != null) {
                        Rectangle curRec = new Rectangle(loc.getXCoordinate()
                                - r, loc.getYCoordinate() - r,
                                loc.getXCoordinate() + r, loc.getYCoordinate()
                                + r, s);

                        covRecs.add(curRec);
                    } else {
                        System.out.println(s + "has no location!");
                    }
                }
            } else {
                System.out.println("found an empty list of services");
            }

        }
		// for (Entry<String, Location> entry : oneLocPerDeviceMap.entrySet()) {
        // String id = entry.getKey();
        // Location loc = entry.getValue();
        // Rectangle curRec = new Rectangle(loc.getXCoordinate() - r,
        // loc.getYCoordinate() - r, loc.getXCoordinate() + r,
        // loc.getYCoordinate() + r, id);
        // covRecs.add(curRec);
        //
        // }
        return covRecs;

    }

    /**
     * the method is used to determine services belong to which parts of the
     * constructed grid based on their interpolated location, some of the
     * services might be outside the grid
     *
     * @param oneLocPerDeviceMap
     * @param gridMap
     * @return
     */
    public static Map<Integer, List<String>> identifySquares(
            Map<String, Location> oneLocPerDeviceMap,
            Map<Integer, BorderPoints> gridMap) {
        Map<Integer, List<String>> serviceSubSetGrid = new HashMap<Integer, List<String>>();
        double totalSize = 0;
        for (Entry<String, Location> entry : oneLocPerDeviceMap.entrySet()) {
            Location thisDeviceLoc = entry.getValue();
            gridLoop:
            for (Entry<Integer, BorderPoints> gridEntry : gridMap
                    .entrySet()) {
                BorderPoints rec = gridEntry.getValue();

                List<String> devicesInRec = new ArrayList<String>();
                if (rec.getBottomLeftPoint().getX() <= thisDeviceLoc
                        .getXCoordinate()
                        && rec.getBottomLeftPoint().getY() <= thisDeviceLoc
                        .getYCoordinate()
                        && rec.getBottomRightPoint().getX() >= thisDeviceLoc
                        .getXCoordinate()
                        && rec.getBottomRightPoint().getY() <= thisDeviceLoc
                        .getYCoordinate()
                        && rec.getUpperLeftPoint().getX() <= thisDeviceLoc
                        .getXCoordinate()
                        && rec.getUpperLeftPoint().getY() >= thisDeviceLoc
                        .getYCoordinate()
                        && rec.getUpperRightPoint().getX() >= thisDeviceLoc
                        .getXCoordinate()
                        && rec.getUpperRightPoint().getY() >= thisDeviceLoc
                        .getYCoordinate()) {
                    if (serviceSubSetGrid.isEmpty() == false) {
                        devicesInRec = serviceSubSetGrid
                                .get(gridEntry.getKey());
                    }
                    if (devicesInRec == null) {
                        devicesInRec = new ArrayList<String>();
                    }

                    String thisAdd = entry.getKey();
					// System.out.println("service address " + thisAdd +
                    // "rec bottom left"+ rec.myBottomLeft.getX());
                    devicesInRec.add(thisAdd);
                    serviceSubSetGrid.put(gridEntry.getKey(), devicesInRec);
                    totalSize = totalSize + 1;
                    break gridLoop;

                }

            }

        }
        if (totalSize != oneLocPerDeviceMap.size()) {
            System.err.println(totalSize + "initial size "
                    + oneLocPerDeviceMap.size()
                    + "------------different size!!!!!!!!----------");
        }
        return serviceSubSetGrid;
    }

    /**
     * the method checks if two paths intersect at the same time at some
     * locations and returns those location as covered.
     *
     * @param mainPathPoints : the Path of the phone to register
     * @param estimatedPathPoints : the Path of a registered service
     * @return a list of locations of the path of the phone to registered that
     * intersect with locations of a registered service path at the same time.
     */
    public static ArrayList<Location> intersects(List<Location> mainPathPoints,
            ArrayList<Location> estimatedPathPoints, double range) {
        ArrayList<Location> covered = new ArrayList<Location>();

        Location l;
        for (Location lMain : mainPathPoints) {

            long t = lMain.getTime();
            l = interpolateOneService(estimatedPathPoints, t);
			// check if the
            // registered service is within sensing range
            // of the location of my path
            if (l != null) {
                if (((lMain.getXCoordinate() - range) < l.getXCoordinate())
                        && ((lMain.getXCoordinate() + range) > l
                        .getXCoordinate())
                        && ((lMain.getYCoordinate() - range) < l
                        .getYCoordinate())
                        && ((lMain.getYCoordinate() + range) > l
                        .getYCoordinate())) {
                    covered.add(lMain);

                }
            }

        }

        return covered;
    }

    /**
     * the method checks if two path intersect at the same time at one specific
     * location and returns true if so and false otherwise.
     *
     * @param mainPathPoints : the Path of the phone to register
     * @param estimatedPathPoints : the Path of a registered service
     * @return a list of locations of the path of the phone to registered that
     * intersect with locations of a registered service path at the same time.
     */
    public static boolean intersectsOneLoc(Location lMain,
            ArrayList<Location> estimatedPathPoints, double range) {

        Location l;
        boolean covered = false;

        long t = lMain.getTime();
        l = interpolateOneService(estimatedPathPoints, t);
		// check if the
        // registered service is within sensing range
        // of the location of my path
        if (l != null) {
            if (((lMain.getXCoordinate() - range) < l.getXCoordinate())
                    && ((lMain.getXCoordinate() + range) > l.getXCoordinate())
                    && ((lMain.getYCoordinate() - range) < l.getYCoordinate())
                    && ((lMain.getYCoordinate() + range) > l.getYCoordinate())) {
                covered = true;
            }
        }

        return covered;
    }

    /**
     * the method is used to read the ontology and parse it into memory //
     */
	// public static void initialize(String absoluteDiskPath) {
    // try {
    // /*
    // * if (rdfFilePath == null) { rdfFilePath = "./"; System.err
    // * .println
    // * ("No path provided for the rdf file, using default path ./"); }
    // * else System.out.println("using path " + rdfFilePath +
    // * " for rdf file location");
    // *
    // * String kbFile = rdfFilePath + "sensorsAndActuators.rdf";
    // */
    // String kbFile = absoluteDiskPath;
    // model = ModelFactory.createDefaultModel();
    // File f = new File(kbFile);
    // InputStream i = null;
    // i = new FileInputStream(f);
    // // m.read(i, null);
    // model.read(i, null);
    // i.close();
    //
    // } catch (IOException e) {
    // System.out.println("SubstituteSetsGenerator.initialize: "
    // + e.getMessage());
    // return;
    // }
    // }
    /**
     * This method is used to generate a set of sensors that, based on some
     * mathematical computations over their measurements, can substitute a
     * sensor of interest. This set is defined in an ontology
     *
     * @param initSensorType : the initial sensor type to be substituted
     * @param initSensorConcept : the initial physical concept measured by the
     * sensor
     * @return a HashMap containing all substitute sets acquired from the
     * ontology and their ids (the ids are provided by the method itself)
     */
	// public static Map<Integer, SubstituteSensorSet> getSubstitutionSets(
    // String initSensorType, String initMeasuredConcept) {
    // // TODO ONLY FORMULAS WITH DIFFERENT SENSORS NEXT
    // String formula;
    // int i = 0;
    // // ArrayList<String> mySensorSet = new ArrayList<String>();
    // String queryStringFormula = "SELECT  DISTINCT ?formula WHERE { "
    // + "<"
    // + Sensors_NAMESPACE
    // + initSensorType
    // +
    // "> <http://www.semanticweb.org/ontologies/2011/7/S&A.owl#measures_concept> ?concept."
    // +
    // "?formula <http://www.semanticweb.org/ontologies/2011/7/S&A.owl#computes_concept> ?concept.} ";
    //
    // Query query = QueryFactory.create(queryStringFormula);
    //
    // // Execute the query and obtain results
    // QueryExecution qe = QueryExecutionFactory.create(query, model);
    // ResultSet results = qe.execSelect();
    // ArrayList<QuerySolution> resultList = (ArrayList<QuerySolution>)
    // ResultSetFormatter
    // .toList(results);
    //
    // Iterator<QuerySolution> itr = resultList.iterator();
    // while (itr.hasNext()) {
    // HashMap<String, String> setOfSensorsAndConcepts = new HashMap<String,
    // String>();
    //
    // String answer = itr.next().toString();
    //
    // answer = answer.substring(answer.indexOf("<") - 1,
    // answer.indexOf(">") + 1);
    // formula = answer;
    // String queryString = "SELECT  DISTINCT ?sensors ?pc WHERE { "
    // + formula
    // +
    // " <http://www.semanticweb.org/ontologies/2011/7/S&A.owl#has_input> ?param."
    // +
    // "?param <http://www.semanticweb.org/ontologies/2011/7/S&A.owl#physical_concept> ?pc."
    // +
    // "?sensors <http://www.semanticweb.org/ontologies/2011/7/S&A.owl#measures_concept> ?pc.} ";
    // Query sensorQuery = QueryFactory.create(queryString);
    // QueryExecution qexec = QueryExecutionFactory.create(sensorQuery,
    // model);
    // ResultSet sensorsResult = qexec.execSelect();
    // ArrayList<QuerySolution> sensorResultList = (ArrayList<QuerySolution>)
    // ResultSetFormatter
    // .toList(sensorsResult);
    // Iterator<QuerySolution> iter = sensorResultList.iterator();
    //
    // while (iter.hasNext()) {
    // String sensorsAnswer = "";
    // sensorsAnswer = iter.next().toString();
    //
    // String[] subSensors = sensorsAnswer.split(" ");
    // String subSensorConcept = subSensors[8];
    // subSensorConcept = subSensorConcept.substring(
    // subSensorConcept.indexOf("#") + 1,
    // subSensorConcept.indexOf(">"));
    // sensorsAnswer = sensorsAnswer.substring(
    // sensorsAnswer.indexOf("#") + 1,
    // sensorsAnswer.indexOf(">"));
    // setOfSensorsAndConcepts.put(sensorsAnswer, subSensorConcept);
    //
    // }
    //
    // i = i + 1;
    //
    // SubstituteSensorSet mySet = new SubstituteSensorSet(
    // setOfSensorsAndConcepts);
    // setsOfSubSensors.put(i, mySet);
    // // System.out.println("my set contains : " +
    // // setsOfSubSensors.get(i).getAllConcepts());
    // qexec.close();
    // }
    //
    // HashMap<String, String> mySensorSet = new HashMap<String, String>();
    // mySensorSet.put(initSensorType, initMeasuredConcept);
    // setsOfSubSensors.put(i + 1, new SubstituteSensorSet(mySensorSet));
    // // System.out.println(setsOfSubSensors.get(i+1).getAllConcepts());
    // qe.close();
    //
    // return setsOfSubSensors;
    //
    // }
}
