package test.java;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;
import org.ow2.choreos.iots.common.BorderPoints;
import org.ow2.choreos.iots.common.Location;
import org.ow2.choreos.iots.common.Point;
import org.ow2.choreos.statistics.StatUtil;

public class TestStatUtil {

	@Before
	public void setUp() throws Exception {
	}

	@Test
	public void testLookupArray() {
		List<Double> lookupArray = new ArrayList<Double>();
		Map<Integer, BorderPoints> gridPoints = new HashMap<Integer, BorderPoints>();
		lookupArray = StatUtil.getLookupArray(10, 30);
		// assertEquals(" the 50th-quantile is wrong", lookupArray.get(0), 0, //
		// 0);
		assertEquals(" the 60th-quantile is wrong", lookupArray.get(0),
				3.8002065, 0.001);
		assertEquals(" the 70th-quantile is wrong", lookupArray.get(1),
				7.866007, 0.001);
		assertEquals(" the 80th-quantile is wrong", lookupArray.get(2), 12.624,
				0.001);
		assertEquals(" the 90th-quantile is wrong", lookupArray.get(3),
				19.22327, 0.001);

	}

	@Test
	public void testInterpolateAllServices() {

		Map<String, Location> interpolatedMap = new HashMap<String, Location>();
		Map<String, List<Location>> fullMap = new HashMap<String, List<Location>>();
		List<Location> locList = new ArrayList<Location>();
		Long t = (long) 1347626255055.0;
		locList.add(new Location(10, 10, t - (long) 200000000000.0));
		locList.add(new Location(22, 25, t - (long) 100000000000.0));
		locList.add(new Location(36, 33, t - (long) 75000000000.0));
		locList.add(new Location(54, 22, t + (long) 300000));

		fullMap.put("service1", locList);

		interpolatedMap = StatUtil.interpolateAllServices(fullMap, t);
		assertEquals(
				"the interpolated x coordinate of the first service is not correct",
				interpolatedMap.get("service1").getXCoordinate(),
				53.9, 0.1);
		assertEquals(
				"the interpolated y coordinate of the first service is not correct",
				interpolatedMap.get("service1").getYCoordinate(),
				22.00004, 0.1);
		// System.out.println(" t " + t + " " // +
		// interpolatedMap.get("service1").getYCoordinate());

	}

	@Test
	public void testIdentifySquares() {

		Map<String, Location> interpolatedMap = new HashMap<String, Location>();
		Map<String, List<Location>> fullMap = new HashMap<String, List<Location>>();
		List<Double> lookupArray = new ArrayList<Double>();
		Map<Integer, BorderPoints> gridPoints = new HashMap<Integer, BorderPoints>();
		Map<Integer, List<String>> serviceSubSetGrid = new HashMap<Integer, List<String>>();
		lookupArray = StatUtil.getLookupArray(10, 30);
		List<Location> locList = new ArrayList<Location>();
		Long t = (long) 1347626255055.0;
		locList.add(new Location(10, 10, t - 200));
		locList.add(new Location(22, 25, t - 100));
		locList.add(new Location(1, 2, t - (long) 75));
		locList.add(new Location(4, 3, t + (long) 30));
		fullMap.put("service1", locList);

		locList = new ArrayList<Location>();
		locList.add(new Location(0, 0, t - 10));
		locList.add(new Location(1, 5, t - 200));
		locList.add(new Location(5, 6, t - 100));
		locList.add(new Location(7, 2, t + 4000));
		fullMap.put("service2", locList);

		locList = new ArrayList<Location>();
		locList.add(new Location(0, 0, t - 2000));
		locList.add(new Location(10, 10, t - 800));
		locList.add(new Location(10, 2, t - 300));
		fullMap.put("service3", locList);

		locList = new ArrayList<Location>();
		locList.add(new Location(3, 0, t - 1000));
		locList.add(new Location(1, 1, t - 200));
		locList.add(new Location(18, 18, t - 100));
		locList.add(new Location(4, 2, t + 300));
		fullMap.put("service4", locList);

		locList = new ArrayList<Location>();
		locList.add(new Location(0, 0, t - 1500));
		locList.add(new Location(1, 5, t - 430));
		locList.add(new Location(17, 17, t - 130));
		locList.add(new Location(7, 2, t + 4000));
		fullMap.put("service5", locList);

		locList = new ArrayList<Location>();
		locList.add(new Location(10, 0, t - 2000));
		locList.add(new Location(23, 25, t - 800));
		locList.add(new Location(2, 2, t - 300));
		locList.add(new Location(3, 20, t + 300));
		fullMap.put("service6", locList);

		locList = new ArrayList<Location>();
		locList.add(new Location(10, 10, t - 200));
		locList.add(new Location(22, 25, t - 100));
		locList.add(new Location(-1, -2, t - (long) 75));
		locList.add(new Location(4, 3, t + (long) 30));
		fullMap.put("service8", locList);

		interpolatedMap = StatUtil.interpolateAllServices(fullMap, t);
		gridPoints = StatUtil.getGridPoints(lookupArray, new Point(0, 0));
		serviceSubSetGrid = StatUtil.identifySquares(interpolatedMap,
				gridPoints);
		assertTrue(" services 1 should be in rectangle 2", serviceSubSetGrid
				.get(1).get(0).equals("service1"));
		assertTrue(" service 8 should be in rectangle 2", serviceSubSetGrid
				.get(1).get(1).equals("service8"));
		assertTrue(" service 2 should be in rectangle 3", serviceSubSetGrid
				.get(2).get(0).equals("service2"));
		assertTrue(" service 6 should be in rectangle 5", serviceSubSetGrid
				.get(3).get(0).equals("service6"));
		
	

	}

	// @Test
	// public void testFinalSubset() {
	//
	// Map<String, Location> interpolatedMap = new HashMap<String, Location>();
	// Map<String, List<Location>> fullMap = new HashMap<String,
	// List<Location>>();
	//
	// List<Location> locList = new ArrayList<Location>();
	// Long t = (long) 1347626255055.0;
	// locList.add(new Location(10, 10, t - 200));
	// locList.add(new Location(22, 25, t - 100));
	// locList.add(new Location(1, 2, t - (long) 75));
	// locList.add(new Location(4, 3, t + (long) 30));
	// fullMap.put("service1", locList);
	//
	// locList = new ArrayList<Location>();
	// locList.add(new Location(0, 0, t - 10));
	// locList.add(new Location(1, 5, t - 200));
	// locList.add(new Location(5, 6, t - 100));
	// locList.add(new Location(7, 2, t + 4000));
	// fullMap.put("service2", locList);
	//
	// locList = new ArrayList<Location>();
	// locList.add(new Location(0, 0, t - 2000));
	// locList.add(new Location(10, 10, t - 800));
	// locList.add(new Location(16, 17, t - 300));
	// fullMap.put("service3", locList);
	//
	// locList = new ArrayList<Location>();
	// locList.add(new Location(3, 0, t - 1000));
	// locList.add(new Location(1, 1, t - 200));
	// locList.add(new Location(18, 20, t - 100));
	// locList.add(new Location(4, 2, t + 300));
	// fullMap.put("service4", locList);
	//
	// locList = new ArrayList<Location>();
	// locList.add(new Location(0, 0, t - 1500));
	// locList.add(new Location(1, 5, t - 430));
	// locList.add(new Location(19, 22, t - 130));
	// locList.add(new Location(7, 2, t + 4000));
	// fullMap.put("service5", locList);
	//
	// locList = new ArrayList<Location>();
	// locList.add(new Location(10, 0, t - 2000));
	// locList.add(new Location(23, 25, t - 800));
	// locList.add(new Location(2, 2, t - 300));
	// fullMap.put("service6", locList);
	//
	// locList = new ArrayList<Location>();
	// locList.add(new Location(3, 0, t - 1000));
	// locList.add(new Location(1, 1, t - 200));
	// locList.add(new Location(11, 3, t - 100));
	// locList.add(new Location(4, 2, t + 300));
	// fullMap.put("service7", locList);
	//
	// locList = new ArrayList<Location>();
	// locList.add(new Location(10, 10, t - 200));
	// locList.add(new Location(22, 25, t - 100));
	// locList.add(new Location(-1, -2, t - (long) 75));
	// locList.add(new Location(4, 3, t + (long) 30));
	// fullMap.put("service8", locList);
	//
	// for (int i = 9; i < 110; i++) {
	//
	// double x;
	// double y;
	// int sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// x = -Math.random() * 3.7;
	// else
	// x = Math.random() * 3.7;
	// sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// y = -Math.random() * 3.7;
	// else
	// y = Math.random() * 3.7;
	// locList = new ArrayList<Location>();
	// locList.add(new Location(x, y, t - 10));
	// String serviceName = "service" + i;
	// fullMap.put(serviceName, locList);
	// }
	//
	// int counter = 110;
	// while (counter < 210) {
	// locList = new ArrayList<Location>();
	// double x;
	// double y;
	// int sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// x = -Math.random() * 7.6;
	// else
	// x = Math.random() * 7.6;
	// sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// y = -Math.random() * 7.6;
	// else
	// y = Math.random() * 7.6;
	// if ((-7.6 < x && x < -3.76 && -7.6 < y && y < 7.6)
	// || (3.76 < x && x < 7.6 && -7.6 < y && y < 7.6)
	// || (-7.6 < x && x < 7.6 && -7.6 < y && y < -3.76)
	// || (-7.6 < x && x < 7.6 && 3.76 < y && y < 7.6)) {
	// locList.add(new Location(x, y, t - 10));
	// String serviceName = "service" + counter;
	// fullMap.put(serviceName, locList);
	// counter++;
	// }
	//
	// }
	//
	// counter = 210;
	// while (counter < 310) {
	// locList = new ArrayList<Location>();
	// double x;
	// double y;
	// int sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// x = -Math.random() * 11.55;
	//
	// else
	// x = Math.random() * 11.55;
	//
	// sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// y = -Math.random() * 11.55;
	// else
	// y = Math.random() * 11.55;
	//
	// if ((-11.55 < x && x < -7.6 && -11.55 < y && y < 11.55)
	// || (7.6 < x && x < 11.55 && -11.55 < y && y < 11.55)
	// || (-11.55 < x && x < 11.55 && -11.55 < y && y < -7.6)
	// || (-11.55 < x && x < 11.55 && 7.6 < y && y < 11.55)) {
	// locList.add(new Location(x, y, t - 10));
	// String serviceName = "service" + counter;
	// fullMap.put(serviceName, locList);
	// counter++;
	// }
	//
	// }
	//
	// counter = 310;
	// while (counter < 410) {
	// locList = new ArrayList<Location>();
	// double x;
	// double y;
	// int sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// x = -Math.random() * 15.73;
	// else
	// x = Math.random() * 15.73;
	// sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// y = -Math.random() * 15.73;
	// else
	// y = Math.random() * 15.73;
	//
	// if ((-15.73 < x && x < -11.55 && -15.73 < y && y < 15.73)
	// || (11.55 < x && x < 15.73 && -15.73 < y && y < 15.73)
	// || (-15.73 < x && x < 15.73 && -15.73 < y && y < -11.55)
	// || (-15.73 < x && x < 15.73 && 11.55 < y && y < 15.73)) {
	// locList.add(new Location(x, y, t - 10));
	// String serviceName = "service" + counter;
	// fullMap.put(serviceName, locList);
	// counter++;
	// }
	//
	// }
	//
	// counter = 410;
	// while (counter < 510) {
	// locList = new ArrayList<Location>();
	// double x;
	// double y;
	// int sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// x = -Math.random() * 20.23;
	// else
	// x = Math.random() * 20.23;
	// sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// y = -Math.random() * 20.23;
	// else
	// y = Math.random() * 20.23;
	//
	// if ((-20.23 < x && x < -15.73 && -20.23 < y && y < 20.23)
	// || (15.73 < x && x < 20.23 && -20.23 < y && y < 20.23)
	// || (-20.23 < x && x < 20.23 && -20.23 < y && y < -15.73)
	// || (-20.23 < x && x < 20.23 && 15.73 < y && y < 20.23)) {
	// locList.add(new Location(x, y, t - 10));
	// String serviceName = "service" + counter;
	// fullMap.put(serviceName, locList);
	// counter++;
	// }
	//
	// }
	//
	// counter = 510;
	// while (counter < 610) {
	// locList = new ArrayList<Location>();
	// double x;
	// double y;
	// int sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// x = -Math.random() * 25.24;
	// else
	// x = Math.random() * 25.24;
	// sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// y = -Math.random() * 25.24;
	// else
	// y = Math.random() * 25.24;
	//
	// if ((-25.23 < x && x < -20.23 && -25.24 < y && y < 25.24)
	// || (20.23 < x && x < 25.24 && -25.24 < y && y < 25.24)
	// || (-25.24 < x && x < 25.24 && -25.24 < y && y < -20.23)
	// || (-25.24 < x && x < 25.24 && 20.23 < y && y < 25.24)) {
	// locList.add(new Location(x, y, t - 10));
	// String serviceName = "service" + counter;
	// fullMap.put(serviceName, locList);
	// counter++;
	// }
	//
	// }
	//
	// counter = 610;
	// while (counter < 710) {
	// locList = new ArrayList<Location>();
	// double x;
	// double y;
	// int sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// x = -Math.random() * 31;
	// else
	// x = Math.random() * 31;
	// sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// y = -Math.random() * 31;
	// else
	// y = Math.random() * 31;
	//
	// if ((-31 < x && x < -25.24 && -31 < y && y < 31)
	// || (25.24 < x && x < 31 && -31 < y && y < 31)
	// || (-31 < x && x < 31 && -31 < y && y < -25.24)
	// || (-31 < x && x < 31 && 25.24 < y && y < 31)) {
	// locList.add(new Location(x, y, t - 10));
	// String serviceName = "service" + counter;
	// fullMap.put(serviceName, locList);
	// counter++;
	// }
	//
	// }
	//
	// counter = 710;
	// while (counter < 810) {
	// locList = new ArrayList<Location>();
	// double x;
	// double y;
	// int sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// x = -Math.random() * 38.4;
	// else
	// x = Math.random() * 38.4;
	// sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// y = -Math.random() * 38.4;
	// else
	// y = Math.random() * 38.4;
	//
	// if ((-38.4 < x && x < -31 && -38.4 < y && y < 38.4)
	// || (31 < x && x < 38.4 && -38.4 < y && y < 38.4)
	// || (-38.4 < x && x < 38.4 && -38.4 < y && y < -31)
	// || (-38.4 < x && x < 38.4 && 31 < y && y < 38.4)) {
	// locList.add(new Location(x, y, t - 10));
	// String serviceName = "service" + counter;
	// fullMap.put(serviceName, locList);
	// counter++;
	// }
	//
	// }
	//
	// counter = 810;
	// while (counter < 910) {
	// locList = new ArrayList<Location>();
	// double x;
	// double y;
	// int sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// x = -Math.random() * 49.34;
	// else
	// x = Math.random() * 49.34;
	// sign = (int) Math.round(Math.random());
	// if (sign == 0)
	// y = -Math.random() * 49.34;
	// else
	// y = Math.random() * 49.34;
	//
	// if ((-49.34 < x && x < -38.4 && -49.34 < y && y < 49.34)
	// || (38.4 < x && x < 49.34 && -49.34 < y && y < 49.34)
	// || (-49.34 < x && x < 49.34 && -49.34 < y && y < -38.4)
	// || (-49.34 < x && x < 49.34 && 38.4 < y && y < 49.34)) {
	// locList.add(new Location(x, y, t - 10));
	// String serviceName = "service" + counter;
	// fullMap.put(serviceName, locList);
	// counter++;
	// }
	//
	// }
	// Point locOfInterest = new Point(0, 0);
	// List<String> selectedServices = new ArrayList<String>();
	// selectedServices = StatUtil.getFinalSubSet(fullMap, 60, locOfInterest,
	// 800, 5, t);
	// // assertEquals("the selected number of services is wrong",
	// // selectedServices.size(), 28, 0);
	// ArrayList<Double> xCoords = new ArrayList<Double>();
	// ArrayList<Double> yCoords = new ArrayList<Double>();
	// interpolatedMap = StatUtil.interpolate(fullMap, t);
	// String vals = "";
	// for (int index = 0; index < selectedServices.size(); index++) {
	// xCoords.add(interpolatedMap.get(selectedServices.get(index))
	// .getXCoordinate());
	// yCoords.add(interpolatedMap.get(selectedServices.get(index))
	// .getYCoordinate());
	// vals = vals.concat("{"
	// + interpolatedMap.get(selectedServices.get(index))
	// .getXCoordinate()
	// + ","
	// + interpolatedMap.get(selectedServices.get(index))
	// .getYCoordinate() + "},");
	//
	// }
	// // System.out.println(vals);
	// // System.out.println(xCoords);
	// // System.out.println(yCoords);
	// // System.out.println(xCoords.size());
	// /*
	// * HashMap<Integer, Integer> freqMap = new HashMap<Integer, Integer>();
	// * for (int index = 0; index < xCoords.size(); index++) { for (int i =
	// * -30; i < 30; i++) { int freq; if (i < xCoords.get(index) &&
	// * xCoords.get(index) < (i + 1)) { try { freq = freqMap.get(i);
	// * freqMap.put(i, freq + 1); } catch (NullPointerException e) {
	// * freqMap.put(i, 1); }
	// *
	// * }
	// *
	// * } } System.out.print(freqMap);
	// */
	// }

	@Test
	public void testGetUniformGrid() {
		Map<Integer, BorderPoints> uniGridPoints = new HashMap<Integer, BorderPoints>();
		BorderPoints AOI = new BorderPoints(new Point(0, 0), new Point(10, 0),
				new Point(0, 10), new Point(10, 10));
		uniGridPoints = StatUtil.getUniformGridPoints(AOI, 20);
		assertEquals(" the first square should have x (lower left) = 0",
				uniGridPoints.get(0).getBottomLeftPoint().getX(), 0, 0);
		assertEquals(" the first square should have y (lower left)  = 0",
				uniGridPoints.get(0).getBottomLeftPoint().getY(), 0, 0);
		assertEquals(" the first square should have x (lower right) = 2",
				uniGridPoints.get(0).getBottomRightPoint().getX(), 2, 0);
		assertEquals(" the first square should have y (lower right) = 0",
				uniGridPoints.get(0).getBottomRightPoint().getY(), 0, 0);
		assertEquals(" the first square should have x (upper left) = 0",
				uniGridPoints.get(0).getUpperLeftPoint().getX(), 0, 0);
		assertEquals(" the first square should have y (upper left) = 2",
				uniGridPoints.get(0).getUpperLeftPoint().getY(), 2, 0);
		assertEquals(" the first square should have x (upper right) = 2",
				uniGridPoints.get(0).getUpperRightPoint().getX(), 2, 0);
		assertEquals(" the first square should have y (upper right) = 2",
				uniGridPoints.get(0).getUpperRightPoint().getY(), 2, 0);
		assertEquals("the size of the map should be 25", uniGridPoints.size(),
				25, 0);

	}

	@Test
	public void testIntersects() {
		List<Location> mainPathPoints = new ArrayList<Location>();
		ArrayList<Location> estimatedPathPoints = new ArrayList<Location>();
		double range = 2;
		mainPathPoints.add(new Location(1, 1, 1));
		mainPathPoints.add(new Location(5, 5, 2));
		mainPathPoints.add(new Location(6, 6, 3));

		estimatedPathPoints.add(new Location(2, 2, 1));
		estimatedPathPoints.add(new Location(4, 4, 2));
		estimatedPathPoints.add(new Location(5, 5, 3));

		ArrayList<Location> resultLocations = StatUtil.intersects(
				mainPathPoints, estimatedPathPoints, range);
		System.out.println(resultLocations.get(0).getXCoordinate());
		assertTrue("", resultLocations.get(0).getXCoordinate() == 1);
		assertTrue("", resultLocations.get(0).getYCoordinate() == 1);
		assertTrue("", resultLocations.get(1).getXCoordinate() == 5);
		assertTrue("", resultLocations.get(1).getYCoordinate() == 5);
		assertTrue("", resultLocations.get(2).getXCoordinate() == 6);
		assertTrue("", resultLocations.get(2).getYCoordinate() == 6);
	}


	// @Test
	// public void testGetUniformSubSet1() {
	// double ratio = 20;
	// Map<Integer, BorderPoints> uniGridPoints = new HashMap<Integer,
	// BorderPoints>();
	// BorderPoints AOI = new BorderPoints(new Point(0, 0), new Point(10, 0),
	// new Point(0, 10), new Point(10, 10));
	// uniGridPoints = StatUtil.getUniformGridPoints(AOI, ratio);
	// Long t = (long) 1347626255055.0;
	// int subsetSize = 75;
	// Map<String, List<Location>> fullMap = new HashMap<String,
	// List<Location>>();
	//
	// int counter = 0;
	// ArrayList<Location> locList;
	// int serviceId = 0;
	// for (int i = 1; i <= 100 / ratio; i++)
	// for (int j = 1; j <= 100 / ratio; j++) {
	// counter = 0;
	// while (counter < 5 * subsetSize / uniGridPoints.size()) {
	//
	// locList = new ArrayList<Location>();
	// double x;
	// double y;
	//
	// x = Math.random() * i * (AOI.getWidth() * ratio / 100);
	//
	// y = Math.random() * j * (AOI.getWidth() * ratio / 100);
	// if (((i - 1) * (AOI.getWidth() * ratio / 100) < x
	// && x < i * (AOI.getWidth() * ratio / 100)
	// && (j - 1) * (AOI.getWidth() * ratio / 100) < y && y < j
	// * (AOI.getWidth() * ratio / 100))) {
	// locList.add(new Location(x, y, t - 10));
	// String serviceName = "service" + serviceId;
	// fullMap.put(serviceName, locList);
	// counter++;
	// serviceId++;
	// }
	//
	// }
	// }
	// List<String> selectedSubSet = StatUtil.getUniformFinalSubSet(fullMap,
	// AOI, subsetSize, ratio, t);
	// ArrayList<Double> xCoords = new ArrayList<Double>();
	// ArrayList<Double> yCoords = new ArrayList<Double>();
	//
	// Map<String, Location> interpolatedMap = new HashMap<String, Location>();
	// interpolatedMap = StatUtil.interpolate(fullMap, t);
	// String vals = "";
	// for (int index = 0; index < selectedSubSet.size(); index++) {
	// xCoords.add(interpolatedMap.get(selectedSubSet.get(index))
	// .getXCoordinate());
	// yCoords.add(interpolatedMap.get(selectedSubSet.get(index))
	// .getYCoordinate());
	// vals = vals.concat("{"
	// + interpolatedMap.get(selectedSubSet.get(index))
	// .getXCoordinate()
	// + ","
	// + interpolatedMap.get(selectedSubSet.get(index))
	// .getYCoordinate() + "},");
	//
	// }
	// System.out.println(vals);
	// System.out.println(selectedSubSet);
	// System.out.println(selectedSubSet.size());
	//
	// }

}
