var updatedMarkers = [];
var myLatLng = [48.8, 2.1];
var URI;
var lookupQuery = "execute_lookup_query?global_region=Paris&concept=location&concept=noiselevel";
var interval = 180000;

var serviceCounter = {};
var mapServices = {};

$(document).ready(function() {
    if (Modernizr.localstorage) {
        var storedURI = localStorage.getItem('registry');
        if (storedURI !== null)
            $("#registry-url").val(storedURI);
    }
    URI = $("#registry-url").val();


    initMap();
    localizeMe();
    getIOT();
    accessServices();
    setInterval(getIOT, interval); //Update markers every 180 seconds
    settings();
    saveURI();

});

/*
 * Localize me
 * @returns {undefined}
 */
function localizeMe() {
    if (navigator.geolocation) {
        navigator.geolocation.getCurrentPosition(success, error);
    } else {
        error('not supported');
    }

}

/*
 * Localization was successful 
 * @returns {undefined}
 */
function success(position) {
    myLatLng = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
    //$("#map").gmap3('get').setCenter(myLatLng);
    $("#map").gmap3({
        map: {
            options: {
                center: myLatLng,
                zoom: 10
            }
        },
        marker: {
            values: [
                {latLng: myLatLng, data: "My Location", options: {icon: "img/pointer.png"}}
            ],
            events: {
                click: function(marker, event, context) {
                    var map = $(this).gmap3("get"),
                            infowindow = $(this).gmap3({get: {name: "infowindow"}});
                    if (infowindow) {
                        infowindow.open(map, marker);
                        infowindow.setContent(context.data);
                    } else {
                        $(this).gmap3({
                            infowindow: {
                                anchor: marker,
                                options: {content: context.data}
                            }
                        });
                    }
                }
            }
        }
    });
    //console.log("Centered" + myLatLng);
}

function error() {
    console.log("Localization error!");
}

function initMap() {
    $("#map").gmap3({
        map: {
            options: {
                zoom: 5,
                mapTypeControl: true,
                mapTypeControlOptions: {
                    style: google.maps.MapTypeControlStyle.DROPDOWN_MENU
                },
                navigationControl: true,
                scrollwheel: true

            }
        }
    });
}

/*
 * Main function for loading the services from the JSON file
 * @returns {undefined}
 */
function getIOT() {

    updatedMarkers = [];
    mapServices = {};
    serviceCounter = {};
    //console.log("URI: " + URI);

    //Load the services from the Registry, save them on a map-like struct for easy access
    var jqxhr = $.getJSON(URI, {format: "json",dataType: "jsonp"})
            .done(function(data) {
        $.each(data, function(i, item) {

            for (j = 0; j < item.length; j++) {
                var serviceObject = item[j];
                var concept = serviceObject["concept"];
                var deviceID = serviceObject["DeviceId"];
                var coordinateX = serviceObject["coordinateX"];
                var coordinateY = serviceObject["coordinateY"];
                var sensor = serviceObject["sensor"];
                var address = serviceObject["address"];

console.log(data);
                //Create an object for each service
                var service = new Object();
                service.address = address;
                service.concept = concept;
                service.deviceID = deviceID;

                serviceCounter[concept] = 0;

                if (deviceID in mapServices) {
                    servicesList = mapServices[deviceID].servicesList;
                    servicesList.push(service);
                }
                else {
                    //console.log("New device: " + deviceID);
                    mapServices[deviceID] = {
                        deviceID: deviceID,
                        latLng: [coordinateX, coordinateY],
                        servicesList: [service]
                    }
                }
            }
        });


        //For every device found loop its services and create the list for the markers
        for (deviceMarker in mapServices) {
            var printInfo = "<table class='service_table'><tr><th colspan=2>Service</th><th>Access</th></tr>";
            //console.log("Service for device: " + mapServices[deviceMarker].deviceID);

            for (service in mapServices[deviceMarker].servicesList) {
                icon = "";
                var serviceInstance = mapServices[deviceMarker].servicesList[service];

                //console.log("Service: " + serviceInstance.address);

                serviceCounter[serviceInstance.concept]++;

                if (serviceInstance.concept === "location") {
                    icon = "img/location_small.png";
                } else if (serviceInstance.concept === "noiselevel") {
                    icon = "img/noise_small.png";
                } else if (serviceInstance.concept === "temperature") {
                    icon = "img/temperature_small.png";
                } else if (serviceInstance.concept === "messaging") {
                    icon = "img/messaging_small.png";
                }
                if (serviceInstance.concept === "messaging")
                    printInfo += "<tr><td><img src='" + icon + "'/></td><td>" + serviceInstance.concept + "</td><td><input id='input_" + serviceInstance.deviceID + "_" + serviceInstance.concept + "' class='message_input' type='text'/><a id='" + serviceInstance.deviceID + "_" + serviceInstance.concept + "' class='access_service' data='" + serviceInstance.concept + "' href='" + serviceInstance.address + "'>Send message</a><img class='loading' src='img/loading.gif'/></td></tr>";
                else
                    printInfo += "<tr><td><img src='" + icon + "'/></td><td>" + serviceInstance.concept + "</td><td><a id='" + serviceInstance.deviceID + "_" + serviceInstance.concept + "' class='access_service' data='" + serviceInstance.concept + "' href='" + serviceInstance.address + "'>Get data</a><img class='loading' src='img/loading.gif'/></td></tr>";
                //console.log(printInfo);
            }

            updatedMarkers.push({
                deviceID: mapServices[deviceMarker].deviceID,
                latLng: mapServices[deviceMarker].latLng,
                data: printInfo,
                options: {
                    icon: "img/sensor.png"
                },
                tag: "sensor"
            });

            printInfo += "</table>";
        }

        //Re-draw the map with the new points
        $("#map").gmap3({
            clear: {tag: ["sensor"]},
            marker: {
                values: updatedMarkers,
                options: {
                    draggable: false
                },
                events: {
                    click: function(marker, event, context) {
                        var map = $(this).gmap3("get");
                        var infowindow = $(this).gmap3({get: {name: "infowindow"}});
                        if (infowindow) {
                            infowindow.open(map, marker);
                            infowindow.setContent(context.data);
                        } else {
                            $(this).gmap3({
                                infowindow: {
                                    anchor: marker,
                                    options: {content: context.data}
                                }
                            });
                        }
                    }
                }
            }
        });


        //Update text on service counters
        $('.location_service_counter').text(serviceCounter["location"]);
        $('.noiselevel_service_counter').text(serviceCounter["noiselevel"]);
        $('.temperature_service_counter').text(serviceCounter["temperature"]);
        $('.messaging_service_counter').text(serviceCounter["messaging"]);

        //console.log("Success");
    })
            .fail(function() {
        console.log("Error");
    });

    jqxhr.complete(function() {
        //console.log("Request completed");
    });
}

/*
 * Toggle settings panel function
 */
function settings() {
    $("#settings .settings-toggle").click(function() {
        $(".slide-settings").fadeToggle(150);
    });

    $("#settings .update-btn").click(function() {
        getIOT();
    });
}

/*
 * Functions for settings configuration
 */
function saveURI() {
    $("#settings-save").click(function() {
        URI = $("#registry-url").val();
        console.log("URI changed to: " + URI);
        if (Modernizr.localstorage)
            localStorage.setItem('registry', URI);
        $(".slide-settings .container .tick").fadeIn(150).delay(2000).fadeOut(200);
        getIOT();
        return false;
    });
}

function accessServices() {
    $('#iot').delegate('.access_service', 'click', function() {

        var selectedElement = $(this);
        var link = selectedElement.attr("href");
        var concept = selectedElement.attr("data");
        var id = selectedElement.attr("id");

        var infowindow = $("#map").gmap3({get: {name: "infowindow"}});
        var infowindowHTML = infowindow.getContent();
        infowindowContent = $.parseHTML(infowindowHTML);

        $("a#" + id).next('.loading').css("visibility", "visible");

        if (link !== null && concept !== null) {
            if (concept === "messaging") {
                $.post(link, $("#input_" + id).val())
                        .always(function(data) {
                    $("a#" + id).next('.loading').css("visibility", "hidden");
                });
            } else {
                var jqxhr = $.getJSON(link, {format: "json"})
                        .done(function(data) {
                    var sensorDataResponse = data["sensorDataResponse"];

                    $("a#" + id).next('.loading').css("visibility", "hidden");

                    if (concept === "noiselevel") {
                        var noiseValue = sensorDataResponse["value"];
                        if ($(infowindowContent).find(".access_value_" + concept).length > 0) {
                            $(infowindowContent).find(".access_value_" + concept).text(noiseValue + " db ");
                        } else {
                            $(infowindowContent).find("a#" + id).before("[ <span class='access_value_" + concept + "'>" + noiseValue + " db</span> ] ");
                            $(infowindowContent).find("a#" + id).text("Refresh");
                        }
                    } else if (concept === "location") {
                        var lat = sensorDataResponse["latitude"];
                        var lon = sensorDataResponse["longitude"];

                        if ($(infowindowContent).find(".access_value_" + concept).length > 0) {
                            $(infowindowContent).find(".access_value_" + concept).text(lat + ", " + lon + " ");
                        } else {
                            $(infowindowContent).find("a#" + id).before("[ <span class='access_value_" + concept + "'>" + lat + ", " + lon + "</span> ] ");
                            $(infowindowContent).find("a#" + id).text("Refresh");
                        }

                    } else if (concept === "temperature") {
                        if ($(infowindowContent).find(".access_value_" + concept).length > 0) {
                            $(infowindowContent).find(".access_value_" + concept).text("XX ");
                        } else {
                            $(infowindowContent).find("a#" + id).before("[ <span class='access_value_" + concept + "'>XX</span> ] ");
                            $(infowindowContent).find("a#" + id).text("Refresh");
                        }
                    }

                    var editedHTML = $("<div />").append($(infowindowContent).clone()).html();
                    infowindow.setContent(editedHTML);
                });
            }
        }

        return false;
    });


}
