package fr.inria.arles.thinglib.RESTAccess;

import java.text.DecimalFormat;
import android.util.Log;
import fr.inria.arles.thinglib.core.ArgumentOutOfRangeException;
import fr.inria.arles.thinglib.core.BadReplyException;
import fr.inria.arles.thinglib.core.Daemon;
import fr.inria.arles.thinglib.core.DaemonConnectionException;
import fr.inria.arles.thinglib.core.Mediator;
import fr.inria.arles.thinglib.core.MediatorListener;
import fr.inria.arles.thinglib.datatypes.DefaultDouble2dLocationData;
import fr.inria.arles.thinglib.datatypes.DefaultDoubleNoiseLevelData;
import fr.inria.arles.thinglib.devices.SensorInfo;
import fr.inria.arles.thinglib.devices.SensorResponse;

public class Controller extends MediatorListener {

	private MainActivity mView;
	private MediatorService mService;

	protected Mediator mSensorMediator;
	private static final String TAG = "Controller";

	private boolean locationEnabled = false;
	private boolean soundEnabled = false;
	private RestletServerResource restlet_server;
	private ResponseListener responseListener = null;
	private ResponseListener _streamListener = null;

	public Controller(MainActivity view) {
		mView = view;
		mSensorMediator = new fr.inria.arles.thinglib.platforms.android.Mediator(
				view, this);
	}

	public Controller(MediatorService service) {
		mService = service;
		mSensorMediator = new fr.inria.arles.thinglib.platforms.android.Mediator(
				mService, this);
		restlet_server = new RestletServerResource();
	}

	@Override
	public void onConnectionStateChanged(int newState) {
		if (newState == mSensorMediator.STATE_CONNECTED) {
			Log.i(TAG, "STATE_CONNECTED");
			restlet_server.startRESTLETServer();
		}
		if (newState == mSensorMediator.STATE_DISCONNECTED) {
			Log.i(TAG, "STATE_DISCONNECTED");
			restlet_server.stopRESTLETServer();
		}
		if (newState == mSensorMediator.STATE_UNINITIALIZED) {
			Log.i(TAG, "STATE_UNITIALIZED");
		}
	}

	@Override
	public void onDiscoveryStateChanged(int newState) {
		Log.i(TAG, "onDiscoveryStateChanged()");
	}

	@Override
	public void onSensingStateChanged(int newState) {
		Log.i(TAG, "onSensingStateChanged() : " + newState);
	}

	@Override
	public void onSensorsChanged(Iterable<SensorInfo> sensors) {
		Log.i(TAG, "onSensorsChanged()");

		for (SensorInfo info : sensors) {
			Log.i(TAG, "Added sensor: " + info.getUid());
			if (info.isIdentifiedBy("fr.inria.arles.sensor.soundAmplitude")) {
				soundEnabled = true;
				setSensorEnabled("fr.inria.arles.sensor.soundAmplitude", true);
			}

			if (info.isIdentifiedBy("fr.inria.arles.sensor.location")) {
				locationEnabled = true;
				setSensorEnabled("fr.inria.arles.sensor.location", true);
			}
		}
	}

	@Override
	public void onNewData(Iterable<SensorResponse> sensorResponses) {
		Log.i(TAG, "onNewData()");

		for (SensorResponse resp : sensorResponses)
			Log.i(TAG, "Got sensor response from: " + resp.sensorInfo.getUid());

		if (responseListener != null) {
			responseListener.onNewResponse(sensorResponses);
			responseListener = null;
		}

		if (_streamListener != null) {
			_streamListener.onNewResponse(sensorResponses);
			/*
			 * Change: Due to the new streaming feature the sensor mediator
			 * should not abort the sensing when a response is given back. The
			 * mediator service will stop the sense on the corresponding classes
			 * of sensing (e.g. LocationRestletResource) by calling the
			 * stopSensing() method of MediatorService in the method
			 * onNewResponce(). Editor: Georgios Mathioudakis
			 * (georgios.mathioudakis@inria.fr)
			 */

			/*
			 * responseListener = null; try { mSensorMediator.abortSensing();
			 * Log.i(TAG, "abortSensing()"); } catch (DaemonConnectionException
			 * e) { Log.i(TAG,
			 * "exception when aborting sensing: "+e.toString()); }
			 */
		}
	}

	public void startMed() {
		if (mSensorMediator.getConnectionState() != Mediator.STATE_CONNECTED) {
			try {
				mSensorMediator.connect();

			} catch (DaemonConnectionException e) {
				return;
			}
		} else {
			int state = Daemon.STATE_DISCOVERY_UNINITIALIZED;

			try {
				state = mSensorMediator.getDiscoveryState();
			} catch (BadReplyException e) {
				return;
			} catch (DaemonConnectionException e) {

				return;
			}
			if (state == Daemon.STATE_DISCOVERY_DONE) {
			}
		}
	}

	public void stopMed() {
		try {
			mSensorMediator.stopSensing();
			mSensorMediator.disconnect();
		} catch (DaemonConnectionException e) {
			return;
		}
	}

	public void stopSensing() {

		/*
		 * Before stop sensing just be sure that there are no subscribers to the
		 * streaming server
		 */
		if(MainActivity.WebSocket_Switch == true){
			boolean wsConnectionsFree 	= MainActivity.Get_WebSocket_Server().connections().isEmpty();
			boolean wsClientStreams 	= MainActivity.Get_WebSocket_Client().Is_Streaming();
			if ( wsConnectionsFree == false || wsClientStreams == true)
				return;
		}
		
		try {
			mSensorMediator.stopSensing();
		} catch (DaemonConnectionException e) {

		}
	}

	public void setSensorEnabled(String sensorUID, boolean enabled) {
		SensorInfo mSensor;
		mSensor = mSensorMediator.getMatchingSensor(sensorUID); // "fr.inria.arles.sensor.soundAmplitude"
																// ,
																// "fr.inria.arles.sensor.location"

		/*
		 * COMMENT AUTHOR: Georgios Mathioudakis What is the
		 * soundEnabled/locationEnabled checking for?
		 */
		if (mSensor != null && (soundEnabled || locationEnabled)) {
			if (enabled) {
				Log.i(TAG, "Setting sensor " + sensorUID + " as enabled");
				mSensorMediator.setSensorEnabled(mSensor, true);
			} else {
				Log.i(TAG, "Setting sensor " + sensorUID + " as disabled");
				mSensorMediator.setSensorEnabled(mSensor, false);
			}
		}
	}

	private void handleLocationResponse(SensorResponse sensorResponse) {
		DecimalFormat mLocDecFmt = new DecimalFormat("#0.0000000");
		DecimalFormat mDecFormat = new DecimalFormat("#0.000");

		String Lat = mLocDecFmt
				.format(((DefaultDouble2dLocationData) sensorResponse.payload)
						.getX());
		String Lon = mLocDecFmt
				.format(((DefaultDouble2dLocationData) sensorResponse.payload)
						.getY());
		String Time = mDecFormat.format(sensorResponse.payload.timestamp);
		Log.i(TAG, "Got location Response: Lat: " + Lat + " Lon: " + Lon);
	}

	private void handleSoundResponse(SensorResponse sensorResponse) {
		DecimalFormat mDecFormat = new DecimalFormat("#0.000");

		double noiseValue = ((DefaultDoubleNoiseLevelData) sensorResponse.payload)
				.getValue();
		String dB = mDecFormat.format(noiseValue);
		String Time = mDecFormat.format(sensorResponse.payload.timestamp);
		Log.i(TAG, "Got sound Response: dB: " + dB);
	}

	public void senseNow(ResponseListener resp) {
		Log.i(TAG, "senseNow()");
		responseListener = resp;
		try {
			mSensorMediator.senseImmediately(10000);
		} catch (ArgumentOutOfRangeException e) {
			Log.e(TAG, "Exception onSensorsChanged(): " + e.toString());
		}
	}

	/*
	 * Added on 4/2/2013 to support the streaming feature Author: Georgios
	 * Mathioudakis (georgios.mathioudakis@inria.fr)
	 */
	public void startSensing(int rate) {
		Log.i(TAG, "startSensing()");
		
		if(_streamListener == null)
			_streamListener = new StreamingResponseListener();
		
		try {
			mSensorMediator.sensePeriodically(rate, 10000);
		} catch (ArgumentOutOfRangeException e) {
			Log.e(TAG, "Exception onSensorsChanged(): " + e.toString());
		}
	}

	/**
	 * @return A boolean value indicating if the Sensor Mediator is currently
	 *         sensing
	 */
	public boolean IsSensing() {
		boolean isSensing = false;
		try {
			isSensing = mSensorMediator.isSensing();
		} catch (BadReplyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DaemonConnectionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return isSensing;
	}
	
}
