package fr.inria.arles.thinglib.RESTAccess;

import java.text.DecimalFormat;

import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.resource.Get;
import org.restlet.resource.ServerResource;

import fr.inria.arles.thinglib.datatypes.DefaultDouble2dLocationData;
import fr.inria.arles.thinglib.datatypes.DefaultDouble4dLocationData;
import fr.inria.arles.thinglib.datatypes.Double4dData;
import fr.inria.arles.thinglib.devices.SensorResponse;

import android.util.Log;

public class ExtendedLocationRestletResource extends ServerResource implements ResponseListener {

	private static final String TAG = "LocationRestletResource";
	private String reply = "";
	private static SensorResponse lastLocationResponse;
	
	private static final double BASE_LAT = 37.97153217698418;
	private static final double BASE_LON = 23.725922012206585;
	
	@Get  
	public String toString() {  
		Log.e(TAG, "Received GET Request");
		
		// MediatorService.getController().setSensorEnabled("fr.inria.arles.sensor.location", true);
		MediatorService.getController().senseNow(this);
		try {
		   Log.i(TAG, "wait()");
		   synchronized(TAG) {
			   TAG.wait(10000);
		   }
		} catch (InterruptedException e) {
			Log.e(TAG, "exception: "+e.toString());
		}
		// MediatorService.getController().setSensorEnabled("fr.inria.arles.sensor.location", false);
		Log.i(TAG, "unstuck from wait(). reply len: "+reply.length());
		
		//reply = createDummyLocation(BASE_LAT, BASE_LON);
		return reply;  
	}

	@Override
	public void onNewResponse(Iterable<SensorResponse> sensorResponses) {
		
		Log.i(TAG, "Received response");
		boolean locationRespRX = false;
		for(SensorResponse response : sensorResponses) {
			Log.i(TAG, "New response from: "+response.sensorInfo.getUid());
			if (response.sensorInfo.implementsInterface("fr.inria.arles.sensor.location")) {
				locationRespRX = true;
				handleLocationResponse(response);
				lastLocationResponse = response;
			}
		}
		
		
		/* TODO: In some cases (probably due to a race condition) the location
		 * responses do not contain a location measurement. If that happens,
		 * return the last known location value;
		 */
		
		if(!locationRespRX && lastLocationResponse != null) {	 
			Log.i(TAG, "Did not receive new location data. Using cached last Location response.");
			handleLocationResponse(lastLocationResponse);
		}
		Log.i(TAG, "reply len: "+reply.length());
		Log.i(TAG, "Notify()");
		synchronized(TAG) {
			TAG.notifyAll();
		}
		MediatorService.getController().stopSensing();
	}
	
	private void handleLocationResponse(SensorResponse sensorResponse) {
		
		String Lat 			= ((Double4dData)sensorResponse.payload).getX() + "";
		String Lon 			= ((Double4dData)sensorResponse.payload).getY() + "";
		String speed 		= "NOT DEFINED YET";
		String direction 	= "NOT DEFINED YET";
		
		JSONObject JSONdata 	= new JSONObject();
		JSONObject JSONreply 	= new JSONObject();
			
		try {
			JSONdata.put("latitude", Lat);
			JSONdata.put("longitude", Lon);
			JSONdata.put("speed", speed);
			JSONdata.put("direction", direction);
			JSONdata.put("timestamp", sensorResponse.payload.timestamp);
			JSONdata.put("dataType", sensorResponse.payload.getUid() );
			JSONreply.put("sensorDataResponse", JSONdata);
		} catch (JSONException e) {
			e.printStackTrace();
		}
		reply = JSONreply.toString();
		Log.i(TAG, "Got location Response: Lat: "+Lat+" Lon: "+Lon+" reply len: "+reply.length());
		
	}
	
	private String createDummyLocation(double latitude, double longitude){
		 // double density = 500;
		 // double density = 750;
		double density = 400;
		 
		 double dummyLat = latitude + (Math.random() / density) - 0.5 / density;
		 double dummyLng = longitude + (Math.random() / density) - 0.5 / density;
		 
		 double dummySpeed = Math.random() * 70;
		 double dummyDirection = Math.random();
		 
		 JSONObject JSONdata = new JSONObject();
		 JSONObject JSONreply = new JSONObject();
				
		 try {
			JSONdata.put("x", ""+dummyLat);
			JSONdata.put("y", ""+dummyLng);
			JSONdata.put("z", ""+dummySpeed);
			JSONdata.put("w", ""+dummyDirection);
			JSONdata.put("timestamp", "0");
			JSONdata.put("dataType",  "org.ow2.choreos.sensordata.2d-double.location");
			JSONreply.put("sensorDataResponse", JSONdata);
		 } catch (JSONException e) {
			e.printStackTrace();
		 }
		 return JSONreply.toString();
	}
}
