package fr.inria.arles.thinglib.RESTAccess;

import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.widget.TextView;

import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.resource.Get;
import org.restlet.resource.ServerResource;

import fr.inria.arles.thinglib.R;
import fr.inria.arles.thinglib.datatypes.Double4dData;
import fr.inria.arles.thinglib.devices.SensorResponse;

public class LocationRestletResource extends ServerResource implements
        ResponseListener {

    private static final String TAG = "LocationRestletResource";
    private static SensorResponse lastLocationResponse;
    private String reply = "";

    @Get
    public String toString() {
        Log.e(TAG, "Received GET Request");

        MediatorService.getController().senseNow(this);
        try {
            Log.i(TAG, "wait()");
            synchronized (TAG) {
                TAG.wait(10000);
            }
        } catch (InterruptedException e) {
            Log.e(TAG, "exception: " + e.toString());
        }
        Log.i(TAG, "unstuck from wait(). reply len: " + reply.length());
        return reply;
    }

    @Override
    public void onNewResponse(Iterable<SensorResponse> sensorResponses) {

        Log.i(TAG, "Received response");
        boolean locationRespRX = false;
        for (SensorResponse response : sensorResponses) {
            Log.i(TAG, "New response from: " + response.sensorInfo.getUid());
            if (response.sensorInfo
                    .implementsInterface("fr.inria.arles.sensor.location")) {
                locationRespRX = true;
                handleLocationResponse(response);
                lastLocationResponse = response;
                writeToLog(response.payload.toString());
            }
        }

		/*
         * TODO: In some cases (probably due to a race condition) the location
		 * responses do not contain a location measurement. If that happens,
		 * return the last known location value;
		 */

        if (!locationRespRX && lastLocationResponse != null) {
            Log.i(TAG,
                    "Did not receive new location data. Using cached last Location response.");
            handleLocationResponse(lastLocationResponse);
        }
        Log.i(TAG, "reply len: " + reply.length());
        Log.i(TAG, "Notify()");
        synchronized (TAG) {
            TAG.notifyAll();
        }
        MediatorService.getController().stopSensing();

    }

    private void handleLocationResponse(SensorResponse sensorResponse) {
        String Lat = ((Double4dData) sensorResponse.payload).getX() + "";
        String Lon = ((Double4dData) sensorResponse.payload).getY() + "";

        JSONObject JSONdata = new JSONObject();
        JSONObject JSONreply = new JSONObject();

        try {
            JSONdata.put("latitude", Lat);
            JSONdata.put("longitude", Lon);
            JSONdata.put("timestamp", sensorResponse.payload.timestamp);
            JSONdata.put("dataType", sensorResponse.payload.getUid());
            JSONreply.put("sensorDataResponse", JSONdata);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        reply = JSONreply.toString();
        Log.i(TAG, "Got location Response: Lat: " + Lat + " Lon: " + Lon
                + " reply len: " + reply.length());
    }

    private String createDummyLocation(double latitude, double longitude) {
        // double density = 500;
        // double density = 750;
        double density = 400;

        double dummyLat = latitude + (Math.random() / density) - 0.5 / density;
        double dummyLng = longitude + (Math.random() / density) - 0.5 / density;

        JSONObject JSONdata = new JSONObject();
        JSONObject JSONreply = new JSONObject();

        try {
            JSONdata.put("x", "" + dummyLat);
            JSONdata.put("y", "" + dummyLng);
            JSONdata.put("timestamp", "0");
            JSONdata.put("dataType",
                    "org.ow2.choreos.sensordata.2d-double.location");
            JSONreply.put("sensorDataResponse", JSONdata);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return JSONreply.toString();
    }

    private void writeToLog(final String text) {
        MainActivity.mainActivity.runOnUiThread(new Runnable() {

            @Override
            public void run() {
                TextView log = (TextView) MainActivity.mainActivity.findViewById(R.id._TV_Log);
                log.setMovementMethod(new ScrollingMovementMethod());
                log.setText("LOCATION - " + text + "\n" + log.getText());
            }
        });
    }
}
