//
//  MainActivity lass
//  SensorAccess
//
//  Created by Georgios Mathioudakis (georgios.mathioudakis@inria.fr) on 4/2/2014
//  Copyright (c) 2014 fr.inria.arles. All rights reserved.
//

package fr.inria.arles.thinglib.RESTAccess;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.PowerManager;
import android.os.PowerManager.WakeLock;
import android.preference.PreferenceManager;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.URL;
import java.net.UnknownHostException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.TimeUnit;

import fr.inria.arles.thinglib.R;
import fr.inria.arles.thinglib.websockets.Start_Websocket_Service;
import fr.inria.arles.thinglib.websockets.Stop_Websocket_Service;
import fr.inria.arles.thinglib.websockets.WebSocket_Client;
import fr.inria.arles.thinglib.websockets.WebSocket_Server;

public class MainActivity extends FragmentActivity implements LocationListener {

    public static Context AppBaseContext;
    public static Activity mainActivity;
    public static String UNIQUE_ID;
    public static ProgressDialog ringProgressDialog;
    public static boolean SensorsEnabled = false;
    public static boolean ServicesWasRegistered = false;
    public static boolean WebSocket_Switch = false;
    public static int WebSocket_Proxy_Status = 0;
    public static Location myLocation = null;
    public static String user_type = "user";
    /* WebSocket related vars */
    private static WebSocket_Server _ws_Server = null;
    private static WebSocket_Client _ws_Client = null;
    private static String _local_Address = "";
    private final String TAG = "MainActivity";
    /* Graphical Elements */
    private TextView _TV_StatusText;
    private TextView _TV_LinkText;
    private TextView _ET_DeviceID;
    private LinearLayout _LL_titleBar;
    private Button _BTN_EnableButton;
    private Button _BTN_DisableButton;
    private PowerManager _powerManager;
    private WakeLock _wakeLock;
    /* Location related vars */
    private LocationManager locationManager;
    private GoogleMap myMap;
    private HashMap<String, ArrayList<JSONObject>> registeredServices;

    /**
     * Converts a byte array to HEX string
     *
     * @param data Byte array
     * @return A string with the hex value
     */
    public static String convertToHex(byte[] data) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < data.length; i++) {
            int halfbyte = (data[i] >>> 4) & 0x0F;
            int two_halfs = 0;
            do {
                if ((0 <= halfbyte) && (halfbyte <= 9)) {
                    buf.append((char) ('0' + halfbyte));
                } else {
                    buf.append((char) ('a' + (halfbyte - 10)));
                }
                halfbyte = data[i] & 0x0F;
            } while (two_halfs++ < 1);
        }
        return buf.toString();
    }

    /**
     * Produces a SHA value of a string
     *
     * @param text text String value
     * @return A string generated by the hash function SHA-1
     * @throws NoSuchAlgorithmException
     * @throws UnsupportedEncodingException
     */
    public static String SHA1(String text) throws NoSuchAlgorithmException,
            UnsupportedEncodingException {
        MessageDigest md;
        md = MessageDigest.getInstance("SHA-1");
        byte[] sha1hash = new byte[40];
        md.update(text.getBytes("utf-8"), 0, text.length());
        sha1hash = md.digest();
        return convertToHex(sha1hash);
    }

    /**
     * @return The instance of the websocket server
     */
    public static WebSocket_Server Get_WebSocket_Server() {
        return _ws_Server;
    }

    /**
     * @return The instance of the websocket client
     */
    public static WebSocket_Client Get_WebSocket_Client() {
        return _ws_Client;
    }

    public static void stopRingDialog() {
        if (ringProgressDialog != null && ringProgressDialog.isShowing()) {
            ringProgressDialog.hide();
        }
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.home);

        initUiVars();                               //Init vars for UI elements
        updateValuesFromPreferences();              //Update Static Addresses class from user's prefs
        initLocationListener();                     //Init the location listener
        generateUniqueID();                         //Generate a Unique id for the app
        setStatus(MediatorService.isEnabled());     //Update UI about the status of the mediator
        startSchedulerService();                    //Init the service responsible for the async communication

        //Map init
        FragmentManager myFragmentManager = getSupportFragmentManager();
        SupportMapFragment myMapFragment = (SupportMapFragment) myFragmentManager.findFragmentById(R.id.map);
        myMap = myMapFragment.getMap();

        loadServices();

        _BTN_EnableButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
                launchRingDialog("Please wait...", "Registering local services...");
                if (!MediatorService.isEnabled()) {
                    startMediatorService();
                    setStatus(true);
                    SensorsEnabled = true;

                    if (haveConnectivity()) {

                        /*
                         Start the websocket server on port 8887
                         */
                        if (WebSocket_Switch == true) {
                            try {
                                _ws_Server = new WebSocket_Server(Static_Addresses.WS_SERVER_PORT);
                            } catch (UnknownHostException e) {
                                Log.e(TAG,
                                        "Unknown host exception on WebSocket server initilization");
                                e.printStackTrace();
                            }
                        }

                        /*
                         Create a new Websocket client that will be used on the Proxy-based communication.
                         To create the client safely ping the proxy server to check if there is one
                         If there is the Proxy should responde with 1 to indicate that the WS server is open
                         If there is no Proxy then no client should be instatiated
                         */
                        if (WebSocket_Switch == true) {
                            PingProxyWSServer();
                        }
                        SchedulerService.CreateNewThreadPoolExecutor();

                        if (WebSocket_Switch == true) {
                            try {
                                SchedulerService.getSchedExecutor().schedule(
                                        new Start_Websocket_Service(), 0,
                                        TimeUnit.MILLISECONDS);
                                Toast.makeText(
                                        getBaseContext(),
                                        "WebSocket server started", Toast.LENGTH_LONG)
                                        .show();
                            } catch (RejectedExecutionException e) {
                                Toast.makeText(
                                        getBaseContext(),
                                        "Start Websocket server error: "
                                                + e.getMessage(), Toast.LENGTH_LONG
                                )
                                        .show();
                                Log.e(TAG, "Restart exception: " + e.getMessage());
                            }
                        }

                        try {
                            SchedulerService.getSchedExecutor().scheduleAtFixedRate(
                                    new PollTask(), 500,
                                    SchedulerService.POLL_DELAY_MILLISEC,
                                    TimeUnit.MILLISECONDS);
                        } catch (RejectedExecutionException e) {
                            Toast.makeText(getBaseContext(),
                                    "Abort policy exception" + e.getMessage(),
                                    Toast.LENGTH_LONG).show();
                            Log.e(TAG, "Restart exception: " + e.getMessage());
                        }

                        try {
                            SchedulerService.getSchedExecutor().schedule(
                                    new RegisterTask(), 200,
                                    TimeUnit.MILLISECONDS);
                            ServicesWasRegistered = true;
                        } catch (RejectedExecutionException e) {
                            Toast.makeText(getBaseContext(),
                                    "Abort policy exception" + e.getMessage(),
                                    Toast.LENGTH_LONG).show();
                            Log.e(TAG, "Restart exception: " + e.getMessage());
                        }

                        /*
                         This is to update the location to the Registry manager so as to have an updated map at every moment
                         */
                        try {
                            SchedulerService.getSchedExecutor().scheduleAtFixedRate(
                                    new UpdateLocationToRegistry(), 60000,
                                    60000,
                                    TimeUnit.MILLISECONDS);
                        } catch (RejectedExecutionException e) {
                            Toast.makeText(getBaseContext(),
                                    "Abort policy exception" + e.getMessage(),
                                    Toast.LENGTH_LONG).show();
                            Log.e(TAG, "Restart exception: " + e.getMessage());
                        }

                        /*
                         This will make the CPU stay on, otherwise the polling
                         service will stop
                         */
                        _wakeLock.acquire();

                    }
                }
            }
        });

        _BTN_DisableButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View arg0) {
                if (MediatorService.isEnabled()) {

                    try {
                        SchedulerService.getSchedExecutor().schedule(
                                new UnRegisterTask(), 0, TimeUnit.MILLISECONDS);

                        if (WebSocket_Switch == true) {
                            SchedulerService.getSchedExecutor().schedule(
                                    new Stop_Websocket_Service(), 0,
                                    TimeUnit.MILLISECONDS);
                        }

                        SchedulerService.getSchedExecutor().shutdown();
                        if (!SchedulerService.getSchedExecutor().awaitTermination(3,
                                TimeUnit.SECONDS)) {
                            List<Runnable> runList = SchedulerService
                                    .getSchedExecutor().shutdownNow();
                            Toast.makeText(getBaseContext(),
                                    "Runnables: " + runList.size(),
                                    Toast.LENGTH_LONG).show();
                            ServicesWasRegistered = false;
                        }
                    } catch (InterruptedException e) {
                        SchedulerService.getSchedExecutor().shutdownNow();
                        Thread.currentThread().interrupt();
                        e.printStackTrace();
                    } finally {
                        SchedulerService.DestroyThreadPoolExecutor();
                    }

                    stopMediatorService();
                    setStatus(false);
                    SensorsEnabled = false;

                    /*
                     Close the websocket connection
                     */
                    if (WebSocket_Switch == true) {
                        if (_ws_Client != null) {
                            _ws_Client.CloseConnection();
                        }
                    }

                    if (_wakeLock.isHeld()) {
                        _wakeLock.release();
                    }
                }

            }
        });
    }

    @Override
    protected void onStart() {
        super.onStart();
    }

    @Override
    public void onResume() {
        super.onResume();
        try {
            /*
             Search for the device's IP address
             */
            Enumeration<NetworkInterface> en = NetworkInterface
                    .getNetworkInterfaces();
            while (en.hasMoreElements()) {
                NetworkInterface iface = en.nextElement();
                if (iface.isUp()
                        && !iface.isLoopback()
                        && (iface.getDisplayName().toLowerCase()
                        .contains("eth0") || iface.getDisplayName()
                        .toLowerCase().contains("wlan"))) {
                    Enumeration<InetAddress> en_addr = iface.getInetAddresses();
                    while (en_addr.hasMoreElements()) {
                        _local_Address = en_addr.nextElement().getHostAddress();
                        Log.i(TAG, iface.getDisplayName() + " : "
                                + _local_Address);
                        _TV_LinkText.setText("http://" + _local_Address + ":8182");
                    }
                }
            }
        } catch (SocketException e) {
            Log.e(TAG, "Socket exception: " + e.toString());
        }
    }

    @Override
    public void onStop() {

        if (locationManager != null) {
            locationManager.removeUpdates(this);
        }

        super.onStop();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        /*
         Inflate the menu; this adds items to the action bar if it is present.
         */
        getMenuInflater().inflate(R.menu.main_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
            case R.id.settings_option:
                startActivity(new Intent(this, SettingsActivity.class));
                return true;
            case R.id.refresh_option:
                loadServices();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
    }

    @Override
    public void onLocationChanged(Location location) {
        myLocation = location;
    }

    @Override
    public void onProviderDisabled(String provider) {
        // TODO Auto-generated method stub

    }

    @Override
    public void onProviderEnabled(String provider) {
        // TODO Auto-generated method stub

    }

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras) {
        // TODO Auto-generated method stub
    }

    /**
     * Starts the mediator service for gathering the sensed data
     */
    private void startMediatorService() {
        startService(new Intent(this, MediatorService.class));
    }

    /**
     * Stops the mediator service
     */
    private void stopMediatorService() {
        stopService(new Intent(this, MediatorService.class));
    }

    /**
     * TODO: UPDATE THIS
     */
    private void startSchedulerService() {
        startService(new Intent(this, SchedulerService.class));
    }

    /**
     * Controls the appearance & text of the visual component (label) that holds the
     * status of the sensing
     *
     * @param enabled True for enabled mediator False for disabled
     */
    public void setStatus(boolean enabled) {
        if (enabled) {
            runOnUiThread(new Runnable() {
                public void run() {
                    _TV_StatusText.setText("Enabled");
                    _LL_titleBar.setBackgroundColor(Color.argb(255, 153, 159, 16));
                    _BTN_EnableButton.setBackgroundColor(Color.argb(255, 90, 90, 90));
                    _BTN_DisableButton.setBackgroundColor(Color.argb(255, 0, 115, 210));
                }
            });
        } else {
            runOnUiThread(new Runnable() {
                public void run() {
                    _TV_StatusText.setText("Disabled");
                    _LL_titleBar.setBackgroundColor(Color.argb(255, 217, 29, 22));
                    _BTN_EnableButton.setBackgroundColor(Color.argb(255, 0, 115, 210));
                    _BTN_DisableButton.setBackgroundColor(Color.argb(255, 90, 90, 90));
                }
            });

        }
    }

    /**
     * Checks if device has Internet connectivity
     *
     * @return True if there is connectivity, False if not
     */
    private boolean haveConnectivity() {
        ConnectivityManager conMgr = (ConnectivityManager) this
                .getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo i = conMgr.getActiveNetworkInfo();
        if (i == null) {
            return false;
        }
        if (!i.isConnected()) {
            return false;
        }
        if (!i.isAvailable()) {
            return false;
        }
        return true;
    }

    /**
     * Updates Servers Addresses according to the current settings defined on the
     * preferences screen
     */
    public void updateValuesFromPreferences() {
        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
        String Registry_Host = "";
        String Proxy_Host = "";
        try {
            if (prefs.contains("RegistryIP")) {
                Registry_Host = (String) prefs.getString(
                        "RegistryIP", Static_Addresses.REGISTRY_HOST);
                Static_Addresses.Set_RegistryHost(Registry_Host);
            }
            if (prefs.contains("ProxyIP")) {
                Proxy_Host = (String) prefs.getString("ProxyIP",
                        Static_Addresses.PROXY_HOST);
                Static_Addresses.Set_ProxyHost(Proxy_Host);
            }
            if (prefs.contains("WebsocketActive")) {
                boolean WebSocket_Switch = prefs.getBoolean(
                        "WebsocketActive", false);
                MainActivity.WebSocket_Switch = WebSocket_Switch;
            }
            if (prefs.contains("userTypeSelection")) {
                user_type = (String) prefs.getString("userTypeSelection", "user");
            }
        } catch (ClassCastException e) {
            Log.d("SETTINGS", e.getMessage());
        }
    }

    /**
     * @return The status of the WebSocket server lying in the Proxy server
     */
    public void PingProxyWSServer() {
        Object temp = null;

        new AsyncTask<Object, Object, Integer>() {
            int inner_status = 0;

            protected void onPostExecute(Integer result) {
                MainActivity.WebSocket_Proxy_Status = result;

                if (WebSocket_Proxy_Status == 1) {
                    _ws_Client = new WebSocket_Client();
                }
            }

            @Override
            protected Integer doInBackground(Object... params) {
                StringBuilder builder = new StringBuilder();
                try {
                    HttpClient client = new DefaultHttpClient();
                    HttpGet httpGet = new HttpGet(Static_Addresses.PROXY_URL + "startws");
                    HttpResponse response = client.execute(httpGet);
                    StatusLine statusLine = response.getStatusLine();
                    int statusCode = statusLine.getStatusCode();
                    if (statusCode == 200) {
                        HttpEntity entity = response.getEntity();
                        InputStream content = entity.getContent();
                        BufferedReader reader = new BufferedReader(
                                new InputStreamReader(content));
                        String line;
                        while ((line = reader.readLine()) != null) {
                            builder.append(line);
                        }
                        inner_status = Integer.parseInt(builder.toString());
                    } else {
                        Log.e(TAG, "Failed to GET to find Proxy");

                    }
                } catch (Exception e) {
                    Log.e(TAG, "Error in getting Proxy" + e.getMessage());
                }
                return inner_status;
            }
        }.execute(temp);
    }

    /**
     * Create a unique ID for the device based on the Android-ID
     * A 64-bit number (as a hex string) that is randomly generated when the user
     * first sets up the device and should remain constant for the lifetime of
     * the user's device.
     */
    private void generateUniqueID() {
        if (UNIQUE_ID == null) {
            String androidId = "" + android.provider.Settings.Secure.getString(getContentResolver(), android.provider.Settings.Secure.ANDROID_ID);
            try {
                UNIQUE_ID = SHA1(androidId);
                _ET_DeviceID.setText(UNIQUE_ID);
                Log.i(TAG, "UNIQUE_ID = " + UNIQUE_ID);
            } catch (NoSuchAlgorithmException e) {
                Log.e(TAG, e.toString());
            } catch (UnsupportedEncodingException e) {
                Log.e(TAG, e.toString());
            }
        }
    }

    private void initUiVars() {
        //Initialize visual elements
        _TV_StatusText = (TextView) findViewById(R.id.TV_Status);
        _TV_LinkText = (TextView) findViewById(R.id.TV_Link);
        _ET_DeviceID = (TextView) findViewById(R.id.ET_DeviceID);
        _LL_titleBar = (LinearLayout) findViewById(R.id.titleBar);
        _BTN_EnableButton = (Button) findViewById(R.id.BTN_Enable);
        _BTN_DisableButton = (Button) findViewById(R.id.BTN_Disable);

        //Initialize main related vars to allow access from other classes
        mainActivity = this;
        AppBaseContext = this.getBaseContext();

        /*
         Prevent the phone from sleeping when the screen dims.
         The reason is that during stream sensing we want the device's sensors
         to always sense without interuption. Normaly screen dim stops all
         services.
         */
        _powerManager = (PowerManager) getSystemService(Context.POWER_SERVICE);
        _wakeLock = _powerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, "Device wakelook");
    }

    private void initLocationListener() {
        //Find user's location. This will be needed to update registry with the current one
        locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
        Criteria criteria = new Criteria();
        String provider = locationManager.getBestProvider(criteria, true);

        /*
         Create default(fallback) location
         */
        Location location = new Location("myProvider");
        location.setLatitude(0);
        location.setLongitude(0);

        if (provider != null) {
            location = locationManager.getLastKnownLocation(provider);
        }

        /*
         Initialize the location fields
         */
        if (location != null) {
            onLocationChanged(location);
        }
    }

    public void launchRingDialog(String title, String msg) {
        ringProgressDialog = ProgressDialog.show(MainActivity.this, title, msg, true);
        ringProgressDialog.setCancelable(true);
    }

    public void loadServices() {
        new AsyncTask<Object, Object, JSONObject>() {

            private JSONObject myFoundServices = new JSONObject();

            protected void onPreExecute() {
                launchRingDialog("Please wait...", "Loading Services");
            }

            protected void onPostExecute(JSONObject result) {
                if (result == null) {
                    stopRingDialog();
                    return;
                } else {
                    JSONArray services;
                    registeredServices = new HashMap<String, ArrayList<JSONObject>>();
                    myMap.clear();
                    services = (JSONArray) result.get("value");
                    if(services == null) {
                        stopRingDialog();
                        return;
                    }
                    for (int i = 0; i < services.size(); i++) {
                        JSONObject service = null;
                        service = (JSONObject) services.get(i);
                        String deviceId = service.get("DeviceId").toString();
                        if(!registeredServices.containsKey(deviceId)){
                            ArrayList<JSONObject> newArray = new ArrayList<JSONObject>();
                            newArray.add(service);
                            registeredServices.put(deviceId,newArray);
                        } else {
                            registeredServices.get(deviceId).add(service);
                        }
                    }
                    for(ArrayList<JSONObject> servicesArray : registeredServices.values()){
                        double lat = Double.parseDouble(servicesArray.get(0).get("coordinateX").toString());
                        double lon = Double.parseDouble(servicesArray.get(0).get("coordinateY").toString());
                        LatLng tempServiceLatlng = new LatLng(lat, lon);
                        String titleString = "";
                        int count = 1;
                        for(JSONObject object : servicesArray){
                            titleString += count + ") " + object.get("name").toString() + "  ";
                            count++;
                        }
                        myMap.addMarker(new MarkerOptions()
                                        .position(tempServiceLatlng)
                                        .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_AZURE))
                                        .title("Services provided")
                                        .snippet(titleString)
                        );
                        myMap.animateCamera(CameraUpdateFactory.newLatLngZoom(tempServiceLatlng,12.0f));
                    }
                    stopRingDialog();
                }
            }

            @Override
            protected JSONObject doInBackground(Object... params) {
                String url = Static_Addresses.REGISTRY_URL + "execute_lookup_query?global_region=Paris&concept=noiselevel&concept=location";
                String responseString = "";
                try {
                    URL urlObj = new URL(url);
                    HttpURLConnection con = (HttpURLConnection) urlObj.openConnection();
                    con.setRequestMethod("GET");
                    int responseCode = con.getResponseCode();
                    BufferedReader in = new BufferedReader(
                            new InputStreamReader(con.getInputStream()));
                    String inputLine;
                    StringBuffer response = new StringBuffer();

                    while ((inputLine = in.readLine()) != null) {
                        response.append(inputLine);
                    }
                    responseString = response.toString();
                    Log.d(TAG, responseString);
                    in.close();
                    con.disconnect();
                } catch (MalformedURLException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                }

                JSONObject json = new JSONObject();
                try {
                    json = (JSONObject) new JSONParser().parse(responseString);
                } catch (ParseException e) {
                    e.printStackTrace();
                }
                myFoundServices = json;
                return myFoundServices;
            }
        }.execute(null);
    }
}
