package fr.inria.arles.thinglib.RESTAccess;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONObject;

import android.R;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.support.v4.app.NotificationCompat;
import android.util.Log;

public class PollTask implements Runnable {

	private static final String TAG = "PollTask";
	private String _localIP = "localhost:8182/";

	@Override
	public void run() {

		// TODO Auto-generated method stub
		String JSONpendingRequests = getPendingRequests();
		if (JSONpendingRequests == null)
			return;

		try {
			JSONObject reqObj = new JSONObject(JSONpendingRequests);
			JSONArray req_list = reqObj.getJSONArray("pendingRequests");
			for (int i = 0; i < req_list.length(); i++) {
				JSONObject jsonObject = req_list.getJSONObject(i);
				String request = jsonObject.getString("serviceName");
				Log.e(TAG, "REQUEST: " + request);
				String JSONreply = null;
				if (request.equals("getnoiselevel")) {
					JSONreply = getLocalNoiseLevel();
					Log.i(TAG, getLocalNoiseLevel());
				} else if (request.equals("getlocation")) {
					JSONreply = getLocalLocation();
					Log.i(TAG, JSONreply);
				} else if (request.equals("pushmessage")) {
					JSONreply = "{}";
					String message = jsonObject.getString("message");
					pushMessage(message);
					Log.i(TAG, "Message Pushed: " + message);
				} else {
					JSONreply = "{}";
					Log.i(TAG, "Unknown request");
				}
				try {
					sendReply(JSONreply, request);
				} catch (UnsupportedEncodingException e) {
					Log.e("TAG", "EXCEPTION : " + e.toString());
				}

			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public String getPendingRequests() {
		StringBuilder builder = new StringBuilder();
		try {
			HttpClient client = new DefaultHttpClient();
			HttpGet httpGet = new HttpGet(Static_Addresses.PROXY_URL
					+ "pendingrequests/" + MainActivity.UNIQUE_ID);

			HttpResponse response = client.execute(httpGet);
			StatusLine statusLine = response.getStatusLine();
			int statusCode = statusLine.getStatusCode();
			if (statusCode == 200) {
				HttpEntity entity = response.getEntity();
				InputStream content = entity.getContent();
				BufferedReader reader = new BufferedReader(
						new InputStreamReader(content));
				String line;
				while ((line = reader.readLine()) != null) {
					builder.append(line);
				}
			} else {
				Log.e(TAG, "Failed to GET Pending requests");
			}
		} catch (Exception e) {
			Log.e(TAG,
					"Error in getting Pending requests --> " + e.getMessage());
		}
		return builder.toString();
	}

	/**
	 * Gets a Push Message request and prints the message using the UI's thread
	 * 
	 * @param msg
	 */
	private void pushMessage(String msg) {
		ShowMessage_UITask uiTask = new ShowMessage_UITask();
		uiTask.setData(msg);
		MainActivity.mainActivity.runOnUiThread(uiTask);
	}

	private String getLocalLocation() {
		StringBuilder builder = new StringBuilder();
		HttpClient client = new DefaultHttpClient();
		HttpGet httpGet = new HttpGet("http://" + _localIP + "getlocation");

		try {
			HttpResponse response = client.execute(httpGet);
			StatusLine statusLine = response.getStatusLine();
			int statusCode = statusLine.getStatusCode();
			if (statusCode == 200) {
				HttpEntity entity = response.getEntity();
				InputStream content = entity.getContent();
				BufferedReader reader = new BufferedReader(
						new InputStreamReader(content));
				String line;
				while ((line = reader.readLine()) != null) {
					builder.append(line);
				}
			} else {
				Log.e(TAG, "Failed to GET Pending requests");
			}
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return builder.toString();
	}

	private String getLocalNoiseLevel() {
		StringBuilder builder = new StringBuilder();
		HttpClient client = new DefaultHttpClient();
		HttpGet httpGet = new HttpGet("http://" + _localIP + "getnoiselevel");

		try {
			HttpResponse response = client.execute(httpGet);
			StatusLine statusLine = response.getStatusLine();
			int statusCode = statusLine.getStatusCode();
			if (statusCode == 200) {
				HttpEntity entity = response.getEntity();
				InputStream content = entity.getContent();
				BufferedReader reader = new BufferedReader(
						new InputStreamReader(content));
				String line;
				while ((line = reader.readLine()) != null) {
					builder.append(line);
				}
			} else {
				Log.e(TAG, "Failed to GET Pending requests");
			}
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return builder.toString();
	}

	public void sendReply(String reply, String serviceName)
			throws UnsupportedEncodingException {
		HttpClient client = new DefaultHttpClient();
		String uri = Static_Addresses.PROXY_URL + "sendresponse/"
				+ MainActivity.UNIQUE_ID + "/" + serviceName;

		HttpPost httpPost = new HttpPost(uri);
		StringEntity stringEntity = new StringEntity(reply);
		httpPost.setEntity(stringEntity);
		httpPost.setHeader("Accept", "application/json");
		httpPost.setHeader("Content-type", "application/json");

		try {
			HttpResponse response = client.execute(httpPost);
			StatusLine statusLine = response.getStatusLine();
			int statusCode = statusLine.getStatusCode();
			if (statusCode == 200) {
				Log.e(TAG, "POSTED reply");
			}
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
