package fr.inria.arles.thinglib.RESTAccess;

import java.text.DecimalFormat;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.resource.Get;
import org.restlet.resource.ServerResource;

import fr.inria.arles.thinglib.R;
import fr.inria.arles.thinglib.datatypes.DefaultDoubleNoiseLevelData;
import fr.inria.arles.thinglib.devices.SensorResponse;

import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.widget.TextView;

public class SoundRestletResource extends ServerResource implements ResponseListener {
	
	private static final String TAG = "SoundRestletResource";
	private String reply = "";
	
	@Get  
	public String toString() {  
		Log.e(TAG, "Received GET Request");
		// MediatorService.getController().setSensorEnabled("fr.inria.arles.sensor.soundAmplitude", true);
		MediatorService.getController().senseNow(this);
		try {
		   Log.i(TAG, "wait()");
		   synchronized(TAG) {
			   TAG.wait(10000);
		   }
		} catch (InterruptedException e) {
			Log.e(TAG, "exception: "+e.toString());
		}
		// MediatorService.getController().setSensorEnabled("fr.inria.arles.sensor.soundAmplitude", false);
		Log.i(TAG, "unstuck from wait(). reply len: "+reply.length());
		return reply;  
	}

	@Override
	public void onNewResponse(Iterable<SensorResponse> sensorResponses) {
		Log.i(TAG, "Received response");
		for(SensorResponse response : sensorResponses) {
			Log.i(TAG, "New response from: "+response.sensorInfo.getUid());
			if (response.sensorInfo.implementsInterface("fr.inria.arles.sensor.soundAmplitude")) {
				handleSoundResponse(response);
                writeToLog(response.payload.toString());
            }
		}
		
		Log.i(TAG, "reply len: "+reply.length());
		Log.i(TAG, "Notify()");
		synchronized(TAG) {
			TAG.notifyAll();
		}
		MediatorService.getController().stopSensing();
	}

	
	private void handleSoundResponse(SensorResponse sensorResponse) {
		DecimalFormat mDecFormat = new DecimalFormat("#0.000");
		
		double noiseValue = ((DefaultDoubleNoiseLevelData) sensorResponse.payload).getNoiseLevel();
		String dB = mDecFormat.format(noiseValue);
		
		JSONObject JSONdata = new JSONObject();
		JSONObject JSONreply = new JSONObject();
			
		try {
			JSONdata.put("value", dB);
			JSONdata.put("timestamp", sensorResponse.payload.timestamp);
			JSONdata.put("dataType", sensorResponse.payload.getUid() );
			JSONreply.put("sensorDataResponse", JSONdata);
		} catch (JSONException e) {
			e.printStackTrace();
		}
		
		reply = JSONreply.toString();
		Log.i(TAG, "Got sound Response: dB: "+dB+" reply len:"+reply.length());
	}

    private void writeToLog(final String text) {
        MainActivity.mainActivity.runOnUiThread(new Runnable() {

            @Override
            public void run() {
                TextView log = (TextView) MainActivity.mainActivity.findViewById(R.id._TV_Log);
                log.setMovementMethod(new ScrollingMovementMethod());
                log.setText("SOUND - " + text + "\n" + log.getText());
            }
        });
    }
}
