/******************************************************************
Author:  		Georgios Mathioudakis (georgios.mathioudakis@inria.fr)
Project: 		AndroidSensorAccessDemo
package: 		fr.inria.arles.thinglib.exampleapps.RESTAccess
Updated: 		
Status:  		Working draft
Description: 	This generic response listener is used to retrieve the data from sensors in order to be sent through streaming websockets
******************************************************************/

package fr.inria.arles.thinglib.RESTAccess;

import org.json.JSONException;
import org.json.JSONObject;
import android.util.Log;
import fr.inria.arles.thinglib.datatypes.Double4dData;
import fr.inria.arles.thinglib.devices.SensorResponse;
import fr.inria.arles.thinglib.websockets.WebSocket_Client;
import fr.inria.arles.thinglib.websockets.WebSocket_Server;

public class StreamingResponseListener implements ResponseListener{

	private String TAG = "StreamingResponseListener";
	private static SensorResponse lastLocationResponse;

	public StreamingResponseListener() {
	}
	
	@Override
	public void onNewResponse(Iterable<SensorResponse> sensorResponses) {
		for (SensorResponse response : sensorResponses) {
			Log.i(TAG, "New response from: " + response.sensorInfo.getUid());

			if (response.sensorInfo.getInterface() == "fr.inria.arles.sensor.location") {
				JSONObject jsonReply = handleLocationResponse(response, "fr.inria.arles.sensor.location");
				lastLocationResponse = response;
				if(MainActivity.Get_WebSocket_Server() != null)
					MainActivity.Get_WebSocket_Server().SendResponce(jsonReply);
				
				if(MainActivity.Get_WebSocket_Client() != null)
					MainActivity.Get_WebSocket_Client().SendMessageToServer(jsonReply);
			}	
		}
	}
	
	private JSONObject handleLocationResponse(SensorResponse sensorResponse, String serviceInterface) {
		String Lat = ((Double4dData) sensorResponse.payload).getX() + "";
		String Lon = ((Double4dData) sensorResponse.payload).getY() + "";

		JSONObject JSONdata = new JSONObject();
		JSONObject JSONreply = new JSONObject();

		try {
			JSONdata.put("deviceID", MainActivity.UNIQUE_ID);
			JSONdata.put("serviceName", serviceInterface);
			JSONdata.put("latitude", Lat);
			JSONdata.put("longitude", Lon);
			JSONdata.put("timestamp", sensorResponse.payload.timestamp);
			JSONdata.put("dataType", sensorResponse.payload.getUid());
			JSONreply.put("sensorDataResponse", JSONdata);
		} catch (JSONException e) {
			e.printStackTrace();
		}
		String reply = JSONreply.toString();

		Log.i(TAG, "Got location Response: Lat: " + Lat + " Lon: " + Lon
				+ " reply len: " + reply.length());
		
		return JSONreply;
	}

}
