package fr.inria.arles.thinglib.core;

import fr.inria.arles.thinglib.devices.SensorInfo;
import fr.inria.arles.thinglib.devices.SensorResponse;

public abstract class MediatorListener extends Thread {

	/**
	 * Override this to be informed whenever the Mediator connects/disconnects
	 * to/from the Daemon.
	 *
	 * @param newState An integer describing the connection state.
	 */
	public abstract void onConnectionStateChanged(int newState);

	/**
	 * Override this to be informed whenever the Daemon 
	 * to/from the Daemon.
	 *
	 * @param newState An integer describing the discovery state. 
	 */
	public abstract void onDiscoveryStateChanged(int newState);

	/**
	 * Override this to be informed whenever the Mediator changes state.
	 * This method is already asynchronous from the Mediator.
	 *
	 * @param newState The new state of the Mediator.
	 */
	public abstract void onSensingStateChanged(int newState);
	
	/**
	 * Override this to be informed whenever new sensors are discovered or old
	 * sensors are removed.  This method is already asynchronous from the
	 * Mediator.
	 *
	 * @param sensors The SensorInfos for the new set of sensors that this 
	 * Mediator can manage.
	 */
	public abstract void onSensorsChanged(Iterable<SensorInfo> sensors);

	/**
	 * Override this to be informed whenever the Mediator collects a new
	 * set of responses.  This method is already asynchronous from the
	 * Mediator.
	 * 
	 *
	 * @param sensorResponses The set responses that the Mediator has
	 * collected in this latest sensing task.
	 */
	public abstract void onNewData(Iterable<SensorResponse> sensorResponses);
}

