package fr.inria.arles.thinglib.core;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.StreamCorruptedException;

public class Messaging {

	// special addresses (always negative)
	public static final int DAEMON_ADDRESS                     = -1;
	public static final int UNREGISTERED_ADDRESS               = -2;
	
	// message codes
	// (Make sure that Messaging.MSG_*_REPLY is always equal to Messaging.MSG_* + 1, and are also the only odd numbers)
	public static final int MSG_DUMMY                          = 0;
	public static final int MSG_NEW_DATA                       = 2;
	public static final int MSG_NEW_COMMAND                    = 4;
	public static final int MSG_DISCOVERY_STATE_CHANGED        = 6;
	public static final int MSG_SENSING_STATE_CHANGED          = 8;
	public static final int MSG_SENSORS_CHANGED                = 10;
	public static final int MSG_REGISTER_CLIENT                = 16;
	public static final int MSG_REGISTER_CLIENT_REPLY          = MSG_REGISTER_CLIENT + 1;
	public static final int MSG_DEREGISTER_CLIENT              = 18;
	public static final int MSG_REGISTER_AUTOSTART             = 20;
	public static final int MSG_REGISTER_AUTOSTART_REPLY       = MSG_REGISTER_AUTOSTART + 1;
	public static final int MSG_DEREGISTER_AUTOSTART           = 22;
	public static final int MSG_GET_DISCOVERY_STATE            = 128;
	public static final int MSG_GET_DISCOVERY_STATE_REPLY      = MSG_GET_DISCOVERY_STATE + 1;
	public static final int MSG_DISCOVER_SENSORS               = 130;
	public static final int MSG_GET_SENSOR_INFOS               = 132;
	public static final int MSG_GET_SENSOR_INFOS_REPLY         = MSG_GET_SENSOR_INFOS + 1;
	public static final int MSG_GET_SENSOR_BY_UID              = 134;
	public static final int MSG_GET_SENSOR_BY_UID_REPLY        = MSG_GET_SENSOR_BY_UID + 1;
	public static final int MSG_GET_MATCHING_SENSOR            = 136;
	public static final int MSG_GET_MATCHING_SENSOR_REPLY      = MSG_GET_MATCHING_SENSOR + 1;
	public static final int MSG_GET_MATCHING_SENSORS           = 138;
	public static final int MSG_GET_MATCHING_SENSORS_REPLY     = MSG_GET_MATCHING_SENSORS + 1;
	public static final int MSG_SET_OUTER_SCOPE                = 140;
	public static final int MSG_IS_RUNNING                     = 142;
	public static final int MSG_IS_RUNNING_REPLY               = MSG_IS_RUNNING + 1;
	public static final int MSG_IS_SENSING                     = 144;
	public static final int MSG_IS_SENSING_REPLY               = MSG_IS_SENSING + 1;
	public static final int MSG_SENSE_IMMEDIATELY              = 146;
	public static final int MSG_SENSE_PERIODICALLY             = 148;
	public static final int MSG_SENSE_EVENT_BASED              = 150;
	public static final int MSG_STOP_SENSING                   = 152;
	public static final int MSG_ABORT_SENSING                  = 154;
	public static final int MSG_SET_SENSOR_ENABLED             = 156;
	public static final int MSG_SET_SENSOR_DISABLED            = 158;
	public static final int MSG_IS_SENSOR_ENABLED              = 160;
	public static final int MSG_IS_SENSOR_ENABLED_REPLY        = MSG_IS_SENSOR_ENABLED + 1;
	public static final int MSG_GET_SENSOR_RESPONSE            = 162;
	public static final int MSG_GET_SENSOR_RESPONSE_REPLY      = MSG_GET_SENSOR_RESPONSE + 1;
	public static final int MSG_GET_SENSOR_RESPONSES           = 164;
	public static final int MSG_GET_SENSOR_RESPONSES_REPLY     = MSG_GET_SENSOR_RESPONSES + 1;
	public static final int MSG_GET_ENABLED_SENSOR_INFOS       = 166;
	public static final int MSG_GET_ENABLED_SENSOR_INFOS_REPLY = MSG_GET_ENABLED_SENSOR_INFOS + 1;

	public static Object toObject(byte[] payload) 
	throws StreamCorruptedException, IOException, ClassNotFoundException {
		
		ByteArrayInputStream byteInputStream = new ByteArrayInputStream(payload);
        ObjectInputStream objectInputStream = new ObjectInputStream(byteInputStream);
		Object object = objectInputStream.readObject();
        objectInputStream.close();
        
        return object;
	}

	public static byte[] toBytes(Object object) 
	throws IOException {
		ByteArrayOutputStream byteOutputStream = new ByteArrayOutputStream();
        ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteOutputStream);
        objectOutputStream.writeObject(object);
        objectOutputStream.close();
		return byteOutputStream.toByteArray();
	}

	public static String toString(int sender, int destination, int type, byte[] payload) {
		return String.format("Messaging = {sender: %d, destination: %d, type: %d}", sender, destination, type);
	}
}

