package fr.inria.arles.thinglib.devices;

// TODO: explain that async sensors assume a thread is generated internally, so
// that the app (periscope) does not have to explicitly launch new threads.
/**
 * This abstract class should be subclassed when implementing a sensor that
 * produces data for each timestamp by sampling, internally, several times. One
 * example is a sensor that measures the average sound amplitude for the
 * past 2 seconds.
 */
public interface IntervalSensor extends Sensor {

	/**
	 * Specifies the frequency and number of samples that will make up the total
	 * sample set, from which statistics can be computed. For instance, if the
	 * period is 100ms and the number of samples is 20, then whenever
	 * Sensor.getData(handler) is called, this IntervalSensor will collect 2
	 * seconds worth of data (20 times 100ms), process it, and only then call
	 * the handler with the result.
	 *
	 * @param periodInMicroSeconds The time in microsenconds that should ellapse
	 * between each of the samples internally acquired by this sensor.
	 *
	 * @param totalNumberOfSamples The number of samples after which to compute
	 * the final result. 
	 */
	public abstract void setSensingInterval(long periodInMicroSeconds, int totalNumberOfSamples);

}
