package fr.inria.arles.thinglib.devices.android;

import android.os.Parcel;
import android.os.Parcelable;
import fr.inria.arles.thinglib.devices.Sensor;
import fr.inria.arles.thinglib.platforms.OuterScopeWrapper;

/**
 * This class contains metadata about the acceleration sensor.
 */
public class AccelerometerSensorInfo extends InertialSensorInfo {

	private static final long serialVersionUID = 5097675881287875861L;

	protected static AccelerometerSensor sSingleton;

	protected final String UID         = "fr.inria.arles.sensor.accelerometer";
	protected final String INTERFACE   = "fr.inria.arles.sensor.accelerometer";

	protected final String LONG_NAME    = "Acceleration sensor";
	protected final String SHORT_NAME   = "Accelerometer";
	protected final String DESCRIPTION  =
	 	"Measures the acceleration of this device as it is" +
	 	"moved, and is also able to tell which side is up and which side" +
	 	"is down (based on the acceleration due to gravity).";

	protected final String AUTHOR  = "INRIA Arles";
	protected final String DOC_URL = null;

	@Override
	public Sensor getSensorInstance(OuterScopeWrapper scope) {
		if (sSingleton == null) {
			sSingleton = new AccelerometerSensor(scope, this);
		}
		return sSingleton;
	}

	@Override public String getUid()              { return UID;         } 
	@Override public String getInterface()        { return INTERFACE;   } 
	@Override public String getLongName()         { return LONG_NAME;   } 
	@Override public String getShortName()        { return SHORT_NAME;  } 
	@Override public String getDescription()      { return DESCRIPTION; } 
	@Override public String getAuthor()           { return AUTHOR;      } 
	@Override public String getDocumentationUrl() { return DOC_URL;     } 
	@Override public boolean isEventBased()       { return false;       }

	public AccelerometerSensorInfo() {}


	/* Android parcelable-related methods */

	public static final Parcelable.Creator<AccelerometerSensorInfo> CREATOR = new Parcelable.Creator<AccelerometerSensorInfo>() {

		public AccelerometerSensorInfo createFromParcel(Parcel in) {
			return new AccelerometerSensorInfo(in);
		}

		public AccelerometerSensorInfo[] newArray(int size) {
			return new AccelerometerSensorInfo[size];
		}
	};

	private AccelerometerSensorInfo(Parcel in) {
		super(in);
	}
}

