package fr.inria.arles.thinglib.devices.android;

import android.os.Parcel;
import android.os.Parcelable;
import fr.inria.arles.thinglib.devices.Sensor;
import fr.inria.arles.thinglib.platforms.OuterScopeWrapper;

/**
 * This class contains metadata about the fall-detection sensor.
 */
public class FallSensorInfo extends ParcelableSensorInfo {

	private static final long serialVersionUID = 1L;

	protected static OrientationSensor sSingleton;

	protected final String UID         = "fr.inria.arles.sensor.simpleFallDetector";
	protected final String INTERFACE   = "fr.inria.arles.sensor.fallDetector";

	protected final String LONG_NAME    = "Fall-Detection sensor";
	protected final String SHORT_NAME   = "Fall-Detector";
	protected final String DESCRIPTION  =
	 	"Detects a fall by looking a quick changes in the phone's orientation." +
	 	"This is an event-based sensor.";

	protected final String AUTHOR  = "INRIA Arles";
	protected final String DOC_URL = null;


	/**
	 * The preferred sensing period for this sensor.
	 */
	public static final long preferredSensingPeriod  = 100;

	@Override
	public Sensor getSensorInstance(OuterScopeWrapper scope) {
		if (sSingleton == null) sSingleton = new OrientationSensor(scope, this);
		return sSingleton;
	}

	@Override public String getUid()              { return UID;         } 
	@Override public String getInterface()        { return INTERFACE;   } 
	@Override public String getLongName()         { return LONG_NAME;   } 
	@Override public String getShortName()        { return SHORT_NAME;  } 
	@Override public String getDescription()      { return DESCRIPTION; } 
	@Override public String getAuthor()           { return AUTHOR;      } 
	@Override public String getDocumentationUrl() { return DOC_URL;     } 
	@Override public boolean isEventBased()       { return true;       }

	public FallSensorInfo() {}


	/* Android parcelable-related methods */

	public static final Parcelable.Creator<FallSensorInfo> CREATOR = new Parcelable.Creator<FallSensorInfo>() {

		public FallSensorInfo createFromParcel(Parcel in) {
			return new FallSensorInfo(in);
		}

		public FallSensorInfo[] newArray(int size) {
			return new FallSensorInfo[size];
		}
	};

	@Override
	public int describeContents() {
		return 0;
	}

	@Override public void writeToParcel(Parcel out, int flags) {}
	private FallSensorInfo(Parcel in) {}
}

