package fr.inria.arles.thinglib.devices.android;

import java.util.Map;

import android.content.Context;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.text.format.Time;
import android.util.Log;

import fr.inria.arles.thinglib.datatypes.DefaultDouble4dLocationData;
import fr.inria.arles.thinglib.devices.ParameterErrorException;
import fr.inria.arles.thinglib.devices.Sensor;
import fr.inria.arles.thinglib.devices.SensorInfo;
import fr.inria.arles.thinglib.devices.SensorListener;
import fr.inria.arles.thinglib.devices.SensorResponse;
import fr.inria.arles.thinglib.platforms.OuterScopeWrapper;

public class LocationSensor implements Sensor {

	private static final String TAG = "LOCATION_SENSOR";
	
	SensorListener mSensorListener;
	private final SensorInfo mSensorInfo;
	Context context;
	
	final long mFreshnessMargin = 3000;	/* milliseconds */
	boolean mIsRunning;
	LocationManager location_manager;
	
	SensorResponse mLatestSensorResponse;

	LocationListener location_listener = new LocationListener() {
		@Override
		public void onStatusChanged(String provider, int status, Bundle extras) {
			/**/
		}
		@Override
		public void onProviderEnabled(String provider) {
			/**/
		}
		@Override
		public void onProviderDisabled(String provider) {
			/**/
		}
		@Override
		public void onLocationChanged(Location location) {
			stopLocationUpdates();
			prepareDataAndCallHandler(location);
		}
	}; 
	
	public LocationSensor(OuterScopeWrapper scope, SensorInfo sensorInfo) {
		context = (Context) scope.getContents();
		location_manager = (LocationManager)context.getSystemService(Context.LOCATION_SERVICE);
		mSensorInfo = sensorInfo;
		mIsRunning = false;
	}
	
	@Override
	public SensorInfo getSensorInfo() {
		return mSensorInfo;
	}

	@Override
	public void setParameter(String key, Object value)
			throws ParameterErrorException {
		// TODO Auto-generated method stub
		
	}

	@Override
	public Object getParameter(String key) throws ParameterErrorException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Map<String, Object> getParameters() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public SensorResponse getData() {
		return mLatestSensorResponse;
	}

	@Override
	public boolean hasFreshData() {
		Time time = new Time();
		time.setToNow();
		// TODO: Make sure we're handling timezones correctly (toMillis argument)
		if (mLatestSensorResponse.payload.timestamp > 
			time.toMillis(false) - mFreshnessMargin) 
				return true;

		return false;
	}
	
	@Override
	public void getData(SensorListener handler) {
		mSensorListener = handler;
		triggerDataCollection();
	}

	@Override
	public void cancelCurrentTask() {
		if(mIsRunning) 
			stopLocationUpdates();
	}

	@Override
	public boolean isSensing() {
		// TODO Auto-generated method stub
		return false;
	}
	
	private void triggerDataCollection() {
		mIsRunning = true;
		
		Handler hn = new Handler(context.getMainLooper());		
		hn.post(new Runnable() {
			@Override
			public void run() {
				Log.e(TAG, "handler run");
				try {				
					location_manager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, 
						location_listener);
				} catch(Exception e) {
					Log.e(TAG, "Exception when registering location listener: "+e.getMessage());

				}
			}
		});
	}
	
	void prepareDataAndCallHandler(Location location) {
		SensorResponse response = new SensorResponse(
						mSensorInfo, new DefaultDouble4dLocationData(location.getLatitude(), 
								location.getLongitude(), location.getSpeed(), location.getBearing(), 
								location.getTime()));
		
		mLatestSensorResponse = response;
		mSensorListener.onNewResponse(response);
	}
	
	void stopLocationUpdates() {
		try {
			location_manager.removeUpdates(location_listener);
		} catch(Exception e) {
			Log.e(TAG, "Exception when unregistering location listener: "+e.getMessage());
		}
		mIsRunning = false;
	}
}
