package fr.inria.arles.thinglib.devices.android;

import android.os.Parcel;
import android.os.Parcelable;
import fr.inria.arles.thinglib.devices.Sensor;
import fr.inria.arles.thinglib.platforms.OuterScopeWrapper;

/**
 * This class contains metadata about the magnetic sensor.
 */
public class MagnetometerSensorInfo extends InertialSensorInfo {

	private static final long serialVersionUID = -6263625527996798176L;

	protected static MagnetometerSensor sSingleton;

	protected final String UID         = "fr.inria.arles.sensor.magnetometer";
	protected final String INTERFACE   = "fr.inria.arles.sensor.magnetometer";

	protected final String LONG_NAME    = "Magnetic field sensor";
	protected final String SHORT_NAME   = "Magnetometer";
	protected final String DESCRIPTION  =
	 	"Measures the magnetic field around this device." +
	 	"This depends on the device's orientation with respect to the Earth's" +
	 	"magnetic field, as well as on the presence of any nearby metals or magnets";

	protected final String AUTHOR  = "INRIA Arles";
	protected final String DOC_URL = null;

	@Override
	public Sensor getSensorInstance(OuterScopeWrapper scope) {
		if (sSingleton == null) {
			sSingleton = new MagnetometerSensor(scope, this);
		}
		return sSingleton;
	}

	@Override public String getUid()              { return UID;         } 
	@Override public String getInterface()        { return INTERFACE;   } 
	@Override public String getLongName()         { return LONG_NAME;   } 
	@Override public String getShortName()        { return SHORT_NAME;  } 
	@Override public String getDescription()      { return DESCRIPTION; } 
	@Override public String getAuthor()           { return AUTHOR;      } 
	@Override public String getDocumentationUrl() { return DOC_URL;     } 
	@Override public boolean isEventBased()       { return false;       }

	public MagnetometerSensorInfo() {}

	
	/* Android parcelable-related methods */

	public static final Parcelable.Creator<MagnetometerSensorInfo> CREATOR = new Parcelable.Creator<MagnetometerSensorInfo>() {

		public MagnetometerSensorInfo createFromParcel(Parcel in) {
			return new MagnetometerSensorInfo(in);
		}

		public MagnetometerSensorInfo[] newArray(int size) {
			return new MagnetometerSensorInfo[size];
		}
	};

	private MagnetometerSensorInfo(Parcel in) {
		super(in);
	}
}

