package fr.inria.arles.thinglib.devices.android;

import android.os.Parcel;
import android.os.Parcelable;
import fr.inria.arles.thinglib.devices.Sensor;
import fr.inria.arles.thinglib.platforms.OuterScopeWrapper;

/**
 * This class contains metadata about the orientation sensor.
 */
public class OrientationSensorInfo extends InertialSensorInfo {

	private static final long serialVersionUID = -5742469577807096931L;
	
	protected static OrientationSensor sSingleton;

	protected final String UID         = "fr.inria.arles.sensor.orientation";
	protected final String INTERFACE   = "fr.inria.arles.sensor.orientation";

	protected final String LONG_NAME    = "Orientation sensor";
	protected final String SHORT_NAME   = "Orientation";
	protected final String DESCRIPTION  =
	 	"Measures the device's orientation by using the magnetometer" +
	 	"and accelerometer together. This tells you how the device is tilted with" +
	 	"respect to gravity and how it is oriented with respect to the geographic" +
	 	"North.";

	protected final String AUTHOR  = "INRIA Arles";
	protected final String DOC_URL = null;

	@Override
	public Sensor getSensorInstance(OuterScopeWrapper scope) {
		if (sSingleton == null) {
			sSingleton = new OrientationSensor(scope, this);
		}
		return sSingleton;
	}

	@Override public String getUid()              { return UID;         } 
	@Override public String getInterface()        { return INTERFACE;   } 
	@Override public String getLongName()         { return LONG_NAME;   } 
	@Override public String getShortName()        { return SHORT_NAME;  } 
	@Override public String getDescription()      { return DESCRIPTION; } 
	@Override public String getAuthor()           { return AUTHOR;      } 
	@Override public String getDocumentationUrl() { return DOC_URL;     } 
	@Override public boolean isEventBased()       { return false;       }

	public OrientationSensorInfo() {}


	/* Android parcelable-related methods */

	public static final Parcelable.Creator<OrientationSensorInfo> CREATOR = new Parcelable.Creator<OrientationSensorInfo>() {

		public OrientationSensorInfo createFromParcel(Parcel in) {
			return new OrientationSensorInfo(in);
		}

		public OrientationSensorInfo[] newArray(int size) {
			return new OrientationSensorInfo[size];
		}
	};

	private OrientationSensorInfo(Parcel in) {
		super(in);
	}
}

