package fr.inria.arles.thinglib.devices.android;

import android.os.Parcel;
import android.os.Parcelable;
import fr.inria.arles.thinglib.devices.Sensor;
import fr.inria.arles.thinglib.platforms.OuterScopeWrapper;

/**
 * This class contains metadata about the orientation sensor.
 */
public class SoundSensorInfo extends ParcelableSensorInfo {

	private static final long serialVersionUID = 7153832198529967859L;
	
	protected final String UID         = "fr.inria.arles.sensor.soundAmplitude";
	protected final String INTERFACE   = "fr.inria.arles.sensor.soundAmplitude";

	protected final String LONG_NAME   = "Ambient audio volume sensor";
	protected final String SHORT_NAME  = "Loudness sensor";
	protected final String DESCRIPTION =
	 	"Measures the volume of the ambient audio (i.e. the \"loudness\" of the " +
	 	"environment).";

	protected final String AUTHOR = "INRIA Arles";
	protected final String DOC_URL = null;

	//public static final long preferredSensingPeriod  = 125; // in microseconds (freq = 8000 samples/sec)
	//public static final int preferredNumberOfSamples = 256;

	protected static SoundSensor sSingleton;

	@Override
	public Sensor getSensorInstance(OuterScopeWrapper scope) {
		if (sSingleton == null) sSingleton = new SoundSensor(scope, this);
		return sSingleton;
	}

	@Override public String getUid()              { return UID;         } 
	@Override public String getInterface()        { return INTERFACE;   } 
	@Override public String getLongName()         { return LONG_NAME;   } 
	@Override public String getShortName()        { return SHORT_NAME;  } 
	@Override public String getDescription()      { return DESCRIPTION; } 
	@Override public String getAuthor()           { return AUTHOR;      } 
	@Override public String getDocumentationUrl() { return DOC_URL;     } 
	@Override public boolean isEventBased()       { return false;       }

	protected SoundSensorInfo() {}


	/* Android parcelable-related methods */

	@Override
	public int describeContents() {
		return 0;
	}

	@Override public void writeToParcel(Parcel out, int flags) {}

	public static final Parcelable.Creator<SoundSensorInfo> CREATOR = new Parcelable.Creator<SoundSensorInfo>() {

		public SoundSensorInfo createFromParcel(Parcel in) {
			return new SoundSensorInfo(in);
		}

		public SoundSensorInfo[] newArray(int size) {
			return new SoundSensorInfo[size];
		}
	};

	private SoundSensorInfo(Parcel in) {}
}


