package fr.inria.arles.thinglib.platforms.android;

import android.app.Activity;
import android.app.Service;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.DeadObjectException;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;

import fr.inria.arles.thinglib.core.BadReplyException;
import fr.inria.arles.thinglib.core.DaemonConnectionException;
import fr.inria.arles.thinglib.core.MediatorListener;
import fr.inria.arles.thinglib.core.Messaging;

public class Mediator extends fr.inria.arles.thinglib.core.Mediator {

	protected Activity mActivity;
	protected Service mService;
	
	//protected final Intent mDaemonIntent;

	protected IBinder mDaemonBinder;
	protected DaemonAidlInterface mDaemonService;
	protected ServiceConnection mDaemonConnection;

	public Mediator(Activity activity) {
		this(activity, null);
	}

	public Mediator(Activity activity, MediatorListener listener) {
		super(listener);
		mActivity = activity;
		//mDaemonIntent = new Intent(mActivity, DaemonService.class);
		mService = null;
	}

	public Mediator(Service service, MediatorListener listener) {
		super(listener);
		mService = service;
		//mDaemonIntent = new Intent(mActivity, DaemonService.class);
		mActivity = null;
	}
	
	@Override
	public void startDaemon() {
		//Log.e("", "startDaemon()");
		//  mActivity.startService(mDaemonIntent);
	}

	@Override
	public void stopDaemon() {
		// TODO: disconnect first?
		// This doesn't really kill it if there are any other bound apps :-/
		  // mActivity.stopService(mDaemonIntent);
	}
	
	@Override
	public boolean isDaemonRunning() {
		return mDaemonBinder.pingBinder();
	}
	
	@Override
	public void connect() {
		
		if (mClientAddress < 0) {

			startDaemon(); 
			
			/* DEBUG: Place a breakpoint at this line if you want to debug the Daemon. When the 
			 * execution reaches here, you should swithc to the "Devices" view and mark the Daemon 
			 * process for debugging. This is requried because the Daemon currently runs in a 
			 * separate process, as it well should :)
			 */
			
			mDaemonConnection = new ServiceConnection() {

				@Override
				public void onServiceConnected(ComponentName className, IBinder binder) {

					mDaemonBinder = binder;
					mDaemonService = DaemonAidlInterface.Stub.asInterface(binder);

					try {
						mClientAddress = (int)(Integer) getObjFromDaemon(Messaging.MSG_REGISTER_CLIENT, null);
						
					} catch (BadReplyException e) {
						
						setConnectionState(Mediator.STATE_DISCONNECTED);
						e.printStackTrace();
						return;
						
					} catch (DaemonConnectionException e) {
						
						setConnectionState(Mediator.STATE_DISCONNECTED);
						e.printStackTrace();
						return;
					}

					try {
						mDaemonService.registerCallback(mClientAddress, mClientCallback);
						
					} catch (RemoteException e) {
						
						if (mClientAddress != Messaging.UNREGISTERED_ADDRESS) disconnect();				
						setConnectionState(Mediator.STATE_DISCONNECTED);
						e.printStackTrace();
						return;
					}
					
					setConnectionState(Mediator.STATE_CONNECTED);
				}

				/**
				 * SOS , TODO, WARNING : This callback DOES NOT seem to be called
				 * when unbinding.
				 */
				@Override
				public void onServiceDisconnected(ComponentName className) {
					
					mClientAddress = Messaging.UNREGISTERED_ADDRESS;
					mDaemonService = null;
					setConnectionState(Mediator.STATE_DISCONNECTED);
				}
			};
			
			if(mActivity != null) {
				mActivity.bindService(new Intent(mActivity, DaemonService.class), 
						mDaemonConnection, 
						( Context.BIND_AUTO_CREATE | Context.BIND_DEBUG_UNBIND ));
			}
			else {
				mService.bindService(new Intent(mService, DaemonService.class), 
						mDaemonConnection, 
						( Context.BIND_AUTO_CREATE | Context.BIND_DEBUG_UNBIND ));
			}
				
			
			// see mDaemonConnection's onServiceConnected()
		}
	}
	
	@Override
	public synchronized void disconnect() {
		
		if (mClientAddress >= 0) {
			sendMessageToDaemon(Messaging.MSG_DEREGISTER_CLIENT, null);
			Log.d("Mediator", "Unbinding service");
			if(mActivity != null)
				mActivity.unbindService(mDaemonConnection);
			else
				mService.unbindService(mDaemonConnection);
			
			mClientAddress = Messaging.UNREGISTERED_ADDRESS;
			mDaemonConnection = null;
			setConnectionState(STATE_DISCONNECTED);
			// see mDaemonConnection's onServiceDisconnected()
		}
	}

	@Override
	protected void sendMessageToDaemon(int messageType, byte[] messagePayload) {

		try {
			mDaemonService.handleMessageFromClient(mClientAddress, messageType, messagePayload);
		} catch (DeadObjectException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	@Override
	protected byte[] sendRpcToDaemon(int messageType, byte[] messagePayload) 
	throws BadReplyException {

		try {
			return mDaemonService.handleRpcFromClient(mClientAddress, messageType, messagePayload);
		} catch (DeadObjectException e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}

		throw new BadReplyException("Dead object.");
	}

	private ClientAidlInterface mClientCallback = new ClientAidlInterface.Stub() {

		@Override
		public void handleMessageFromDaemon(int messageType, byte[] messagePayload) {

			processMessage(messageType, messagePayload);
		}
	};

}


