package fr.inria.arles.thinglib.websockets;

import java.net.URI;
import java.net.URISyntaxException;

import org.json.JSONException;
import org.json.JSONObject;
import org.java_websocket.client.WebSocketClient;
import org.java_websocket.handshake.ServerHandshake;

import android.os.Handler;
import android.util.Log;

import fr.inria.arles.thinglib.RESTAccess.MainActivity;
import fr.inria.arles.thinglib.RESTAccess.MediatorService;
import fr.inria.arles.thinglib.RESTAccess.Static_Addresses;

public class WebSocket_Client {

	private WebSocketClient WebSocket_Conn;
	private URI Server_Address;
	private String TAG = "WebSocket_Client";
	private boolean _isStreaming = false;
	private WebSocket_Client instance = null;

	Handler RegisterDeviceHandler = new Handler();
	Runnable RegisterDeviceSched = new Runnable() {

		@Override
		public void run() {
			InitConnection();
		}
	};

	Handler StopSensingHandler = new Handler();
	Runnable StopSensingSched = new Runnable() {

		@Override
		public void run() {
			/*
			 * If there are no subscribers stop the periodic sensing
			 */
			MediatorService.getController().stopSensing();
		}
	};

	public WebSocket_Client() {
		try {
			Server_Address = new URI(Static_Addresses.PROXY_WEBSOCKET_URL);
			instance = this;
		} catch (URISyntaxException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		WebSocket_Conn = new WebSocketClient(Server_Address) {

			@Override
			public void onMessage(String message) {
				int rate = 3000;

				JSONObject reqObj;
				try {
					reqObj = new JSONObject(message);

					if (reqObj.has("sensorDataRequest")) {

						JSONObject request = reqObj
								.getJSONObject("sensorDataRequest");
						// rate = Integer.valueOf(request.getString("rate"));

						MediatorService.getController().startSensing(rate);
						Log.i(TAG, "Mediator is sensing");

						_isStreaming = true;

					} else if (reqObj.has("proxyStopSensing")) {
						StopSensingHandler.postDelayed(StopSensingSched, 2000);
						Log.i(TAG, "Mediator stopped sensing");

						_isStreaming = false;
					}

				} catch (JSONException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

			@Override
			public void onOpen(ServerHandshake handshake) {
				Log.i(TAG, "WS Client open");
			}

			@Override
			public void onClose(int code, String reason, boolean remote) {
				if (MediatorService.getController().IsSensing()) {
					MediatorService.getController().stopSensing();
					Log.i(TAG, "Mediator is sensing");
				}
			}

			@Override
			public void onError(Exception ex) {
				Log.i(TAG, "WebSocket client error");
			}
		};

		WebSocket_Conn.connect();
		Log.i(TAG, "Connection initialized");

		RegisterDeviceHandler.postDelayed(RegisterDeviceSched, 3000);
	}

	public void SendMessageToServer(JSONObject Message) {
		if ((WebSocket_Conn.getConnection().isOpen())) {
			WebSocket_Conn.send(Message.toString());
			Log.i(TAG, "Sending data to WS: " + Message);
		}
	}

	public void InitConnection() {
		if (WebSocket_Conn.getConnection().isOpen()) {
			WebSocket_Conn.send("{\"registerDevice\":{\"deviceID\":\""
					+ MainActivity.UNIQUE_ID + "\"}}");
			Log.i(TAG, "Register device to Proxy WS server");
		}
	}

	public void CloseConnection() {
		if (WebSocket_Conn.getConnection().isOpen()) {
			WebSocket_Conn.send("{\"unRegisterDevice\":{\"deviceID\":\""
					+ MainActivity.UNIQUE_ID + "\"}}");
			Log.i(TAG, "UnRegister device to Proxy WS server");

			WebSocket_Conn.close();
			Log.i(TAG, "Connection closed");
		}
	}

	/**
	 * @return True if the WebSocket client is streaming data to the Proxy
	 */
	public boolean Is_Streaming() {
		return _isStreaming;
	}

}
