package org.ow2.choreos.choreosiotsmanager.impl;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;

import org.ow2.choreos.iots.common.Service;
import org.ow2.choreos.choreosiotsmanager.interfaces.ThingsRegistrationManager;
import org.ow2.choreos.registration.impl.RegistrationEstimatorImpl;
import org.ow2.choreos.iots.common.Location;
public class ThingsRegistrationManagerImpl implements
		ThingsRegistrationManager {
	
	@Override
	/**
	 * This method is called by the services hosted on a device and abstracting
	 * sensors to decide, non-deterministically if the device should register
	 * this service or not.
	 * 
	 * @param deviceId
	 *            : the unique id of the hosting device
	 * @param serviceName
	 *            : the name of the service
	 * @param serviceType
	 *            : the type of the service
	 * @param serviceAccuracy
	 *            : the accuracy of the measurement provided by the service
	 * @param sensorType
	 *            : the type of the sensor abstracted by the service
	 * @param physicalConcept
	 *            : the physical concept measured by the service
	 * @param range
	 *            : the range of the service's measurements
	 * @param threshold
	 *            : the probability threshold, defined by the developer who
	 *            created the service, used by the middleware to perform the
	 *            non-deterministic computations
	 * @param Address
	 *            : the unique address of the service
	 * @return true if the host device registered the service and false
	 *         otherwise.
	 * @throws IOException
	 */
	public boolean executeRegistrationQuery(String deviceId, String serviceName,
			String serviceType, double serviceAccuracy, String sensorType,
			String physicalConcept,  double range,
			double threshold, String serviceAddress, String dataType, double locationCoordinateX , double locationCoordinateY, ArrayList<Location> path,String unit) throws IOException {
		if (serviceName.equals("")){
			System.out.println("ThingsRegistrationManagerImpl.executeRegistrationQuery: serviceName is empty");
			return false;
		}
		if (serviceType.equals("")){
			System.out.println("ThingsRegistrationManagerImpl.executeRegistrationQuery: serviceType is empty");
			return false;
		}
		if (sensorType.equals("")){
			System.out.println("ThingsRegistrationManagerImpl.executeRegistrationQuery: sensorType is empty");
			return false;
		}
		if (physicalConcept.equals("")){
			System.out.println("ThingsRegistrationManagerImpl.executeRegistrationQuery: physicalConcept is empty");
			return false;
		}
		if (serviceAddress.equals("")){
			System.out.println("ThingsRegistrationManagerImpl.executeRegistrationQuery: serviceAddress is empty");
			return false;
		}
		RegistrationEstimatorImpl myRegEst = new RegistrationEstimatorImpl();
		Service phoneService = new Service(deviceId, serviceName, serviceType,
				serviceAccuracy, sensorType, physicalConcept, 
				range, threshold, serviceAddress, dataType, locationCoordinateX , locationCoordinateY, unit);
		phoneService.setEstimatedPath(path);
		boolean decision = myRegEst.registerMe(phoneService);
		return decision;
	}
	
	/**
	 * This method is called to unregister a service
	 * 
	 * @param deviceId
	 *            : the unique id of the hosting device
	 * @param serviceName
	 *            : the name of the service
	 * @param serviceType
	 *            : the type of the service
	 * @param serviceAccuracy
	 *            : the accuracy of the measurement provided by the service
	 * @param sensorType
	 *            : the type of the sensor abstracted by the service
	 * @param physicalConcept
	 *            : the physical concept measured by the service
	 * @param range
	 *            : the range of the service's measurements
	 * @param threshold
	 *            : the probability threshold, defined by the developer who
	 *            created the service, used by the middleware to perform the
	 *            non-deterministic computations
	 * @param Address
	 *            : the unique address of the service
	 * @return true if the host device registered the service and false
	 *         otherwise.
	 * @throws IOException
	 */
	public boolean executeUnRegistrationQuery(String deviceId, String serviceName,
			String serviceType, double serviceAccuracy, String sensorType,
			String physicalConcept,  double range,
			double threshold, String serviceAddress, String dataType, double locationCoordinateX , double locationCoordinateY, String unit) throws IOException {
		if (serviceName.equals("")){
			System.out.println("ThingsRegistrationManagerImpl.executeRegistrationQuery: serviceName is empty");
			return false;
		}
		if (serviceType.equals("")){
			System.out.println("ThingsRegistrationManagerImpl.executeRegistrationQuery: serviceType is empty");
			return false;
		}
		if (sensorType.equals("")){
			System.out.println("ThingsRegistrationManagerImpl.executeRegistrationQuery: sensorType is empty");
			return false;
		}
		if (physicalConcept.equals("")){
			System.out.println("ThingsRegistrationManagerImpl.executeRegistrationQuery: physicalConcept is empty");
			return false;
		}
		if (serviceAddress.equals("")){
			System.out.println("ThingsRegistrationManagerImpl.executeRegistrationQuery: serviceAddress is empty");
			return false;
		}
		RegistrationEstimatorImpl myRegEst = new RegistrationEstimatorImpl();
		Service phoneService = new Service(deviceId, serviceName, serviceType,
				serviceAccuracy, sensorType, physicalConcept, 
				range, threshold, serviceAddress, dataType, locationCoordinateX , locationCoordinateY, unit);
		boolean decision = myRegEst.unRegisterMe(phoneService);
		return decision;
	}

}
