package org.ow2.choreos.choreosiotsmanager.interfaces;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;

import org.ow2.choreos.iots.common.Location;

public interface ThingsRegistrationManager {
	/**
	 * This method is called by the services hosted on a device and abstracting
	 * sensors to decide, non-deterministically if the device should register
	 * this service or not.
	 * 
	 * @param deviceId
	 *            : the unique id of the hosting device
	 * @param serviceName
	 *            : the name of the service
	 * @param serviceType
	 *            : the type of the service
	 * @param serviceAccuracy
	 *            : the accuracy of the measurement provided by the service
	 * @param sensorType
	 *            : the type of the sensor abstracted by the service
	 * @param physicalConcept
	 *            : the physical concept measured by the service
	 * @param range
	 *            : the range of the service's measurements
	 * @param threshold
	 *            : the probability threshold, defined by the developer who
	 *            created the service, used by the middleware to perform the
	 *            non-deterministic computations
	 * @param Address
	 *            : the unique address of the service
	 * @return true if the host device registered the service and false
	 *         otherwise.
	 * @throws IOException
	 */
	public boolean executeRegistrationQuery(String deviceId, String serviceName,
			String serviceType, double serviceAccuracy, String sensorType,
			String physicalConcept, double range, double threshold,
			String Address, String dataType, double locationCoordinateX, double locationCoordinateY, ArrayList<Location> path, String unit) throws IOException;

}
