package org.ow2.choreos.iots.registry;

import org.json.simple.JSONObject;

public class ServiceDescription
{
  private static final String ACCURACY = "accuracy";
  private static final String ADDRESS = "address";
  private static final String NAME = "name";
  private static final String DEVICEID = "DeviceId";
  private static final String TYPE = "type";
  private static final String X = "coordinateX";
  private static final String Y = "coordinateY";
  private static final String GLOBALREGION = "global_region";
  private static final String RANGE = "range";
  private static final String SENSOR = "sensor";
  private static final String CONCEPT = "concept";
  private static final String DATATYPE = "dataType";
  String deviceID;
  String address;
  String name;
  String type;
  Double accuracy;
  double locationX;
  double locationY;
  String globalRegion;
  Double range;
  String sensor;
  String concept;
  String dataType;

  private ServiceDescription()
  {
  }

  public ServiceDescription(String deviceID, String address, String name, String type, double accuracy, String globalRegion, double range, String sensor, String concept, String dataType, double locationCoordinateX, double locationCoordinateY)
  {
    this.deviceID = deviceID;
    this.address = address;
    this.name = name;
    this.type = type;
    this.accuracy = Double.valueOf(accuracy);
    this.locationX = locationCoordinateX;
    this.locationY = locationCoordinateY;
    this.globalRegion = globalRegion;
    this.range = Double.valueOf(range);
    this.sensor = sensor;
    this.concept = concept;
    this.dataType = dataType;
  }

  public String getDeviceID()
  {
    return this.deviceID;
  }

  public String getAddress() {
    return this.address;
  }

  public String getName() {
    return this.name;
  }

  public String getType() {
    return this.type;
  }

  public double getAccuracy() {
    return this.accuracy.doubleValue();
  }

  public double getLocationX() {
    return this.locationX;
  }

  public double getLocationY() {
    return this.locationY;
  }

  public String getGlobalRegion() {
    return this.globalRegion;
  }

  public double getRange() {
    return this.range.doubleValue();
  }

  public String getSensor() {
    return this.sensor;
  }

  public String getConcept() {
    return this.concept;
  }

  public String getDataType() {
    return this.dataType;
  }

  public JSONObject serializeAsJSON() {
    JSONObject ret = new JSONObject();

    ret.put("accuracy", this.accuracy);
    ret.put("address", this.address);
    ret.put("name", this.name);
    ret.put("concept", this.concept);
    ret.put("DeviceId", this.deviceID);
    ret.put("global_region", this.globalRegion);
    ret.put("coordinateX", Double.valueOf(this.locationX));
    ret.put("coordinateY", Double.valueOf(this.locationY));
    ret.put("range", this.range);
    ret.put("sensor", this.sensor);
    ret.put("type", this.type);
    ret.put("dataType", this.dataType);
    return ret;
  }

  public static ServiceDescription deserializeFromJSONObject(JSONObject j)
  {
    ServiceDescription temp = new ServiceDescription();

    temp.accuracy = Double.valueOf(Double.parseDouble(j.get("accuracy").toString()));
    temp.address = j.get("address").toString();
    temp.name = j.get("name").toString();
    temp.concept = j.get("concept").toString();
    temp.deviceID = j.get("DeviceId").toString();
    temp.globalRegion = j.get("global_region").toString();
    temp.locationX = Double.parseDouble(j.get("coordinateX").toString());
    temp.locationY = Double.parseDouble(j.get("coordinateY").toString());
    temp.range = Double.valueOf(Double.parseDouble(j.get("range").toString()));

    temp.sensor = j.get("sensor").toString();
    temp.type = j.get("type").toString();
    temp.dataType = j.get("dataType").toString();
    return temp;
  }
}