package org.ow2.choreos.registration.impl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.ow2.choreos.iots.common.Circle;
import org.ow2.choreos.iots.common.DensityPerType;
import org.ow2.choreos.iots.common.Sensor;
import org.ow2.choreos.iots.common.SubstituteSensorSet;
import org.ow2.choreos.distributions.Constants;
import org.ow2.choreos.distributions.CoverageProbability;
import org.ow2.choreos.registration.interfaces.ProbabilityEstimator;

public class ProbabilityEstimatorImpl implements ProbabilityEstimator {

	public double probCvgOneLocWithSub, probabilityAllLocsNoSub, finalProb,
			finalProbabilityWithSub;

	DensityPerType pdt, phoneSensorDist;
	// boolean setCoversLocation;
	double myThreshold;
	Sensor myPhoneSensor;
	ArrayList<DensityPerType> myPDistPerTypeList;
	boolean coverageWithSub = false;
	public double newThresholdNoSub;
	public double finalProbabilityWithNoSub = 1;
	double myNumberOfLocations;
	HashMap<String, Double> myNumberOfServices;
	double neighbors;

	public ProbabilityEstimatorImpl() {
	}

	public double getProbOfCoverageAtOneLocWithSub(
			Map<Integer, SubstituteSensorSet> setsOfSubSensors, double area) {
		double probabilityAllSets = 1;
		double probabilityForOneType = 1;
		// for each set
		for (Entry<Integer, SubstituteSensorSet> entry : setsOfSubSensors
				.entrySet()) {
			// for (int i = 0; i< entry.getValue().getAllSensorTypes().size(); i
			// ++){
			// if
			// (entry.getValue().getAllSensorTypes().get(i).equals(myPhoneSensor.getSensorType()))
			// break;
			// else{
			double probabilityPerSetTypes = 1;
			SubstituteSensorSet setOfSensors = entry.getValue();

			Iterator<String> itr = setOfSensors.getAllConcepts().iterator();
			while (itr.hasNext()) {
				String concept = itr.next();
				if (concept.equals(myPhoneSensor.getSensorMeasuredConcept())) {
					probabilityPerSetTypes = 0;
					break;

				} else {
					pdt = getDistributionOfType(concept);
					probabilityForOneType = getProbabilityValuePerType(
							pdt.getDensity(), area);
					probabilityPerSetTypes = probabilityPerSetTypes
							* probabilityForOneType;
				}
			}
			probabilityAllSets = probabilityAllSets
					* (1 - probabilityPerSetTypes);

		}
		phoneSensorDist = getDistributionOfType(myPhoneSensor
				.getSensorMeasuredConcept());
		probCvgOneLocWithSub = 1
				- (1 - getProbabilityValuePerType(phoneSensorDist.getDensity(),
						area)) * probabilityAllSets;

		return probCvgOneLocWithSub;
	}

	public double getProbabilityValuePerType(double densityPerType, double area) {
		return (1 - Math.exp(-densityPerType * area));
	}

	public double getCoverageProbabilityAllLocs(DensityPerType pdist,
			double coverageArea) {
		probabilityAllLocsNoSub = Math.pow(
				getProbabilityValuePerType(pdist.getDensity(), coverageArea),
				myNumberOfLocations);

		return probabilityAllLocsNoSub;
	}

	public double getCoverageProbabilityAllLocs(
			Map<Integer, SubstituteSensorSet> setsOfSubSensors, double area) {
		double probabilityAllLocsWithSub;
		probabilityAllLocsWithSub = Math.pow(
				getProbOfCoverageAtOneLocWithSub(setsOfSubSensors, area),
				myNumberOfLocations);

		return probabilityAllLocsWithSub;
	}

	public double getFinalProbability(double coverageArea,
			Map<Integer, SubstituteSensorSet> setsOfSubSensors)
			throws IOException {
		// SubstituteSetsGeneratorImpl mySetGenerator = new
		// SubstituteSetsGeneratorImpl();
		// mySetGenerator.initialize();
		double prob;
		DensityPerType phoneTypeDist = getDistributionOfType(myPhoneSensor
				.getSensorMeasuredConcept());

		finalProbabilityWithNoSub = getCoverageProbabilityAllLocs(
				phoneTypeDist, coverageArea);

		newThresholdNoSub = generateNewThresholdNoSub(finalProbabilityWithNoSub);
		if (finalProbabilityWithNoSub < myThreshold) {
			coverageWithSub = true;

			finalProbabilityWithSub = getCoverageProbabilityAllLocs(
					setsOfSubSensors, coverageArea);
		} else
			coverageWithSub = false;
		if (coverageWithSub == false)
			prob = finalProbabilityWithNoSub;
		else
			prob = finalProbabilityWithSub;
		return prob;
	}

	public DensityPerType getDistributionOfType(String concept) {
		DensityPerType distribution = new DensityPerType("", 0);
		Iterator<DensityPerType> itr = myPDistPerTypeList.iterator();
		while (itr.hasNext()) {
			DensityPerType dist = itr.next();
			if (dist.getConcept().equals(concept)) {
				distribution = dist;
				break;
			} else
				distribution = new DensityPerType(concept, 0);
		}
		//  .out.println("the concept is: " + distribution.getConcept() +
		// " the density is:" + distribution.getDensity());
		return distribution;
	}

	@Override
	/**
	 * This method is called by the registration estimator to perform the mathematical calculations
	 * in order to return the a new value of the probability threshold below which it is considered
	 * that services are needed.
	 * @param pDistPerTypeList: the density for each type of service
	 * @param phoneSensor: the sensor abstracted by the service that is being evaluated
	 * @param numberOfLocations: the number of locations that should be covered by the service for a period of time
	 * after which the evaluation should take place again
	 * @param threshold: the initial threshold defined by the service developer
	 * @param setsOfSubSensors: the set of sensors that can substitute the initial sensor abstracted by the service
	 * being evaluated. if such a set exists, then the services abstracting its elements should also be evaluated
	 * @return a mathematical value representing the new threshold
	 * @throws IOException
	 */
	public double generateNewThresholdRandomWalk(
			ArrayList<DensityPerType> pDistPerTypeList, Sensor phoneSensor,
			double numberOfLocations, double threshold,
			Map<Integer, SubstituteSensorSet> setsOfSubSensors)
			throws IOException {

		myThreshold = threshold;
		myPDistPerTypeList = pDistPerTypeList;
		myPhoneSensor = phoneSensor;
		myNumberOfLocations = numberOfLocations;

		double newThresholdRW;

		double radius = phoneSensor.getSensorRange();
		Circle coverageCircle = new Circle(radius);
		double newArea = coverageCircle.getArea();
		double probabilityOfActualCoverage = getFinalProbability(newArea,
				setsOfSubSensors);

		newThresholdRW = 1 - ((1 - myThreshold) / (1 - probabilityOfActualCoverage));
		finalProb = probabilityOfActualCoverage;
		return newThresholdRW;

	}

	/*public double generateNewThresholdLevyWalk(
			HashMap<String, Double> NbrOfServiceInstances, Sensor phoneSensor,
			double threshold,
			Map<Integer, SubstituteSensorSet> setsOfSubSensors,
			ArrayList<Location> estimatedPath) throws IOException {
		double prob;
		double newThresholdLW = 0;
		ArrayList<Double> myXs = new ArrayList<Double>();
		ArrayList<Double> myYs = new ArrayList<Double>();
		for (int i = 0; i < estimatedPath.size(); i++) {
			myXs.add(estimatedPath.get(i).getXCoordinate());
			myYs.add(estimatedPath.get(i).getYCoordinate());
		}

		myThreshold = threshold;

		myPhoneSensor = phoneSensor;

		double newThresholdRW;

		double radius = phoneSensor.getSensorRange();
		Circle coverageCircle = new Circle(radius);
		double newArea = coverageCircle.getArea();
		double probabilityOfActualCoverage = 0;
		String commands = "";
		double nbrOfPhoneSnsrSrvsInst;
		if (NbrOfServiceInstances != null &&  NbrOfServiceInstances.isEmpty() == false && NbrOfServiceInstances.size() >  0 )
			{System.out.println(myPhoneSensor.getSensorType());
		nbrOfPhoneSnsrSrvsInst = NbrOfServiceInstances.get(myPhoneSensor
				.getSensorMeasuredConcept());}
		else
			nbrOfPhoneSnsrSrvsInst = 0;
		int size = estimatedPath.size();
		commands = commands + size + " ";
		for (int i = 0; i < myXs.size(); i++) {
			commands = commands + myXs.get(i) + " ";
		}
		for (int j = 0; j < myXs.size(); j++) {
			commands = commands + myYs.get(j) + " ";
		}

		commands = commands + Constants.DIFFUSIONCONSTANT + " ";
		commands = commands + Constants.VELOCITY+ " ";
		commands = commands + Constants.SENSINGRANGE + " ";
		commands = commands + nbrOfPhoneSnsrSrvsInst;
		// TODO: here the registry sends this information and the fact that
		// it sent this should determine the number
		// of args after, so it is sent as an object that has a known number
		// of parameters;
		commands = commands + " " + Constants.BINORMALDISTRIBUTION;
		commands = commands + " 0" + " 0.2";

		finalProbabilityWithNoSub = CoverageProbability.getValue(commands);
		newThresholdNoSub = generateNewThresholdNoSub(finalProbabilityWithNoSub);
		if (finalProbabilityWithNoSub < myThreshold) {
			coverageWithSub = true;
			// call mathematica for each substitute type in the set ;
			finalProbabilityWithSub = getCoverageProbabilityWithSubLW(NbrOfServiceInstances, phoneSensor,
					setsOfSubSensors , estimatedPath);
		} 
		
		else
			coverageWithSub = false;
		if (coverageWithSub == false)
			prob = finalProbabilityWithNoSub;
		else
			prob = 1
					- (1 - finalProbabilityWithNoSub) * finalProbabilityWithSub;

		newThresholdLW = 1 - ((1 - myThreshold) / (1 - prob));
		finalProb = prob;

		return newThresholdLW;

	}*/

	public double generateNewThresholdNoSub(double probVal) {
		double newThreshold;
		newThreshold = 1 - ((1 - myThreshold) / (1 - probVal));
		return newThreshold;

	}
	
	/*public double getCoverageProbabilityWithSubLW(HashMap<String, Double> NbrOfServiceInstances, Sensor phoneSensor,
			
			Map<Integer, SubstituteSensorSet> setsOfSubSensors,
			ArrayList<Location> estimatedPath) {
		double newThresholdLW = 0;
	
		double probabilityAllSets = 1;
		double probabilityForOneType = 1;
		// for each set
		for (Entry<Integer, SubstituteSensorSet> entry : setsOfSubSensors
				.entrySet()) {
			// for (int i = 0; i< entry.getValue().getAllSensorTypes().size(); i
			// ++){
			// if
			// (entry.getValue().getAllSensorTypes().get(i).equals(myPhoneSensor.getSensorType()))
			// break;
			// else{
			double probabilityPerSetTypes = 1;
			SubstituteSensorSet setOfSensors = entry.getValue();

			Iterator<String> itr = setOfSensors.getAllConcepts().iterator();
			while (itr.hasNext()) {
				String concept = itr.next();
				if (concept.equals(myPhoneSensor.getSensorMeasuredConcept())) {
					probabilityPerSetTypes = 0;
					break;

				} else {
					//TODO ask server for distribution and number of devices in the max area covered by the device
						double radius = phoneSensor.getSensorRange();
					Circle coverageCircle = new Circle(radius);
					double newArea = coverageCircle.getArea();
					double probabilityOfActualCoverage = 0;
					String commands = "";
					double nbrOfPhoneSnsrSrvsInst;
					nbrOfPhoneSnsrSrvsInst = NbrOfServiceInstances.get(concept);
					commands = commands + estimatedPath.size();
		

					commands = commands + Constants.DIFFUSIONCONSTANT + " ";
					commands = commands + Constants.VELOCITY;
					commands = commands + Constants.SENSINGRANGE;
					commands = commands + nbrOfPhoneSnsrSrvsInst;
					// TODO: here the registry sends this information and the fact that
					// it sent this should determine the number
					// of args after, so it is sent as an object that has a known number
					// of parameters;
					commands = commands + "'binormal'";
					commands = commands + "0" + "0.2";

					probabilityForOneType  = CoverageProbability.getValue(commands);
probabilityPerSetTypes = probabilityPerSetTypes
						* probabilityForOneType;
			}
		

		}
		probabilityAllSets = probabilityAllSets
					* (1 - probabilityPerSetTypes);
		}
		return probabilityAllSets;
	}*/

}
