package org.ow2.choreos.registration.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.ow2.choreos.iots.common.SubstituteSensorSet;
import org.ow2.choreos.registration.interfaces.SubstituteSetsGenerator;


import android.content.Context;
import android.content.res.AssetManager;
import android.util.Log;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import fr.inria.arles.thinglib.RESTAccess.MainActivity;


public class SubstituteSetsGeneratorImpl implements SubstituteSetsGenerator {
	
	private String TAG = "Model_Creator";
	
	protected static Model model = null;
	public static String Sensors_NAMESPACE 		= "http://www.semanticweb.org/ontologies/2011/7/S&A.owl#";
	public static String RDF_NAMESPACE 			= "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	public static String RDFS_NAMESPACE 		= "http://www.w3.org/2000/01/rdf-schema#";
	private static String _KnowledgeBaseFile 	= "sensorsAndActuators.rdf";
	private static String kbFileURL 			= "http://128.93.51.58:8081/Phone/sensorsAndActuators.rdf";

	static Map<Integer, SubstituteSensorSet> setsOfSubSensors = new HashMap<Integer, SubstituteSensorSet>();
	/**
	 * the method is used to read the ontology and parse it into memory
	 */
	public  void initialize() {
		
		if(model != null)
			return; 
		
		try {
			//model = ModelFactory.createDefaultModel().read(kbFileURL);
			//Log.i(TAG, "-- Model created --");
			
			Context _MainActivityContext = MainActivity.AppBaseContext;
			AssetManager _AssetsManager = _MainActivityContext.getAssets();
			InputStream _InputStream = null;
			_InputStream = _AssetsManager.open(_KnowledgeBaseFile);
			model = ModelFactory.createDefaultModel();
			model.read(_InputStream, null);
			Log.i(TAG, "-- Model created --" + model.size());
		} catch (Exception e) {
			System.out.println("SubstituteSetsGenerator.initialize: " + e.getMessage());
			return;
		}
	}
	/**
	 * This method is used to generate a set of sensors that, based on some mathematical computations
	 * over their measurements, can substitute a sensor of interest. This set is defined in an ontology
	 * @param initSensorType: the initial sensor type to be substituted
	 * @param initSensorConcept: the initial physical concept measured by the sensor
	 * @return a HashMap containing all substitute sets acquired from the ontology and their ids (the ids 
	 * are provided by the method itself)
	 */
	public Map<Integer, SubstituteSensorSet> getSubstitutionSets(String initSensorType,
			String initMeasuredConcept) {
		// TODO ONLY FORMULAS WITH DIFFERENT SENSORS NEXT
		String formula;
		int i = 0;
		// ArrayList<String> mySensorSet = new ArrayList<String>();
		String queryStringFormula = "SELECT  DISTINCT ?formula WHERE { "
				+ "<"
				+ Sensors_NAMESPACE
				+ initSensorType
				+ "> <http://www.semanticweb.org/ontologies/2011/7/S&A.owl#measures_concept> ?concept."
				+ "?formula <http://www.semanticweb.org/ontologies/2011/7/S&A.owl#computes_concept> ?concept.} ";

		Query query = QueryFactory.create(queryStringFormula);

		// Execute the query and obtain results
		QueryExecution qe = QueryExecutionFactory.create(query, model);
		ResultSet results = qe.execSelect();
		ArrayList<QuerySolution> resultList = (ArrayList<QuerySolution>) ResultSetFormatter
				.toList(results);

		Iterator<QuerySolution> itr = resultList.iterator();
		while (itr.hasNext()) {
			HashMap<String, String> setOfSensorsAndConcepts = new HashMap<String, String>();

			String answer = itr.next().toString();
			
			answer = answer.substring(answer.indexOf("<") - 1,
					answer.indexOf(">") + 1);
			formula = answer;
			String queryString = "SELECT  DISTINCT ?sensors ?pc WHERE { "
					+ formula
					+ " <http://www.semanticweb.org/ontologies/2011/7/S&A.owl#has_input> ?param."
					+ "?param <http://www.semanticweb.org/ontologies/2011/7/S&A.owl#physical_concept> ?pc."
					+ "?sensors <http://www.semanticweb.org/ontologies/2011/7/S&A.owl#measures_concept> ?pc.} ";
			Query sensorQuery = QueryFactory.create(queryString);
			QueryExecution qexec = QueryExecutionFactory.create(sensorQuery,
					model);
			ResultSet sensorsResult = qexec.execSelect();
			ArrayList<QuerySolution> sensorResultList = (ArrayList<QuerySolution>) ResultSetFormatter
					.toList(sensorsResult);
			Iterator<QuerySolution> iter = sensorResultList.iterator();

			while (iter.hasNext()) {
				String sensorsAnswer = "";
				sensorsAnswer = iter.next().toString();
			
				String[] subSensors = sensorsAnswer.split(" ");
				String subSensorConcept = subSensors[8];
				subSensorConcept = subSensorConcept.substring(
						subSensorConcept.indexOf("#") + 1,
						subSensorConcept.indexOf(">"));
				sensorsAnswer = sensorsAnswer.substring(
						sensorsAnswer.indexOf("#") + 1,
						sensorsAnswer.indexOf(">"));
				setOfSensorsAndConcepts.put(sensorsAnswer, subSensorConcept);

			}

			i = i + 1;
			
			SubstituteSensorSet mySet = new SubstituteSensorSet(setOfSensorsAndConcepts);
			setsOfSubSensors.put(i, mySet);
			System.out.println("my set contains : " + setsOfSubSensors.get(i).getAllConcepts());
			qexec.close();
		}

		HashMap<String, String> mySensorSet = new HashMap<String, String>();
		mySensorSet.put(initSensorType, initMeasuredConcept);
		setsOfSubSensors.put(i + 1, new SubstituteSensorSet(mySensorSet));
		System.out.println(setsOfSubSensors.get(i+1).getAllConcepts());
		qe.close();

		return setsOfSubSensors;

	}

}
