package org.ow2.choreos.registration.interfaces;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.ow2.choreos.iots.common.DensityPerType;
import org.ow2.choreos.iots.common.Sensor;
import org.ow2.choreos.iots.common.SubstituteSensorSet;




public interface ProbabilityEstimator {
/**
 * This method is called by the registration estimator to perform the mathematical calculations
 * in order to return the a new value of the probability threshold below which it is considered
 * that services are needed.
 * @param pDistPerTypeList: the density for each type of service
 * @param phoneSensor: the sensor abstracted by the service that is being evaluated
 * @param numberOfLocations: the number of locations that should be covered by the service for a period of time
 * after which the evaluation should take place again
 * @param threshold: the initial threshold defined by the service developer
 * @param setsOfSubSensors: the set of sensors that can substitute the initial sensor abstracted by the service
 * being evaluated. if such a set exists, then the services abstracting its elements should also be evaluated
 * @return a mathematical value representing the new threshold
 * @throws IOException
 */
	public double generateNewThresholdRandomWalk(
			ArrayList<DensityPerType> pDistPerTypeList, Sensor phoneSensor,
			double numberOfLocations, double threshold, Map<Integer, SubstituteSensorSet>setsOfSubSensors) throws IOException;

	/*public double generateNewThresholdLevyWalk(
			HashMap<String, Double> NbrOfServicesPerType, Sensor phoneSensor,
			 double threshold, Map<Integer, SubstituteSensorSet>setsOfSubSensors , ArrayList<Location> estimatedPath) throws IOException;*/


}
