package org.ow2.choreos.registryclient.interfaces;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;

import org.ow2.choreos.iots.common.Service;






public interface RegistryClient {
	/**
	 * This method is called by the registration manager to find the number of services measuring concepts of interest
	 * the number should be provided for each concept 
	 * @param myConcepts: the concepts of interest
	 * @param locationOfInterest: the location at which concepts of interest should be measured
	 * @param deviceId: the id of the device hosting the initial service. This is important
	 * because it should be excluded when returning the number of services measuring each concept.
	 * @return the HashMap containing the concept of interest and the number of registered services that
	 * measure this concept.
	 */
	public HashMap<String, Double> findNumberOfServicesReq(ArrayList<String> myConcepts,
			String locationOfInterest, String deviceId) ;
/**
 * The method is called to perform the actual registration of the service in a device registry.
 * if the registry is remote, it invokes a web service.
 * @param ps: the service to register
 * @return true if the registration was successful and false otherwise.
 */
	public boolean registerServiceReq(Service ps
		) ;

/**
 * This method is called to remove the service from the registry if needed. If the registry is remote,
 * this method invokes a web service.
 * @param phoneService: the service to remove. 
 */
	public void removeServiceReq(Service phoneService);

}
