package org.ow2.choreos.iots.linesweep;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.ow2.choreos.iots.Objects.Coordinate;
import org.ow2.choreos.iots.Objects.Rectangle;

public class LineSweep {

	/**
	 * This method sorts the coordinates by the x-value
	 * 
	 * @param coords
	 * @return
	 */
	public static List<Coordinate> sortY(List<Coordinate> coords) {

		Collections.sort(coords, new CustomComparatorY());
		return coords;
	}

	/**
	 * This method sorts the coordinates by the x-value
	 * 
	 * @param coords
	 * @return
	 */
	public static List<Coordinate> sort(List<Coordinate> coords) {

		Collections.sort(coords, new CustomComparator());
		return coords;
	}

	public static double sweep(List<Rectangle> recs) {
		double area = 0;
		List<Coordinate> coords = new ArrayList<Coordinate>();
		List<Coordinate> sortedCoords = new ArrayList<Coordinate>();
		List<Double> areas = new ArrayList<Double>();
		List<Double> intersections = new ArrayList<Double>();
		boolean foundL = false, foundLWithEndPoint = false;
		double l = 0;
		// check if any rectangles are covered by other, if so remove them from
		// the list
		for (int i = 0; i < recs.size(); i++) {
			for (int j = 0; j < recs.size(); j++) {
				if (i != j && recs.get(i).covers(recs.get(j))) {
					recs.remove(j);

				}
			}
		}
		// create a list of coordinates from the rectangles
		for (Rectangle r : recs) {
			coords.add(r.getBottomLeftCoord());
			coords.add(r.getBottomRightCoord());
		}
		sortedCoords = sort(coords);
		// System.out.println(sortedCoords.size());
		// compute area
		for (int i = 0; i < sortedCoords.size() - 1; i++) {
			if (i == 0) {

				l = coords.get(i).getRectangle().getyMax()
						- coords.get(i).getRectangle().getyMin();
				intersections.add(l);
				// System.out.println("l is " + l);

			} else {
				// to compute the length of the intersection between the swipe
				// line and the rectangles we need to check if the current
				// rectangle's x-coordinate is between the endpoints of another
				// rectangle, if so the length of both rectangles should be
				// considered
				foundL = false;
				foundLWithEndPoint = false;
				l = 0;
				double length = 0;
				ArrayList<Rectangle> tempRecs = new ArrayList<Rectangle>();
				for (int j = 0; j < i; j++) {
					if (coords.get(j).getRectangle().getxMax() > coords.get(i)
							.getX()
							&& coords.get(i).getX() != coords.get(i)
									.getRectangle().getxMax()) {
//						System.out.println("current coordinate"
//								+ coords.get(i).getX()
//								+ " and xMax coordinate of its rectangle is "
//								+ coords.get(i).getRectangle().getxMax());
						tempRecs.add(coords.get(j).getRectangle());

						foundL = true;
						tempRecs.add(coords.get(i).getRectangle());

					}
					// if the end point of the current rectangle is between the
					// endpoints of another rectangle r then only the length of
					// r should be considered
					else if (coords.get(j).getRectangle().getxMax() > coords
							.get(i).getX()
							&& coords.get(i).getX() == coords.get(i)
									.getRectangle().getxMax()) {
					//	length = length + intersections.get(j);
					//	length = length +coords.get(j).getRectangle().getyMax() - coords.get(j).getRectangle().getyMin();
						tempRecs.add(coords.get(j).getRectangle());
						foundLWithEndPoint = true;

						// System.out.println("l with intersection is " + l);
					}

				}
				if (foundL == true || foundLWithEndPoint == true) {

					l = computeIntersection(tempRecs);
					l = l + length;
					intersections.add(l);
					// System.out.println("l with intersection is " + l);
				}

				else if (foundL == false) {
					l = coords.get(i).getRectangle().getyMax()
							- coords.get(i).getRectangle().getyMin();
					intersections.add(l);
					// System.out.println("l without intersection is " + l);

				}
			}

			double thisArea = Math.abs(l
					* (coords.get(i + 1).getX() - coords.get(i).getX()));
			areas.add(thisArea);
			// System.out.println("area is " + thisArea);

		}
		for (double a : areas) {
			area = area + a;
		}
		return area;

	}

	/**
	 * The method takes a list of rectangles and finds intersecting segments
	 * (based on the y coordinates) and combines them as one then computes the
	 * final length of disjoint segments and returns this length
	 * 
	 * @param tempRecs
	 * @return
	 */
	private static double computeIntersection(ArrayList<Rectangle> tempRecs) {

		List<Coordinate> coords = new ArrayList<Coordinate>();
		List<Coordinate> sortedCoords = new ArrayList<Coordinate>();
		// create a list of coordinates from the rectangles
		for (Rectangle r : tempRecs) {
			coords.add(r.getBottomLeftCoord());
			coords.add(r.getUpperLeftCoord());
		}
		sortedCoords = sortY(coords);
//		for (Coordinate s : sortedCoords)
//			System.out.println(s.getY());
		int counterBottom = 0;
		int counterUp = 0;
		double length = 0;
		double minY = 0;
		double maxY = 0;
		for (Coordinate tempCoord : sortedCoords) {
			if (tempCoord.getRectangle().getBottomLeftCoord().getY() == tempCoord
					.getY()) {
				counterBottom++;
				if (counterBottom == 1) {
					minY = tempCoord.getY();
				}

			} else if (tempCoord.getRectangle().getUpperLeftCoord().getY() == tempCoord
					.getY()) {
				counterUp++;
				maxY = tempCoord.getY();
			}
			if (counterUp - counterBottom == 0) {
				length = length + maxY - minY;
				counterBottom = 0;
				counterUp = 0;
			}
		}
		return length;

	}
}
